@@ -18,6 +18,7 @@
 
 package org.apache.paimon.function;
 
+import org.apache.paimon.catalog.Identifier;
 import org.apache.paimon.types.DataField;
 
 import java.util.List;
@@ -31,6 +32,8 @@ public interface Function {
 
     String fullName();
 
+    Identifier identifier();
+
     Optional<List<DataField>> inputParams();
 
     Optional<List<DataField>> returnParams();
