@@ -145,11 +145,7 @@ public class ArrowUtils {
                             keyField.getChildren());
 
             Field valueField =
-                    toArrowField(
-                            MapVector.VALUE_NAME,
-                            fieldId,
-                            mapType.getValueType().notNull(),
-                            depth + 1);
+                    toArrowField(MapVector.VALUE_NAME, fieldId, mapType.getValueType(), depth + 1);
             FieldType valueType = valueField.getFieldType();
             valueField =
                     new Field(
@@ -212,8 +208,10 @@ public class ArrowUtils {
     }
 
     public static ArrowCStruct serializeToCStruct(
-            VectorSchemaRoot vsr, ArrowArray array, ArrowSchema schema) {
-        BufferAllocator bufferAllocator = vsr.getVector(0).getAllocator();
+            VectorSchemaRoot vsr,
+            ArrowArray array,
+            ArrowSchema schema,
+            BufferAllocator bufferAllocator) {
         Data.exportVectorSchemaRoot(bufferAllocator, vsr, null, array, schema);
         return ArrowCStruct.of(array, schema);
     }
