@@ -42,6 +42,7 @@ import org.junit.jupiter.params.provider.ValueSource;
 
 import java.math.BigDecimal;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Random;
@@ -118,6 +119,7 @@ public class ArrowFormatWriterTest {
                             .isEqualTo(fieldGetter.getFieldOrNull(expectec));
                 }
             }
+            vectorSchemaRoot.close();
         }
     }
 
@@ -160,6 +162,60 @@ public class ArrowFormatWriterTest {
                             .isEqualTo(fieldGetter.getFieldOrNull(expectec));
                 }
             }
+            vectorSchemaRoot.close();
+        }
+    }
+
+    @ParameterizedTest
+    @ValueSource(booleans = {false, true})
+    public void testWriteWithMemoryLimit(boolean limitMemory) {
+        RowType rowType =
+                new RowType(
+                        Arrays.asList(
+                                new DataField(0, "f0", DataTypes.BYTES()),
+                                new DataField(1, "f1", DataTypes.BYTES())));
+        Long memoryLimit = limitMemory ? 100 * 1024 * 1024L : null;
+        try (ArrowFormatWriter writer = new ArrowFormatWriter(rowType, 4096, true, memoryLimit)) {
+
+            GenericRow genericRow = new GenericRow(2);
+            genericRow.setField(0, randomBytes(1024 * 1024, 1024 * 1024));
+            genericRow.setField(1, randomBytes(1024 * 1024, 1024 * 1024));
+
+            // normal write
+            for (int i = 0; i < 200; i++) {
+                boolean success = writer.write(genericRow);
+                if (!success) {
+                    writer.flush();
+                    writer.reset();
+                    writer.write(genericRow);
+                }
+            }
+            writer.reset();
+
+            if (limitMemory) {
+                for (int i = 0; i < 64; i++) {
+                    Assertions.assertThat(writer.write(genericRow)).isTrue();
+                }
+                Assertions.assertThat(writer.write(genericRow)).isFalse();
+            }
+            writer.reset();
+
+            // Write batch records
+            for (int i = 0; i < 2000; i++) {
+                boolean success = writer.write(genericRow);
+                if (!success) {
+                    writer.flush();
+                    writer.reset();
+                    writer.write(genericRow);
+                }
+            }
+
+            if (limitMemory) {
+                Assertions.assertThat(writer.memoryUsed()).isLessThan(memoryLimit);
+                Assertions.assertThat(writer.getAllocator().getAllocatedMemory())
+                        .isGreaterThan(memoryLimit)
+                        .isLessThan(2 * memoryLimit);
+            }
         }
     }
 
@@ -192,6 +248,7 @@ public class ArrowFormatWriterTest {
                             .isEqualTo(fieldGetter.getFieldOrNull(expectec));
                 }
             }
+            vectorSchemaRoot.close();
         }
     }
 
@@ -219,6 +276,25 @@ public class ArrowFormatWriterTest {
         }
     }
 
+    @Test
+    public void testArrowFormatCWriterWithEmptySchema() {
+        RowType emptyschema = new RowType(new ArrayList<>());
+
+        try (RootAllocator rootAllocator = new RootAllocator();
+                BufferAllocator allocator =
+                        rootAllocator.newChildAllocator("paimonWriter", 0, Long.MAX_VALUE);
+                ArrowFormatCWriter writer =
+                        new ArrowFormatCWriter(emptyschema, 4096, true, allocator)) {
+            for (int i = 0; i < 100; i++) {
+                writer.write(GenericRow.of());
+            }
+            writer.flush();
+            ArrowCStruct cStruct = writer.toCStruct();
+            Assertions.assertThat(cStruct).isNotNull();
+            writer.release();
+        }
+    }
+
     private void writeAndCheck(ArrowFormatCWriter writer) {
         List<InternalRow> list = new ArrayList<>();
         List<InternalRow.FieldGetter> fieldGetters = new ArrayList<>();
@@ -248,7 +324,7 @@ public class ArrowFormatWriterTest {
                         .isEqualTo(fieldGetter.getFieldOrNull(expectec));
             }
         }
-        writer.release();
+        vectorSchemaRoot.close();
     }
 
     private Object[] randomRowValues(boolean[] nullable) {
