@@ -28,7 +28,6 @@ import org.apache.paimon.types.RowType;
 
 import javax.annotation.Nullable;
 
-import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -56,11 +55,12 @@ public abstract class FileFormat {
     /**
      * Create a {@link FormatReaderFactory} from the type, with projection pushed down.
      *
+     * @param dataSchemaRowType The full table schema type.
      * @param projectedRowType Type with projection.
      * @param filters A list of filters in conjunctive form for filtering on a best-effort basis.
      */
     public abstract FormatReaderFactory createReaderFactory(
-            RowType projectedRowType, @Nullable List<Predicate> filters);
+            RowType dataSchemaRowType, RowType projectedRowType, @Nullable List<Predicate> filters);
 
     /** Create a {@link FormatWriterFactory} from the type. */
     public abstract FormatWriterFactory createWriterFactory(RowType type);
@@ -68,10 +68,6 @@ public abstract class FileFormat {
     /** Validate data field type supported or not. */
     public abstract void validateDataFields(RowType rowType);
 
-    public FormatReaderFactory createReaderFactory(RowType rowType) {
-        return createReaderFactory(rowType, new ArrayList<>());
-    }
-
     public Optional<SimpleStatsExtractor> createStatsExtractor(
             RowType type, SimpleColStatsCollector.Factory[] statsCollectors) {
         return Optional.empty();
@@ -84,6 +80,7 @@ public abstract class FileFormat {
                         options,
                         options.get(CoreOptions.READ_BATCH_SIZE),
                         options.get(CoreOptions.WRITE_BATCH_SIZE),
+                        options.get(CoreOptions.WRITE_BATCH_MEMORY),
                         options.get(CoreOptions.FILE_COMPRESSION_ZSTD_LEVEL),
                         options.get(CoreOptions.FILE_BLOCK_SIZE)));
     }
