@@ -22,13 +22,11 @@ import org.apache.paimon.CoreOptions.ExternalPathStrategy;
 import org.apache.paimon.catalog.RenamingSnapshotCommit;
 import org.apache.paimon.catalog.SnapshotCommit;
 import org.apache.paimon.data.InternalRow;
-import org.apache.paimon.deletionvectors.DeletionVectorsIndexFile;
 import org.apache.paimon.format.FileFormat;
 import org.apache.paimon.fs.FileIO;
 import org.apache.paimon.fs.Path;
 import org.apache.paimon.iceberg.IcebergCommitCallback;
 import org.apache.paimon.iceberg.IcebergOptions;
-import org.apache.paimon.index.HashIndexFile;
 import org.apache.paimon.index.IndexFileHandler;
 import org.apache.paimon.manifest.IndexManifestFile;
 import org.apache.paimon.manifest.ManifestFile;
@@ -38,20 +36,17 @@ import org.apache.paimon.metastore.AddPartitionTagCallback;
 import org.apache.paimon.metastore.TagPreviewCommitCallback;
 import org.apache.paimon.operation.ChangelogDeletion;
 import org.apache.paimon.operation.FileStoreCommitImpl;
-import org.apache.paimon.operation.FileStoreScan;
 import org.apache.paimon.operation.Lock;
 import org.apache.paimon.operation.ManifestsReader;
 import org.apache.paimon.operation.PartitionExpire;
 import org.apache.paimon.operation.SnapshotDeletion;
 import org.apache.paimon.operation.TagDeletion;
-import org.apache.paimon.options.MemorySize;
 import org.apache.paimon.partition.PartitionExpireStrategy;
 import org.apache.paimon.schema.SchemaManager;
 import org.apache.paimon.schema.TableSchema;
 import org.apache.paimon.service.ServiceManager;
 import org.apache.paimon.stats.StatsFile;
 import org.apache.paimon.stats.StatsFileHandler;
-import org.apache.paimon.table.BucketMode;
 import org.apache.paimon.table.CatalogEnvironment;
 import org.apache.paimon.table.FileStoreTable;
 import org.apache.paimon.table.PartitionHandler;
@@ -64,6 +59,7 @@ import org.apache.paimon.tag.TagPreview;
 import org.apache.paimon.types.RowType;
 import org.apache.paimon.utils.ChangelogManager;
 import org.apache.paimon.utils.FileStorePathFactory;
+import org.apache.paimon.utils.IndexFilePathFactories;
 import org.apache.paimon.utils.InternalRowPartitionComputer;
 import org.apache.paimon.utils.SegmentsCache;
 import org.apache.paimon.utils.SnapshotManager;
@@ -79,6 +75,7 @@ import java.util.Collections;
 import java.util.Comparator;
 import java.util.List;
 
+import static org.apache.paimon.catalog.Identifier.DEFAULT_MAIN_BRANCH;
 import static org.apache.paimon.partition.PartitionExpireStrategy.createPartitionExpireStrategy;
 import static org.apache.paimon.utils.Preconditions.checkArgument;
 
@@ -97,7 +94,6 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
     protected final RowType partitionType;
     protected final CatalogEnvironment catalogEnvironment;
 
-    @Nullable private final SegmentsCache<Path> writeManifestCache;
     @Nullable private SegmentsCache<Path> readManifestCache;
     @Nullable private Cache<Path, Snapshot> snapshotCache;
 
@@ -116,9 +112,6 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
         this.options = options;
         this.partitionType = partitionType;
         this.catalogEnvironment = catalogEnvironment;
-        this.writeManifestCache =
-                SegmentsCache.create(
-                        options.pageSize(), options.writeManifestCache(), Long.MAX_VALUE);
     }
 
     @Override
@@ -138,7 +131,8 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
                 options.fileSuffixIncludeCompression(),
                 options.fileCompression(),
                 options.dataFilePathDirectory(),
-                createExternalPaths());
+                createExternalPaths(),
+                options.indexFileInDataFileDir());
     }
 
     private List<Path> createExternalPaths() {
@@ -193,10 +187,6 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
 
     @Override
     public ManifestFile.Factory manifestFileFactory() {
-        return manifestFileFactory(false);
-    }
-
-    protected ManifestFile.Factory manifestFileFactory(boolean forWrite) {
         return new ManifestFile.Factory(
                 fileIO,
                 schemaManager,
@@ -205,21 +195,17 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
                 options.manifestCompression(),
                 pathFactory(),
                 options.manifestTargetSize().getBytes(),
-                forWrite ? writeManifestCache : readManifestCache);
+                readManifestCache);
     }
 
     @Override
     public ManifestList.Factory manifestListFactory() {
-        return manifestListFactory(false);
-    }
-
-    protected ManifestList.Factory manifestListFactory(boolean forWrite) {
         return new ManifestList.Factory(
                 fileIO,
                 FileFormat.manifestFormat(options),
                 options.manifestCompression(),
                 pathFactory(),
-                forWrite ? writeManifestCache : readManifestCache);
+                readManifestCache);
     }
 
     @Override
@@ -235,17 +221,12 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
     @Override
     public IndexFileHandler newIndexFileHandler() {
         return new IndexFileHandler(
+                fileIO,
                 snapshotManager(),
-                pathFactory().indexFileFactory(),
                 indexManifestFileFactory().create(),
-                new HashIndexFile(fileIO, pathFactory().indexFileFactory()),
-                new DeletionVectorsIndexFile(
-                        fileIO,
-                        pathFactory().indexFileFactory(),
-                        bucketMode() == BucketMode.BUCKET_UNAWARE
-                                ? options.deletionVectorIndexFileTargetSize()
-                                : MemorySize.ofBytes(Long.MAX_VALUE),
-                        options.deletionVectorBitmap64()));
+                new IndexFilePathFactories(pathFactory()),
+                options.dvIndexFileTargetSize(),
+                options.deletionVectorBitmap64());
     }
 
     @Override
@@ -256,12 +237,12 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
                 new StatsFile(fileIO, pathFactory().statsFileFactory()));
     }
 
-    protected ManifestsReader newManifestsReader(boolean forWrite) {
+    protected ManifestsReader newManifestsReader() {
         return new ManifestsReader(
                 partitionType,
                 options.partitionDefaultName(),
                 snapshotManager(),
-                manifestListFactory(forWrite));
+                manifestListFactory());
     }
 
     @Override
@@ -279,14 +260,6 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
         return schemaManager.mergeSchema(rowType, allowExplicitCast);
     }
 
-    protected abstract FileStoreScan newScan(ScanType scanType);
-
-    protected enum ScanType {
-        FOR_READ,
-        FOR_WRITE,
-        FOR_COMMIT
-    }
-
     @Override
     public FileStoreCommitImpl newCommit(String commitUser, FileStoreTable table) {
         SnapshotManager snapshotManager = snapshotManager();
@@ -308,7 +281,7 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
                 manifestFileFactory(),
                 manifestListFactory(),
                 indexManifestFileFactory(),
-                newScan(ScanType.FOR_COMMIT),
+                newScan(),
                 options.bucket(),
                 options.manifestTargetSize(),
                 options.manifestFullCompactionThresholdSize(),
@@ -322,7 +295,13 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
                 createCommitCallbacks(commitUser, table),
                 options.commitMaxRetries(),
                 options.commitTimeout(),
-                options.commitStrictModeLastSafeSnapshot().orElse(null));
+                options.commitMinRetryWait(),
+                options.commitMaxRetryWait(),
+                options.commitStrictModeLastSafeSnapshot().orElse(null),
+                options.rowTrackingEnabled(),
+                !schema.primaryKeys().isEmpty(),
+                options.deletionVectorsEnabled(),
+                newIndexFileHandler());
     }
 
     @Override
@@ -336,7 +315,7 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
                 newStatsFileHandler(),
                 options.changelogProducer() != CoreOptions.ChangelogProducer.NONE,
                 options.cleanEmptyDirectories(),
-                options.deleteFileThreadNum());
+                options.fileOperationThreadNum());
     }
 
     @Override
@@ -349,12 +328,12 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
                 newIndexFileHandler(),
                 newStatsFileHandler(),
                 options.cleanEmptyDirectories(),
-                options.deleteFileThreadNum());
+                options.fileOperationThreadNum());
     }
 
     @Override
     public TagManager newTagManager() {
-        return new TagManager(fileIO, options.path());
+        return new TagManager(fileIO, options.path(), DEFAULT_MAIN_BRANCH, options);
     }
 
     @Override
@@ -367,14 +346,14 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
                 newIndexFileHandler(),
                 newStatsFileHandler(),
                 options.cleanEmptyDirectories(),
-                options.deleteFileThreadNum());
+                options.fileOperationThreadNum());
     }
 
     public abstract Comparator<InternalRow> newKeyComparator();
 
     private List<CommitCallback> createCommitCallbacks(String commitUser, FileStoreTable table) {
         List<CommitCallback> callbacks =
-                new ArrayList<>(CallbackUtils.loadCommitCallbacks(options));
+                new ArrayList<>(CallbackUtils.loadCommitCallbacks(options, table));
 
         if (options.partitionedTableInMetastore() && !schema.partitionKeys().isEmpty()) {
             PartitionHandler partitionHandler = catalogEnvironment.partitionHandler();
@@ -448,7 +427,7 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
                 expirationTime,
                 checkInterval,
                 expireStrategy,
-                newScan(ScanType.FOR_COMMIT),
+                newScan(),
                 newCommit(commitUser, table),
                 partitionHandler,
                 options.endInputCheckPartitionExpire(),
@@ -457,7 +436,7 @@ abstract class AbstractFileStore<T> implements FileStore<T> {
     }
 
     @Override
-    public TagAutoManager newTagCreationManager(FileStoreTable table) {
+    public TagAutoManager newTagAutoManager(FileStoreTable table) {
         return TagAutoManager.create(
                 options,
                 snapshotManager(),
