@@ -39,10 +39,8 @@ import org.apache.paimon.table.FormatTable;
 import org.apache.paimon.table.Instant;
 import org.apache.paimon.table.Table;
 import org.apache.paimon.table.TableSnapshot;
-import org.apache.paimon.table.object.ObjectTable;
 import org.apache.paimon.table.sink.BatchTableCommit;
 import org.apache.paimon.table.system.SystemTableLoader;
-import org.apache.paimon.types.RowType;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -59,10 +57,10 @@ import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 import java.util.Optional;
+import java.util.Set;
 import java.util.stream.Collectors;
 
 import static org.apache.paimon.CoreOptions.DATA_FILE_EXTERNAL_PATHS;
-import static org.apache.paimon.CoreOptions.OBJECT_LOCATION;
 import static org.apache.paimon.CoreOptions.PATH;
 import static org.apache.paimon.CoreOptions.TYPE;
 import static org.apache.paimon.catalog.CatalogUtils.checkNotBranch;
@@ -74,7 +72,6 @@ import static org.apache.paimon.catalog.CatalogUtils.validateCreateTable;
 import static org.apache.paimon.catalog.Identifier.DEFAULT_MAIN_BRANCH;
 import static org.apache.paimon.options.CatalogOptions.LOCK_ENABLED;
 import static org.apache.paimon.options.CatalogOptions.LOCK_TYPE;
-import static org.apache.paimon.utils.Preconditions.checkArgument;
 
 /** Common implementation of {@link Catalog}. */
 public abstract class AbstractCatalog implements Catalog {
@@ -253,17 +250,27 @@ public abstract class AbstractCatalog implements Catalog {
 
     @Override
     public PagedList<String> listTablesPaged(
-            String databaseName, Integer maxResults, String pageToken, String tableNamePattern)
+            String databaseName,
+            Integer maxResults,
+            String pageToken,
+            String tableNamePattern,
+            String tableType)
             throws DatabaseNotExistException {
         CatalogUtils.validateNamePattern(this, tableNamePattern);
+        CatalogUtils.validateTableType(this, tableType);
         return new PagedList<>(listTables(databaseName), null);
     }
 
     @Override
     public PagedList<Table> listTableDetailsPaged(
-            String databaseName, Integer maxResults, String pageToken, String tableNamePattern)
+            String databaseName,
+            @Nullable Integer maxResults,
+            @Nullable String pageToken,
+            @Nullable String tableNamePattern,
+            @Nullable String tableType)
             throws DatabaseNotExistException {
         CatalogUtils.validateNamePattern(this, tableNamePattern);
+        CatalogUtils.validateTableType(this, tableType);
         if (isSystemDatabase(databaseName)) {
             List<Table> systemTables =
                     SystemTableLoader.loadGlobalTableNames().stream()
@@ -288,16 +295,21 @@ public abstract class AbstractCatalog implements Catalog {
         // check db exists
         getDatabase(databaseName);
 
-        return listTableDetailsPagedImpl(databaseName, maxResults, pageToken);
+        return listTableDetailsPagedImpl(
+                databaseName, maxResults, pageToken, tableNamePattern, tableType);
     }
 
     protected abstract List<String> listTablesImpl(String databaseName);
 
     protected PagedList<Table> listTableDetailsPagedImpl(
-            String databaseName, Integer maxResults, String pageToken)
+            String databaseName,
+            @Nullable Integer maxResults,
+            @Nullable String pageToken,
+            @Nullable String tableNamePattern,
+            @Nullable String tableType)
             throws DatabaseNotExistException {
         PagedList<String> pagedTableNames =
-                listTablesPaged(databaseName, maxResults, pageToken, null);
+                listTablesPaged(databaseName, maxResults, pageToken, tableNamePattern, tableType);
         return new PagedList<>(
                 pagedTableNames.getElements().stream()
                         .map(
@@ -323,23 +335,21 @@ public abstract class AbstractCatalog implements Catalog {
         checkNotBranch(identifier, "dropTable");
         checkNotSystemTable(identifier, "dropTable");
 
-        List<Path> externalPaths = new ArrayList<>();
+        Set<Path> externalPaths = new HashSet<>();
         try {
             Table table = getTable(identifier);
             if (table instanceof FileStoreTable) {
                 FileStoreTable fileStoreTable = (FileStoreTable) table;
-                externalPaths =
-                        fileStoreTable.schemaManager().listAll().stream()
-                                .map(
-                                        schema ->
-                                                schema.toSchema()
-                                                        .options()
-                                                        .get(DATA_FILE_EXTERNAL_PATHS.key()))
-                                .filter(Objects::nonNull)
-                                .flatMap(externalPath -> Arrays.stream(externalPath.split(",")))
-                                .map(Path::new)
-                                .distinct()
-                                .collect(Collectors.toList());
+                List<Path> schemaExternalPaths =
+                        getSchemaExternalPaths(fileStoreTable.schemaManager().listAll());
+                externalPaths.addAll(schemaExternalPaths);
+                // get table branch external path
+                List<String> branches = fileStoreTable.branchManager().branches();
+                for (String branch : branches) {
+                    SchemaManager schemaManager =
+                            fileStoreTable.schemaManager().copyWithBranch(branch);
+                    externalPaths.addAll(getSchemaExternalPaths(schemaManager.listAll()));
+                }
             }
         } catch (TableNotExistException e) {
             if (ignoreIfNotExists) {
@@ -348,7 +358,20 @@ public abstract class AbstractCatalog implements Catalog {
             throw new TableNotExistException(identifier);
         }
 
-        dropTableImpl(identifier, externalPaths);
+        dropTableImpl(identifier, new ArrayList<>(externalPaths));
+    }
+
+    private List<Path> getSchemaExternalPaths(List<TableSchema> schemas) {
+        if (schemas == null) {
+            return Collections.emptyList();
+        }
+        return schemas.stream()
+                .map(schema -> schema.toSchema().options().get(DATA_FILE_EXTERNAL_PATHS.key()))
+                .filter(Objects::nonNull)
+                .flatMap(externalPath -> Arrays.stream(externalPath.split(",")))
+                .map(Path::new)
+                .distinct()
+                .collect(Collectors.toList());
     }
 
     protected abstract void dropTableImpl(Identifier identifier, List<Path> externalPaths);
@@ -380,30 +403,17 @@ public abstract class AbstractCatalog implements Catalog {
             case MATERIALIZED_TABLE:
                 createTableImpl(identifier, schema);
                 break;
-            case OBJECT_TABLE:
-                createObjectTable(identifier, schema);
-                break;
             case FORMAT_TABLE:
                 createFormatTable(identifier, schema);
                 break;
+            case OBJECT_TABLE:
+                throw new UnsupportedOperationException(
+                        String.format(
+                                "Catalog %s cannot support object tables.",
+                                this.getClass().getName()));
         }
     }
 
-    private void createObjectTable(Identifier identifier, Schema schema) {
-        RowType rowType = schema.rowType();
-        checkArgument(
-                rowType.getFields().isEmpty()
-                        || new HashSet<>(ObjectTable.SCHEMA.getFields())
-                                .containsAll(rowType.getFields()),
-                "Schema of Object Table can be empty or %s, but is %s.",
-                ObjectTable.SCHEMA,
-                rowType);
-        checkArgument(
-                schema.options().containsKey(OBJECT_LOCATION.key()),
-                "Object table should have object-location option.");
-        createTableImpl(identifier, schema.copy(ObjectTable.SCHEMA));
-    }
-
     protected abstract void createTableImpl(Identifier identifier, Schema schema);
 
     @Override
