@@ -33,10 +33,13 @@ import org.apache.paimon.table.FileStoreTable;
 import org.apache.paimon.table.FileStoreTableFactory;
 import org.apache.paimon.table.FormatTable;
 import org.apache.paimon.table.Table;
+import org.apache.paimon.table.iceberg.IcebergTable;
+import org.apache.paimon.table.lance.LanceTable;
 import org.apache.paimon.table.object.ObjectTable;
 import org.apache.paimon.table.system.AllTableOptionsTable;
 import org.apache.paimon.table.system.CatalogOptionsTable;
 import org.apache.paimon.table.system.SystemTableLoader;
+import org.apache.paimon.types.DataField;
 import org.apache.paimon.utils.InternalRowPartitionComputer;
 import org.apache.paimon.utils.Preconditions;
 
@@ -51,7 +54,7 @@ import java.util.function.Function;
 
 import static org.apache.paimon.CoreOptions.AUTO_CREATE;
 import static org.apache.paimon.CoreOptions.PARTITION_DEFAULT_NAME;
-import static org.apache.paimon.CoreOptions.PARTITION_GENERATE_LEGCY_NAME;
+import static org.apache.paimon.CoreOptions.PARTITION_GENERATE_LEGACY_NAME;
 import static org.apache.paimon.CoreOptions.PATH;
 import static org.apache.paimon.CoreOptions.PRIMARY_KEY;
 import static org.apache.paimon.catalog.Catalog.SYSTEM_DATABASE_NAME;
@@ -59,6 +62,7 @@ import static org.apache.paimon.catalog.Catalog.TABLE_DEFAULT_OPTION_PREFIX;
 import static org.apache.paimon.options.OptionsUtils.convertToPropertiesPrefixKey;
 import static org.apache.paimon.table.system.AllTableOptionsTable.ALL_TABLE_OPTIONS;
 import static org.apache.paimon.table.system.CatalogOptionsTable.CATALOG_OPTIONS;
+import static org.apache.paimon.utils.DefaultValueUtils.validateDefaultValue;
 import static org.apache.paimon.utils.Preconditions.checkArgument;
 
 /** Utils for {@link Catalog}. */
@@ -145,6 +149,9 @@ public class CatalogUtils {
                     "Cannot define %s for format table.",
                     PRIMARY_KEY.key());
         }
+        for (DataField field : schema.fields()) {
+            validateDefaultValue(field.type(), field.defaultValue());
+        }
     }
 
     public static void validateNamePattern(Catalog catalog, String namePattern) {
@@ -156,6 +163,15 @@ public class CatalogUtils {
         }
     }
 
+    public static void validateTableType(Catalog catalog, String tableType) {
+        if (Objects.nonNull(tableType) && !catalog.supportsListTableByType()) {
+            throw new UnsupportedOperationException(
+                    String.format(
+                            "Current catalog %s does not support table type filter.",
+                            catalog.getClass().getSimpleName()));
+        }
+    }
+
     public static List<Partition> listPartitionsFromFileSystem(Table table) {
         Options options = Options.fromMap(table.options());
         InternalRowPartitionComputer computer =
@@ -163,7 +179,7 @@ public class CatalogUtils {
                         options.get(PARTITION_DEFAULT_NAME),
                         table.rowType().project(table.partitionKeys()),
                         table.partitionKeys().toArray(new String[0]),
-                        options.get(PARTITION_GENERATE_LEGCY_NAME));
+                        options.get(PARTITION_GENERATE_LEGACY_NAME));
         List<PartitionEntry> partitionEntries =
                 table.newReadBuilder().newScan().listPartitionEntries();
         List<Partition> partitions = new ArrayList<>(partitionEntries.size());
@@ -207,9 +223,30 @@ public class CatalogUtils {
             return toFormatTable(identifier, schema, dataFileIO);
         }
 
+        if (options.type() == TableType.OBJECT_TABLE) {
+            return toObjectTable(identifier, schema, dataFileIO);
+        }
+
+        if (options.type() == TableType.LANCE_TABLE) {
+            return toLanceTable(identifier, schema, dataFileIO);
+        }
+
+        if (options.type() == TableType.ICEBERG_TABLE) {
+            return toIcebergTable(identifier, schema, dataFileIO);
+        }
+
+        Identifier tableIdentifier = identifier;
+        if (identifier.isSystemTable()) {
+            tableIdentifier =
+                    new Identifier(
+                            identifier.getDatabaseName(),
+                            identifier.getTableName(),
+                            identifier.getBranchName());
+        }
+
         CatalogEnvironment catalogEnv =
                 new CatalogEnvironment(
-                        identifier,
+                        tableIdentifier,
                         metadata.uuid(),
                         catalog.catalogLoader(),
                         lockFactory,
@@ -219,10 +256,6 @@ public class CatalogUtils {
         FileStoreTable table =
                 FileStoreTableFactory.create(dataFileIO.apply(path), path, schema, catalogEnv);
 
-        if (options.type() == TableType.OBJECT_TABLE) {
-            table = toObjectTable(externalFileIO, table);
-        }
-
         if (identifier.isSystemTable()) {
             return CatalogUtils.createSystemTable(identifier, table);
         }
@@ -295,14 +328,43 @@ public class CatalogUtils {
     }
 
     private static ObjectTable toObjectTable(
-            Function<Path, FileIO> fileIOLoader, FileStoreTable underlyingTable) {
-        CoreOptions options = underlyingTable.coreOptions();
-        String objectLocation = options.objectLocation();
-        FileIO objectFileIO = fileIOLoader.apply(new Path(objectLocation));
+            Identifier identifier, TableSchema schema, Function<Path, FileIO> fileIO) {
+        Map<String, String> options = schema.options();
+        String location = options.get(CoreOptions.PATH.key());
         return ObjectTable.builder()
-                .underlyingTable(underlyingTable)
-                .objectLocation(objectLocation)
-                .objectFileIO(objectFileIO)
+                .fileIO(fileIO.apply(new Path(location)))
+                .identifier(identifier)
+                .location(location)
+                .comment(schema.comment())
+                .build();
+    }
+
+    private static LanceTable toLanceTable(
+            Identifier identifier, TableSchema schema, Function<Path, FileIO> fileIO) {
+        Map<String, String> options = schema.options();
+        String location = options.get(CoreOptions.PATH.key());
+        return LanceTable.builder()
+                .fileIO(fileIO.apply(new Path(location)))
+                .identifier(identifier)
+                .location(location)
+                .rowType(schema.logicalRowType())
+                .options(options)
+                .comment(schema.comment())
+                .build();
+    }
+
+    private static IcebergTable toIcebergTable(
+            Identifier identifier, TableSchema schema, Function<Path, FileIO> fileIO) {
+        Map<String, String> options = schema.options();
+        String location = options.get(CoreOptions.PATH.key());
+        return IcebergTable.builder()
+                .fileIO(fileIO.apply(new Path(location)))
+                .identifier(identifier)
+                .location(location)
+                .rowType(schema.logicalRowType())
+                .partitionKeys(schema.partitionKeys())
+                .options(options)
+                .comment(schema.comment())
                 .build();
     }
 }
