@@ -20,8 +20,8 @@ package org.apache.paimon.deletionvectors;
 
 import org.apache.paimon.fs.FileIO;
 import org.apache.paimon.index.IndexFileMeta;
+import org.apache.paimon.index.IndexPathFactory;
 import org.apache.paimon.options.MemorySize;
-import org.apache.paimon.utils.PathFactory;
 
 import java.io.IOException;
 import java.util.ArrayList;
@@ -33,24 +33,40 @@ import java.util.Map;
 /** Writer for deletion vector index file. */
 public class DeletionVectorIndexFileWriter {
 
-    private final PathFactory indexPathFactory;
+    private final IndexPathFactory indexPathFactory;
     private final FileIO fileIO;
     private final long targetSizeInBytes;
 
     public DeletionVectorIndexFileWriter(
-            FileIO fileIO, PathFactory pathFactory, MemorySize targetSizePerIndexFile) {
+            FileIO fileIO, IndexPathFactory pathFactory, MemorySize targetSizePerIndexFile) {
         this.indexPathFactory = pathFactory;
         this.fileIO = fileIO;
         this.targetSizeInBytes = targetSizePerIndexFile.getBytes();
     }
 
     /**
-     * For unaware-bucket mode, this method will write out multiple index files, else, it will write
-     * out only one index file.
+     * The deletion file of the bucketed table is updated according to the bucket. If a compaction
+     * occurs and there is no longer a deletion file, an empty deletion file needs to be generated
+     * to overwrite the old file.
+     *
+     * <p>TODO: We can consider sending a message to delete the deletion file in the future.
      */
-    public List<IndexFileMeta> write(Map<String, DeletionVector> input) throws IOException {
+    public IndexFileMeta writeSingleFile(Map<String, DeletionVector> input) throws IOException {
+        DeletionFileWriter writer = new DeletionFileWriter(indexPathFactory, fileIO);
+        try {
+            for (Map.Entry<String, DeletionVector> entry : input.entrySet()) {
+                writer.write(entry.getKey(), entry.getValue());
+            }
+        } finally {
+            writer.close();
+        }
+        return writer.result();
+    }
+
+    public List<IndexFileMeta> writeWithRolling(Map<String, DeletionVector> input)
+            throws IOException {
         if (input.isEmpty()) {
-            return emptyIndexFile();
+            return Collections.emptyList();
         }
         List<IndexFileMeta> result = new ArrayList<>();
         Iterator<Map.Entry<String, DeletionVector>> iterator = input.entrySet().iterator();
@@ -62,7 +78,7 @@ public class DeletionVectorIndexFileWriter {
 
     private IndexFileMeta tryWriter(Iterator<Map.Entry<String, DeletionVector>> iterator)
             throws IOException {
-        DeletionFileWriter writer = new DeletionFileWriter(indexPathFactory.newPath(), fileIO);
+        DeletionFileWriter writer = new DeletionFileWriter(indexPathFactory, fileIO);
         try {
             while (iterator.hasNext()) {
                 Map.Entry<String, DeletionVector> entry = iterator.next();
@@ -76,17 +92,4 @@ public class DeletionVectorIndexFileWriter {
         }
         return writer.result();
     }
-
-    /**
-     * The deletion file of the bucketed table is updated according to the bucket. If a compaction
-     * occurs and there is no longer a deletion file, an empty deletion file needs to be generated
-     * to overwrite the old file.
-     *
-     * <p>TODO: We can consider sending a message to delete the deletion file in the future.
-     */
-    private List<IndexFileMeta> emptyIndexFile() throws IOException {
-        DeletionFileWriter writer = new DeletionFileWriter(indexPathFactory.newPath(), fileIO);
-        writer.close();
-        return Collections.singletonList(writer.result());
-    }
 }
