@@ -24,9 +24,9 @@ import org.apache.paimon.fs.SeekableInputStream;
 import org.apache.paimon.index.DeletionVectorMeta;
 import org.apache.paimon.index.IndexFile;
 import org.apache.paimon.index.IndexFileMeta;
+import org.apache.paimon.index.IndexPathFactory;
 import org.apache.paimon.options.MemorySize;
 import org.apache.paimon.table.source.DeletionFile;
-import org.apache.paimon.utils.PathFactory;
 
 import java.io.DataInputStream;
 import java.io.IOException;
@@ -51,7 +51,7 @@ public class DeletionVectorsIndexFile extends IndexFile {
 
     public DeletionVectorsIndexFile(
             FileIO fileIO,
-            PathFactory pathFactory,
+            IndexPathFactory pathFactory,
             MemorySize targetSizePerIndexFile,
             boolean bitmap64) {
         super(fileIO, pathFactory);
@@ -71,13 +71,11 @@ public class DeletionVectorsIndexFile extends IndexFile {
      * @throws UncheckedIOException If an I/O error occurs while reading from the file.
      */
     public Map<String, DeletionVector> readAllDeletionVectors(IndexFileMeta fileMeta) {
-        LinkedHashMap<String, DeletionVectorMeta> deletionVectorMetas =
-                fileMeta.deletionVectorMetas();
+        LinkedHashMap<String, DeletionVectorMeta> deletionVectorMetas = fileMeta.dvRanges();
         checkNotNull(deletionVectorMetas);
 
-        String indexFileName = fileMeta.fileName();
         Map<String, DeletionVector> deletionVectors = new HashMap<>();
-        Path filePath = pathFactory.toPath(indexFileName);
+        Path filePath = pathFactory.toPath(fileMeta);
         try (SeekableInputStream inputStream = fileIO.newInputStream(filePath)) {
             checkVersion(inputStream);
             DataInputStream dataInputStream = new DataInputStream(inputStream);
@@ -141,29 +139,27 @@ public class DeletionVectorsIndexFile extends IndexFile {
         }
     }
 
-    /**
-     * Write deletion vectors to a new file, the format of this file can be referenced at: <a
-     * href="https://cwiki.apache.org/confluence/x/Tws4EQ">PIP-16</a>.
-     *
-     * @param input A map where the key represents which file the DeletionVector belongs to, and the
-     *     value is the corresponding DeletionVector object.
-     * @return A Pair object specifying the name of the written new file and a map where the key
-     *     represents which file the DeletionVector belongs to and the value is a Pair object
-     *     specifying the range (start position and size) within the file where the deletion vector
-     *     data is located.
-     * @throws UncheckedIOException If an I/O error occurs while writing to the file.
-     */
-    public List<IndexFileMeta> write(Map<String, DeletionVector> input) {
+    public IndexFileMeta writeSingleFile(Map<String, DeletionVector> input) {
         try {
-            DeletionVectorIndexFileWriter writer =
-                    new DeletionVectorIndexFileWriter(
-                            this.fileIO, this.pathFactory, this.targetSizePerIndexFile);
-            return writer.write(input);
+            return createWriter().writeSingleFile(input);
         } catch (IOException e) {
             throw new RuntimeException("Failed to write deletion vectors.", e);
         }
     }
 
+    public List<IndexFileMeta> writeWithRolling(Map<String, DeletionVector> input) {
+        try {
+            return createWriter().writeWithRolling(input);
+        } catch (IOException e) {
+            throw new RuntimeException("Failed to write deletion vectors.", e);
+        }
+    }
+
+    private DeletionVectorIndexFileWriter createWriter() {
+        return new DeletionVectorIndexFileWriter(
+                this.fileIO, this.pathFactory, this.targetSizePerIndexFile);
+    }
+
     private void checkVersion(InputStream in) throws IOException {
         int version = in.read();
         if (version != VERSION_ID_V1) {
