@@ -43,12 +43,12 @@ import org.apache.paimon.iceberg.metadata.IcebergSnapshot;
 import org.apache.paimon.iceberg.metadata.IcebergSnapshotSummary;
 import org.apache.paimon.index.DeletionVectorMeta;
 import org.apache.paimon.index.IndexFileHandler;
-import org.apache.paimon.index.IndexFileMeta;
 import org.apache.paimon.io.DataFileMeta;
 import org.apache.paimon.io.DataFilePathFactory;
 import org.apache.paimon.manifest.IndexManifestEntry;
 import org.apache.paimon.manifest.ManifestCommittable;
 import org.apache.paimon.manifest.ManifestEntry;
+import org.apache.paimon.manifest.SimpleFileEntry;
 import org.apache.paimon.options.Options;
 import org.apache.paimon.partition.PartitionPredicate;
 import org.apache.paimon.schema.SchemaManager;
@@ -90,6 +90,7 @@ import java.util.Set;
 import java.util.UUID;
 import java.util.function.Function;
 import java.util.stream.Collectors;
+import java.util.stream.IntStream;
 import java.util.stream.Stream;
 
 import static org.apache.paimon.deletionvectors.DeletionVectorsIndexFile.DELETION_VECTORS_INDEX;
@@ -208,6 +209,7 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
                 return IcebergOptions.StorageLocation.TABLE_LOCATION;
             case HIVE_CATALOG:
             case HADOOP_CATALOG:
+            case REST_CATALOG:
                 return IcebergOptions.StorageLocation.CATALOG_STORAGE;
             default:
                 throw new UnsupportedOperationException(
@@ -220,13 +222,14 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
 
     @Override
     public void call(
-            List<ManifestEntry> committedEntries,
+            List<SimpleFileEntry> baseFiles,
+            List<ManifestEntry> deltaFiles,
             List<IndexManifestEntry> indexFiles,
             Snapshot snapshot) {
         createMetadata(
                 snapshot,
                 (removedFiles, addedFiles) ->
-                        collectFileChanges(committedEntries, removedFiles, addedFiles),
+                        collectFileChanges(deltaFiles, removedFiles, addedFiles),
                 indexFiles);
     }
 
@@ -341,14 +344,20 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
                         pathFactory.toManifestListPath(manifestListFileName).toString(),
                         schemaId);
 
-        Map<String, IcebergRef> icebergTags =
-                table.tagManager().tags().entrySet().stream()
-                        .collect(
-                                Collectors.toMap(
-                                        entry -> entry.getValue().get(0),
-                                        entry -> new IcebergRef(entry.getKey().id())));
+        // Tags can only be included in Iceberg if they point to an Iceberg snapshot that
+        // exists. Otherwise an Iceberg client fails to parse the metadata and all reads fail.
+        // Only the latest snapshot ID is added to Iceberg in this code path. Since this snapshot
+        // has just been committed to Paimon, it is not possible for any Paimon tag to reference it
+        // yet.
+        // After https://github.com/apache/paimon/issues/6107 we can add tags here.
+        Map<String, IcebergRef> refs = new HashMap<>();
 
         String tableUuid = UUID.randomUUID().toString();
+
+        List<IcebergSchema> allSchemas =
+                IntStream.rangeClosed(0, schemaId)
+                        .mapToObj(schemaCache::get)
+                        .collect(Collectors.toList());
         IcebergMetadata metadata =
                 new IcebergMetadata(
                         formatVersion,
@@ -356,7 +365,7 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
                         table.location().toString(),
                         snapshotId,
                         icebergSchema.highestFieldId(),
-                        Collections.singletonList(icebergSchema),
+                        allSchemas,
                         schemaId,
                         Collections.singletonList(new IcebergPartitionSpec(partitionFields)),
                         partitionFields.stream()
@@ -367,7 +376,7 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
                                         IcebergPartitionField.FIRST_FIELD_ID - 1),
                         Collections.singletonList(snapshot),
                         (int) snapshotId,
-                        icebergTags);
+                        refs);
 
         Path metadataPath = pathFactory.toMetadataPath(snapshotId);
         table.fileIO().tryToWriteAtomic(metadataPath, metadata.toJson());
@@ -379,7 +388,17 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
         expireAllBefore(snapshotId);
 
         if (metadataCommitter != null) {
-            metadataCommitter.commitMetadata(metadataPath, null);
+            switch (metadataCommitter.identifier()) {
+                case "hive":
+                    metadataCommitter.commitMetadata(metadataPath, null);
+                    break;
+                case "rest":
+                    metadataCommitter.commitMetadata(metadata, null);
+                    break;
+                default:
+                    throw new UnsupportedOperationException(
+                            "Unsupported metadata committer: " + metadataCommitter.identifier());
+            }
         }
     }
 
@@ -557,14 +576,22 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
                                         newDVManifestFileMetas.stream())
                                 .collect(Collectors.toList()));
 
-        // add new schema if needed
+        // add new schemas if needed
         SchemaCache schemaCache = new SchemaCache();
         int schemaId = (int) schemaCache.getLatestSchemaId();
         IcebergSchema icebergSchema = schemaCache.get(schemaId);
         List<IcebergSchema> schemas = baseMetadata.schemas();
         if (baseMetadata.currentSchemaId() != schemaId) {
+            Preconditions.checkArgument(
+                    schemaId > baseMetadata.currentSchemaId(),
+                    "currentSchemaId{%s} in paimon should be greater than currentSchemaId{%s} in base metadata.",
+                    schemaId,
+                    baseMetadata.currentSchemaId());
             schemas = new ArrayList<>(schemas);
-            schemas.add(icebergSchema);
+            schemas.addAll(
+                    IntStream.rangeClosed(baseMetadata.currentSchemaId() + 1, schemaId)
+                            .mapToObj(schemaCache::get)
+                            .collect(Collectors.toList()));
         }
 
         List<IcebergSnapshot> snapshots = new ArrayList<>(baseMetadata.snapshots());
@@ -589,6 +616,18 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
             }
         }
 
+        // Tags can only be included in Iceberg if they point to an Iceberg snapshot that
+        // exists. Otherwise an Iceberg client fails to parse the metadata and all reads fail.
+        Set<Long> snapshotIds =
+                snapshots.stream().map(IcebergSnapshot::snapshotId).collect(Collectors.toSet());
+        Map<String, IcebergRef> refs =
+                table.tagManager().tags().entrySet().stream()
+                        .filter(entry -> snapshotIds.contains(entry.getKey().id()))
+                        .collect(
+                                Collectors.toMap(
+                                        entry -> entry.getValue().get(0),
+                                        entry -> new IcebergRef(entry.getKey().id())));
+
         IcebergMetadata metadata =
                 new IcebergMetadata(
                         baseMetadata.formatVersion(),
@@ -602,7 +641,7 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
                         baseMetadata.lastPartitionId(),
                         snapshots,
                         (int) snapshotId,
-                        baseMetadata.refs());
+                        refs);
 
         Path metadataPath = pathFactory.toMetadataPath(snapshotId);
         table.fileIO().tryToWriteAtomic(metadataPath, metadata.toJson());
@@ -619,7 +658,17 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
         }
 
         if (metadataCommitter != null) {
-            metadataCommitter.commitMetadata(metadataPath, baseMetadataPath);
+            switch (metadataCommitter.identifier()) {
+                case "hive":
+                    metadataCommitter.commitMetadata(metadataPath, baseMetadataPath);
+                    break;
+                case "rest":
+                    metadataCommitter.commitMetadata(metadata, baseMetadata);
+                    break;
+                default:
+                    throw new UnsupportedOperationException(
+                            "Unsupported metadata committer: " + metadataCommitter.identifier());
+            }
         }
     }
 
@@ -942,8 +991,8 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
                 }
                 table.fileIO().deleteQuietly(listPath);
             }
-            deleteApplicableMetadataFiles(snapshotId);
         }
+        deleteApplicableMetadataFiles(snapshotId);
     }
 
     private void deleteApplicableMetadataFiles(long snapshotId) throws IOException {
@@ -995,6 +1044,24 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
             IcebergMetadata baseMetadata =
                     IcebergMetadata.fromPath(table.fileIO(), baseMetadataPath);
 
+            // Tags can only be included in Iceberg if they point to an Iceberg snapshot that
+            // exists. Otherwise an Iceberg client fails to parse the metadata and all reads fail.
+            boolean tagSnapshotInIceberg = false;
+            for (IcebergSnapshot snapshot : baseMetadata.snapshots()) {
+                if (snapshot.snapshotId() == snapshotId) {
+                    tagSnapshotInIceberg = true;
+                    break;
+                }
+            }
+
+            if (!tagSnapshotInIceberg) {
+                LOG.warn(
+                        "Snapshot {} does not exist in Iceberg metadata. Unable to create tag {}.",
+                        snapshotId,
+                        tagName);
+                return;
+            }
+
             baseMetadata.refs().put(tagName, new IcebergRef(snapshotId));
 
             IcebergMetadata metadata =
@@ -1111,8 +1178,7 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
             return Collections.emptyList();
         }
         for (IndexManifestEntry entry : newIndexes) {
-            IndexFileMeta indexFileMeta = entry.indexFile();
-            LinkedHashMap<String, DeletionVectorMeta> dvMetas = indexFileMeta.deletionVectorMetas();
+            LinkedHashMap<String, DeletionVectorMeta> dvMetas = entry.indexFile().dvRanges();
             Path bucketPath = fileStorePathFactory.bucketPath(entry.partition(), entry.bucket());
             if (dvMetas != null) {
                 for (DeletionVectorMeta dvMeta : dvMetas.values()) {
@@ -1125,16 +1191,16 @@ public class IcebergCommitCallback implements CommitCallback, TagCallback {
                             "cardinality in DeletionVector is null, stop generate dv for iceberg. "
                                     + "dataFile path is {}, indexFile path is {}",
                             new Path(bucketPath, dvMeta.dataFileName()),
-                            indexFileHandler.filePath(indexFileMeta).toString());
+                            indexFileHandler.filePath(entry).toString());
 
                     IcebergDataFileMeta deleteFileMeta =
                             IcebergDataFileMeta.createForDeleteFile(
                                     IcebergDataFileMeta.Content.POSITION_DELETES,
-                                    indexFileHandler.filePath(indexFileMeta).toString(),
+                                    indexFileHandler.filePath(entry).toString(),
                                     PUFFIN_FORMAT,
                                     entry.partition(),
                                     dvMeta.cardinality(),
-                                    indexFileMeta.fileSize(),
+                                    entry.indexFile().fileSize(),
                                     new Path(bucketPath, dvMeta.dataFileName()).toString(),
                                     (long) dvMeta.offset(),
                                     (long) dvMeta.length());
