@@ -91,21 +91,16 @@ public class PartitionIndex {
             }
         }
 
-        if (-1 == maxBucketsNum || totalBucketSet.isEmpty() || maxBucketId < maxBucketsNum - 1) {
+        int globalMaxBucketId = (maxBucketsNum == -1 ? Short.MAX_VALUE : maxBucketsNum) - 1;
+        if (totalBucketSet.isEmpty() || maxBucketId < globalMaxBucketId) {
             // 3. create a new bucket
-            for (int i = 0; i < Short.MAX_VALUE; i++) {
+            for (int i = 0; i <= globalMaxBucketId; i++) {
                 if (bucketFilter.test(i) && !totalBucketSet.contains(i)) {
-                    // The new bucketId may still be larger than the upper bound
-                    if (-1 == maxBucketsNum || i <= maxBucketsNum - 1) {
-                        nonFullBucketInformation.put(i, 1L);
-                        totalBucketSet.add(i);
-                        totalBucketArray.add(i);
-                        hash2Bucket.put(hash, (short) i);
-                        return i;
-                    } else {
-                        // No need to enter the next iteration when upper bound exceeded
-                        break;
-                    }
+                    nonFullBucketInformation.put(i, 1L);
+                    totalBucketSet.add(i);
+                    totalBucketArray.add(i);
+                    hash2Bucket.put(hash, (short) i);
+                    return i;
                 }
             }
             if (-1 == maxBucketsNum) {
@@ -132,7 +127,10 @@ public class PartitionIndex {
         Int2ShortHashMap.Builder mapBuilder = Int2ShortHashMap.builder();
         Map<Integer, Long> buckets = new HashMap<>();
         for (IndexManifestEntry file : files) {
-            try (IntIterator iterator = indexFileHandler.readHashIndex(file.indexFile())) {
+            try (IntIterator iterator =
+                    indexFileHandler
+                            .hashIndex(file.partition(), file.bucket())
+                            .read(file.indexFile())) {
                 while (true) {
                     try {
                         int hash = iterator.next();
