@@ -35,7 +35,7 @@ import org.apache.paimon.manifest.ManifestFileMeta;
 import org.apache.paimon.manifest.ManifestList;
 import org.apache.paimon.stats.StatsFileHandler;
 import org.apache.paimon.utils.DataFilePathFactories;
-import org.apache.paimon.utils.FileDeletionThreadPool;
+import org.apache.paimon.utils.FileOperationThreadPool;
 import org.apache.paimon.utils.FileStorePathFactory;
 import org.apache.paimon.utils.Pair;
 import org.apache.paimon.utils.SnapshotManager;
@@ -43,6 +43,7 @@ import org.apache.paimon.utils.SnapshotManager;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -101,7 +102,7 @@ public abstract class FileDeletionBase<T extends Snapshot> {
         this.statsFileHandler = statsFileHandler;
         this.cleanEmptyDirectories = cleanEmptyDirectories;
         this.deletionBuckets = new HashMap<>();
-        this.deleteFileExecutor = FileDeletionThreadPool.getExecutorService(deleteFileThreadNum);
+        this.deleteFileExecutor = FileOperationThreadPool.getExecutorService(deleteFileThreadNum);
     }
 
     /**
@@ -286,9 +287,15 @@ public abstract class FileDeletionBase<T extends Snapshot> {
         // clean index manifests
         String indexManifest = snapshot.indexManifest();
         // check exists, it may have been deleted by other snapshots
-        if (indexManifest != null && indexFileHandler.existsManifest(indexManifest)) {
-            List<IndexManifestEntry> indexManifestEntries =
-                    indexFileHandler.readManifest(indexManifest);
+        if (indexManifest != null) {
+            List<IndexManifestEntry> indexManifestEntries;
+            try {
+                indexManifestEntries = indexFileHandler.readManifestWithIOException(indexManifest);
+            } catch (FileNotFoundException e) {
+                return;
+            } catch (IOException e) {
+                throw new RuntimeException(e);
+            }
             indexManifestEntries.removeIf(
                     entry -> skippingSet.contains(entry.indexFile().fileName()));
             deleteFiles(indexManifestEntries, indexFileHandler::deleteIndexFile);
@@ -406,11 +413,6 @@ public abstract class FileDeletionBase<T extends Snapshot> {
         return false;
     }
 
-    /** Changelogs were not checked. Let the subclass determine whether to delete them. */
-    public Set<String> manifestSkippingSet(Snapshot skippingSnapshot) {
-        return manifestSkippingSet(Collections.singletonList(skippingSnapshot));
-    }
-
     public Set<String> manifestSkippingSet(List<Snapshot> skippingSnapshots) {
         Set<String> skippingSet = new HashSet<>();
 
