@@ -27,6 +27,7 @@ import org.apache.paimon.predicate.Predicate;
 import org.apache.paimon.stats.SimpleStats;
 import org.apache.paimon.table.source.ScanMode;
 import org.apache.paimon.types.RowType;
+import org.apache.paimon.utils.BiFilter;
 import org.apache.paimon.utils.SnapshotManager;
 
 import javax.annotation.Nullable;
@@ -52,6 +53,7 @@ public class ManifestsReader {
     @Nullable private Integer specifiedBucket = null;
     @Nullable private Integer specifiedLevel = null;
     @Nullable private PartitionPredicate partitionFilter = null;
+    @Nullable private BiFilter<Integer, Integer> levelMinMaxFilter = null;
 
     public ManifestsReader(
             RowType partitionType,
@@ -79,6 +81,11 @@ public class ManifestsReader {
         return this;
     }
 
+    public ManifestsReader withLevelMinMaxFilter(BiFilter<Integer, Integer> minMaxFilter) {
+        this.levelMinMaxFilter = minMaxFilter;
+        return this;
+    }
+
     public ManifestsReader withPartitionFilter(Predicate predicate) {
         this.partitionFilter = PartitionPredicate.fromPredicate(partitionType, predicate);
         return this;
@@ -160,6 +167,9 @@ public class ManifestsReader {
                     && (specifiedLevel < minLevel || specifiedLevel > maxLevel)) {
                 return false;
             }
+            if (levelMinMaxFilter != null && !levelMinMaxFilter.test(minLevel, maxLevel)) {
+                return false;
+            }
         }
 
         if (partitionFilter == null) {
