@@ -21,7 +21,8 @@ package org.apache.paimon.postpone;
 import org.apache.paimon.CoreOptions;
 import org.apache.paimon.KeyValue;
 import org.apache.paimon.data.BinaryRow;
-import org.apache.paimon.deletionvectors.DeletionVectorsMaintainer;
+import org.apache.paimon.deletionvectors.BucketedDvMaintainer;
+import org.apache.paimon.disk.IOManager;
 import org.apache.paimon.format.FileFormat;
 import org.apache.paimon.format.avro.AvroSchemaConverter;
 import org.apache.paimon.fs.FileIO;
@@ -29,6 +30,8 @@ import org.apache.paimon.io.DataFileMeta;
 import org.apache.paimon.io.KeyValueFileReaderFactory;
 import org.apache.paimon.io.KeyValueFileWriterFactory;
 import org.apache.paimon.mergetree.compact.ConcatRecordReader;
+import org.apache.paimon.mergetree.compact.LookupMergeFunction;
+import org.apache.paimon.mergetree.compact.MergeFunctionFactory;
 import org.apache.paimon.operation.FileStoreScan;
 import org.apache.paimon.operation.FileStoreWrite;
 import org.apache.paimon.operation.MemoryFileStoreWrite;
@@ -71,6 +74,7 @@ public class PostponeBucketFileStoreWrite extends MemoryFileStoreWrite<KeyValue>
     private final KeyValueFileWriterFactory.Builder writerFactoryBuilder;
     private final FileIO fileIO;
     private final FileStorePathFactory pathFactory;
+    private final MergeFunctionFactory<KeyValue> mfFactory;
     private final KeyValueFileReaderFactory.Builder readerFactoryBuilder;
 
     private boolean forceBufferSpill = false;
@@ -83,6 +87,7 @@ public class PostponeBucketFileStoreWrite extends MemoryFileStoreWrite<KeyValue>
             RowType partitionType,
             RowType keyType,
             RowType valueType,
+            MergeFunctionFactory<KeyValue> mfFactory,
             BiFunction<CoreOptions, String, FileStorePathFactory> formatPathFactory,
             KeyValueFileReaderFactory.Builder readerFactoryBuilder,
             SnapshotManager snapshotManager,
@@ -93,6 +98,7 @@ public class PostponeBucketFileStoreWrite extends MemoryFileStoreWrite<KeyValue>
         super(snapshotManager, scan, options, partitionType, null, null, tableName);
         this.fileIO = fileIO;
         this.pathFactory = pathFactory;
+        this.mfFactory = mfFactory;
         this.readerFactoryBuilder = readerFactoryBuilder;
 
         Options newOptions = new Options(options.toMap());
@@ -143,6 +149,15 @@ public class PostponeBucketFileStoreWrite extends MemoryFileStoreWrite<KeyValue>
         withIgnorePreviousFiles(true);
     }
 
+    @Override
+    public PostponeBucketFileStoreWrite withIOManager(IOManager ioManager) {
+        super.withIOManager(ioManager);
+        if (mfFactory instanceof LookupMergeFunction.Factory) {
+            ((LookupMergeFunction.Factory) mfFactory).withIOManager(ioManager);
+        }
+        return this;
+    }
+
     @Override
     protected void forceBufferSpill() throws Exception {
         if (ioManager == null) {
@@ -176,7 +191,7 @@ public class PostponeBucketFileStoreWrite extends MemoryFileStoreWrite<KeyValue>
             long restoredMaxSeqNumber,
             @Nullable CommitIncrement restoreIncrement,
             ExecutorService compactExecutor,
-            @Nullable DeletionVectorsMaintainer deletionVectorsMaintainer) {
+            @Nullable BucketedDvMaintainer deletionVectorsMaintainer) {
         Preconditions.checkArgument(bucket == BucketMode.POSTPONE_BUCKET);
         Preconditions.checkArgument(
                 restoreFiles.isEmpty(),
@@ -189,6 +204,7 @@ public class PostponeBucketFileStoreWrite extends MemoryFileStoreWrite<KeyValue>
                 options.spillCompressOptions(),
                 options.writeBufferSpillDiskSize(),
                 ioManager,
+                mfFactory.create(),
                 writerFactory,
                 files -> newFileRead(partition, bucket, files),
                 forceBufferSpill,
@@ -211,4 +227,17 @@ public class PostponeBucketFileStoreWrite extends MemoryFileStoreWrite<KeyValue>
     protected Function<WriterContainer<KeyValue>, Boolean> createWriterCleanChecker() {
         return createNoConflictAwareWriterCleanChecker();
     }
+
+    public static int getWriteId(String fileName) {
+        try {
+            String[] parts = fileName.split("-s-");
+            return Integer.parseInt(parts[1].substring(0, parts[1].indexOf('-')));
+        } catch (Exception e) {
+            throw new RuntimeException(
+                    "Data file name "
+                            + fileName
+                            + " does not match the pattern. This is unexpected.",
+                    e);
+        }
+    }
 }
