@@ -18,11 +18,12 @@
 
 package org.apache.paimon.table.sink;
 
+import org.apache.paimon.operation.WriteRestore;
+
 /** Inner {@link TableWrite} contains overwrite setter. */
 public interface InnerTableWrite extends StreamTableWrite, BatchTableWrite {
 
-    InnerTableWrite withIgnorePreviousFiles(boolean ignorePreviousFiles);
+    InnerTableWrite withWriteRestore(WriteRestore writeRestore);
 
-    // we detect whether in streaming mode, and do some optimization
-    InnerTableWrite withExecutionMode(boolean isStreamingMode);
+    InnerTableWrite withIgnorePreviousFiles(boolean ignorePreviousFiles);
 }
