@@ -27,16 +27,17 @@ import org.apache.paimon.disk.IOManager;
 import org.apache.paimon.io.BundleRecords;
 import org.apache.paimon.io.DataFileMeta;
 import org.apache.paimon.memory.MemoryPoolFactory;
-import org.apache.paimon.memory.MemorySegmentPool;
 import org.apache.paimon.metrics.MetricRegistry;
 import org.apache.paimon.operation.BundleFileStoreWriter;
 import org.apache.paimon.operation.FileStoreWrite;
 import org.apache.paimon.operation.FileStoreWrite.State;
+import org.apache.paimon.operation.WriteRestore;
 import org.apache.paimon.table.BucketMode;
 import org.apache.paimon.types.DataField;
 import org.apache.paimon.types.RowKind;
 import org.apache.paimon.types.RowType;
 import org.apache.paimon.utils.Restorable;
+import org.apache.paimon.utils.RowKindFilter;
 
 import javax.annotation.Nullable;
 
@@ -58,7 +59,7 @@ public class TableWriteImpl<T> implements InnerTableWrite, Restorable<List<State
     private final KeyAndBucketExtractor<InternalRow> keyAndBucketExtractor;
     private final RecordExtractor<T> recordExtractor;
     @Nullable private final RowKindGenerator rowKindGenerator;
-    private final boolean ignoreDelete;
+    @Nullable private final RowKindFilter rowKindFilter;
 
     private boolean batchCommitted = false;
     private BucketMode bucketMode;
@@ -72,13 +73,13 @@ public class TableWriteImpl<T> implements InnerTableWrite, Restorable<List<State
             KeyAndBucketExtractor<InternalRow> keyAndBucketExtractor,
             RecordExtractor<T> recordExtractor,
             @Nullable RowKindGenerator rowKindGenerator,
-            boolean ignoreDelete) {
+            @Nullable RowKindFilter rowKindFilter) {
         this.rowType = rowType;
         this.write = write;
         this.keyAndBucketExtractor = keyAndBucketExtractor;
         this.recordExtractor = recordExtractor;
         this.rowKindGenerator = rowKindGenerator;
-        this.ignoreDelete = ignoreDelete;
+        this.rowKindFilter = rowKindFilter;
 
         List<String> notNullColumnNames =
                 rowType.getFields().stream()
@@ -89,15 +90,19 @@ public class TableWriteImpl<T> implements InnerTableWrite, Restorable<List<State
         this.defaultValueRow = DefaultValueRow.create(rowType);
     }
 
+    public FileStoreWrite<T> fileStoreWrite() {
+        return write;
+    }
+
     @Override
-    public TableWriteImpl<T> withIgnorePreviousFiles(boolean ignorePreviousFiles) {
-        write.withIgnorePreviousFiles(ignorePreviousFiles);
+    public InnerTableWrite withWriteRestore(WriteRestore writeRestore) {
+        this.write.withWriteRestore(writeRestore);
         return this;
     }
 
     @Override
-    public TableWriteImpl<T> withExecutionMode(boolean isStreamingMode) {
-        write.withExecutionMode(isStreamingMode);
+    public TableWriteImpl<T> withIgnorePreviousFiles(boolean ignorePreviousFiles) {
+        write.withIgnorePreviousFiles(ignorePreviousFiles);
         return this;
     }
 
@@ -108,11 +113,12 @@ public class TableWriteImpl<T> implements InnerTableWrite, Restorable<List<State
     }
 
     @Override
-    public TableWriteImpl<T> withMemoryPool(MemorySegmentPool memoryPool) {
-        write.withMemoryPool(memoryPool);
+    public TableWriteImpl<T> withWriteType(RowType writeType) {
+        write.withWriteType(writeType);
         return this;
     }
 
+    @Override
     public TableWriteImpl<T> withMemoryPoolFactory(MemoryPoolFactory memoryPoolFactory) {
         write.withMemoryPoolFactory(memoryPoolFactory);
         return this;
@@ -164,26 +170,19 @@ public class TableWriteImpl<T> implements InnerTableWrite, Restorable<List<State
 
     @Nullable
     public SinkRecord writeAndReturn(InternalRow row) throws Exception {
-        checkNullability(row);
-        row = wrapDefaultValue(row);
-        RowKind rowKind = RowKindGenerator.getRowKind(rowKindGenerator, row);
-        if (ignoreDelete && rowKind.isRetract()) {
-            return null;
-        }
-        SinkRecord record = toSinkRecord(row);
-        write.write(record.partition(), record.bucket(), recordExtractor.extract(record, rowKind));
-        return record;
+        return writeAndReturn(row, -1);
     }
 
     @Nullable
     public SinkRecord writeAndReturn(InternalRow row, int bucket) throws Exception {
         checkNullability(row);
+        row = wrapDefaultValue(row);
         RowKind rowKind = RowKindGenerator.getRowKind(rowKindGenerator, row);
-        if (ignoreDelete && rowKind.isRetract()) {
+        if (rowKindFilter != null && !rowKindFilter.test(rowKind)) {
             return null;
         }
-        SinkRecord record = toSinkRecord(row, bucket);
-        write.write(record.partition(), bucket, recordExtractor.extract(record, rowKind));
+        SinkRecord record = bucket == -1 ? toSinkRecord(row) : toSinkRecord(row, bucket);
+        write.write(record.partition(), record.bucket(), recordExtractor.extract(record, rowKind));
         return record;
     }
 
