@@ -20,9 +20,10 @@ package org.apache.paimon.table.source;
 
 import org.apache.paimon.annotation.Public;
 import org.apache.paimon.data.InternalRow;
+import org.apache.paimon.partition.PartitionPredicate;
 import org.apache.paimon.predicate.Predicate;
 import org.apache.paimon.predicate.PredicateBuilder;
-import org.apache.paimon.table.Table;
+import org.apache.paimon.predicate.TopN;
 import org.apache.paimon.types.RowType;
 import org.apache.paimon.utils.Filter;
 
@@ -100,6 +101,9 @@ public interface ReadBuilder extends Serializable {
     /** Push partition filter. */
     ReadBuilder withPartitionFilter(Map<String, String> partitionSpec);
 
+    /** Push partition filters. */
+    ReadBuilder withPartitionFilter(PartitionPredicate partitionPredicate);
+
     ReadBuilder withBucket(int bucket);
 
     /**
@@ -115,7 +119,7 @@ public interface ReadBuilder extends Serializable {
     /**
      * Push read row type to the reader, support nested row pruning.
      *
-     * @param readType read row type, can be a pruned type from {@link Table#rowType()}
+     * @param readType read row type
      * @since 1.0.0
      */
     ReadBuilder withReadType(RowType readType);
@@ -129,6 +133,12 @@ public interface ReadBuilder extends Serializable {
     /** the row number pushed down. */
     ReadBuilder withLimit(int limit);
 
+    /**
+     * Push TopN filter. Will filter the data as much as possible, but it is not guaranteed that it
+     * is a complete filter.
+     */
+    ReadBuilder withTopN(TopN topN);
+
     /**
      * Specify the shard to be read, and allocate sharded files to read records. Note that this
      * method cannot be used simultaneously with {@link #withBucketFilter(Filter)}.
