@@ -274,6 +274,9 @@ public class FilesTable implements ReadonlyTable {
                             return levelPredicate.test(
                                     GenericRow.of(null, null, null, null, null, level));
                         });
+            } else {
+                // avoid that batchScanSkipLevel0 is true
+                scan.withLevelFilter(level -> true);
             }
             return scan.plan();
         }
@@ -464,7 +467,9 @@ public class FilesTable implements ReadonlyTable {
             lazyNullValueCounts = new TreeMap<>();
             lazyLowerValueBounds = new TreeMap<>();
             lazyUpperValueBounds = new TreeMap<>();
-            for (int i = 0; i < min.getFieldCount(); i++) {
+            int length =
+                    Math.min(min.getFieldCount(), simpleStatsEvolutions.tableDataFields().size());
+            for (int i = 0; i < length; i++) {
                 DataField field = simpleStatsEvolutions.tableDataFields().get(i);
                 String name = field.name();
                 DataType type = field.type();
