@@ -22,6 +22,7 @@ import org.apache.paimon.data.InternalRow;
 import org.apache.paimon.format.FormatReaderFactory;
 import org.apache.paimon.format.FormatWriter;
 import org.apache.paimon.format.FormatWriterFactory;
+import org.apache.paimon.format.SupportsDirectWrite;
 import org.apache.paimon.fs.FileIO;
 import org.apache.paimon.fs.Path;
 import org.apache.paimon.fs.PositionOutputStream;
@@ -40,7 +41,7 @@ import java.util.List;
 import static org.apache.paimon.utils.FileUtils.checkExists;
 
 /** A file which contains several {@link T}s, provides read and write. */
-public class ObjectsFile<T> implements SimpleFileReader<T> {
+public abstract class ObjectsFile<T> implements SimpleFileReader<T> {
 
     protected final FileIO fileIO;
     protected final ObjectSerializer<T> serializer;
@@ -49,7 +50,7 @@ public class ObjectsFile<T> implements SimpleFileReader<T> {
     protected final String compression;
     protected final PathFactory pathFactory;
 
-    @Nullable private final ObjectsCache<Path, T> cache;
+    @Nullable protected final ObjectsCache<Path, T, ?> cache;
 
     public ObjectsFile(
             FileIO fileIO,
@@ -66,15 +67,12 @@ public class ObjectsFile<T> implements SimpleFileReader<T> {
         this.writerFactory = writerFactory;
         this.compression = compression;
         this.pathFactory = pathFactory;
-        this.cache =
-                cache == null
-                        ? null
-                        : new ObjectsCache<>(
-                                cache,
-                                serializer,
-                                formatType,
-                                this::fileSize,
-                                this::createIterator);
+        this.cache = cache == null ? null : createCache(cache, formatType);
+    }
+
+    protected ObjectsCache<Path, T, ?> createCache(SegmentsCache<Path> cache, RowType formatType) {
+        return new SimpleObjectsCache<>(
+                cache, serializer, formatType, this::fileSize, this::createIterator);
     }
 
     public ObjectsFile<T> withCacheMetrics(@Nullable CacheMetrics cacheMetrics) {
@@ -102,8 +100,7 @@ public class ObjectsFile<T> implements SimpleFileReader<T> {
     }
 
     public List<T> read(String fileName, @Nullable Long fileSize) {
-        return read(
-                fileName, fileSize, Filter.alwaysTrue(), Filter.alwaysTrue(), Filter.alwaysTrue());
+        return read(fileName, fileSize, Filter.alwaysTrue(), Filter.alwaysTrue());
     }
 
     public List<T> readWithIOException(String fileName) throws IOException {
@@ -112,8 +109,7 @@ public class ObjectsFile<T> implements SimpleFileReader<T> {
 
     public List<T> readWithIOException(String fileName, @Nullable Long fileSize)
             throws IOException {
-        return readWithIOException(
-                fileName, fileSize, Filter.alwaysTrue(), Filter.alwaysTrue(), Filter.alwaysTrue());
+        return readWithIOException(fileName, fileSize, Filter.alwaysTrue(), Filter.alwaysTrue());
     }
 
     public boolean exists(String fileName) {
@@ -127,11 +123,10 @@ public class ObjectsFile<T> implements SimpleFileReader<T> {
     public List<T> read(
             String fileName,
             @Nullable Long fileSize,
-            Filter<InternalRow> loadFilter,
             Filter<InternalRow> readFilter,
             Filter<T> readTFilter) {
         try {
-            return readWithIOException(fileName, fileSize, loadFilter, readFilter, readTFilter);
+            return readWithIOException(fileName, fileSize, readFilter, readTFilter);
         } catch (IOException e) {
             throw new RuntimeException("Failed to read " + fileName, e);
         }
@@ -140,13 +135,12 @@ public class ObjectsFile<T> implements SimpleFileReader<T> {
     private List<T> readWithIOException(
             String fileName,
             @Nullable Long fileSize,
-            Filter<InternalRow> loadFilter,
             Filter<InternalRow> readFilter,
             Filter<T> readTFilter)
             throws IOException {
         Path path = pathFactory.toPath(fileName);
         if (cache != null) {
-            return cache.read(path, fileSize, loadFilter, readFilter, readTFilter);
+            return cache.read(path, fileSize, new ObjectsCache.Filters<>(readFilter, readTFilter));
         }
 
         return readFromIterator(
@@ -160,19 +154,29 @@ public class ObjectsFile<T> implements SimpleFileReader<T> {
     protected Pair<String, Long> writeWithoutRolling(Iterator<T> records) {
         Path path = pathFactory.newPath();
         try {
-            PositionOutputStream out = fileIO.newOutputStream(path, false);
-            long pos;
-            try {
-                try (FormatWriter writer = writerFactory.create(out, compression)) {
+            if (writerFactory instanceof SupportsDirectWrite) {
+                try (FormatWriter writer =
+                        ((SupportsDirectWrite) writerFactory).create(fileIO, path, compression)) {
                     while (records.hasNext()) {
                         writer.addElement(serializer.toRow(records.next()));
                     }
                 }
-            } finally {
-                pos = out.getPos();
-                out.close();
+                return Pair.of(path.getName(), fileIO.getFileSize(path));
+            } else {
+                PositionOutputStream out = fileIO.newOutputStream(path, false);
+                long pos;
+                try {
+                    try (FormatWriter writer = writerFactory.create(out, compression)) {
+                        while (records.hasNext()) {
+                            writer.addElement(serializer.toRow(records.next()));
+                        }
+                    }
+                } finally {
+                    pos = out.getPos();
+                    out.close();
+                }
+                return Pair.of(path.getName(), pos);
             }
-            return Pair.of(path.getName(), pos);
         } catch (Throwable e) {
             fileIO.deleteQuietly(path);
             throw new RuntimeException(
@@ -180,13 +184,13 @@ public class ObjectsFile<T> implements SimpleFileReader<T> {
         }
     }
 
-    private CloseableIterator<InternalRow> createIterator(Path file, @Nullable Long fileSize)
+    public CloseableIterator<InternalRow> createIterator(Path file, @Nullable Long fileSize)
             throws IOException {
         return FileUtils.createFormatReader(fileIO, readerFactory, file, fileSize)
                 .toCloseableIterator();
     }
 
-    private long fileSize(Path file) throws IOException {
+    public long fileSize(Path file) throws IOException {
         try {
             return fileIO.getFileSize(file);
         } catch (IOException e) {
