@@ -167,7 +167,11 @@ public class SnapshotManager implements Serializable {
     public @Nullable Snapshot latestSnapshot() {
         if (snapshotLoader != null) {
             try {
-                return snapshotLoader.load().orElse(null);
+                Snapshot snapshot = snapshotLoader.load().orElse(null);
+                if (snapshot != null && cache != null) {
+                    cache.put(snapshotPath(snapshot.id()), snapshot);
+                }
+                return snapshot;
             } catch (UnsupportedOperationException ignored) {
             } catch (IOException e) {
                 throw new UncheckedIOException(e);
