@@ -18,15 +18,19 @@
 
 package org.apache.paimon.utils;
 
+import org.apache.paimon.CoreOptions;
 import org.apache.paimon.Snapshot;
 import org.apache.paimon.annotation.VisibleForTesting;
 import org.apache.paimon.fs.FileIO;
 import org.apache.paimon.fs.FileStatus;
 import org.apache.paimon.fs.Path;
+import org.apache.paimon.iceberg.IcebergCommitCallback;
 import org.apache.paimon.manifest.ExpireFileEntry;
 import org.apache.paimon.operation.TagDeletion;
 import org.apache.paimon.table.sink.TagCallback;
 import org.apache.paimon.tag.Tag;
+import org.apache.paimon.tag.TagPeriodHandler;
+import org.apache.paimon.tag.TagTimeExtractor;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -48,7 +52,7 @@ import java.util.TreeMap;
 import java.util.function.Predicate;
 import java.util.stream.Collectors;
 
-import static org.apache.paimon.utils.BranchManager.DEFAULT_MAIN_BRANCH;
+import static org.apache.paimon.catalog.Identifier.DEFAULT_MAIN_BRANCH;
 import static org.apache.paimon.utils.BranchManager.branchPath;
 import static org.apache.paimon.utils.FileUtils.listVersionedFileStatus;
 import static org.apache.paimon.utils.Preconditions.checkArgument;
@@ -63,20 +67,44 @@ public class TagManager {
     private final FileIO fileIO;
     private final Path tablePath;
     private final String branch;
+    @Nullable private final TagPeriodHandler tagPeriodHandler;
 
     public TagManager(FileIO fileIO, Path tablePath) {
-        this(fileIO, tablePath, DEFAULT_MAIN_BRANCH);
+        this(fileIO, tablePath, DEFAULT_MAIN_BRANCH, (TagPeriodHandler) null);
+    }
+
+    public TagManager(FileIO fileIO, Path tablePath, String branch) {
+        this(fileIO, tablePath, branch, (TagPeriodHandler) null);
     }
 
     /** Specify the default branch for data writing. */
-    public TagManager(FileIO fileIO, Path tablePath, String branch) {
+    public TagManager(FileIO fileIO, Path tablePath, String branch, CoreOptions options) {
+        this(fileIO, tablePath, branch, createIfNecessary(options));
+    }
+
+    @Nullable
+    private static TagPeriodHandler createIfNecessary(CoreOptions options) {
+        return TagTimeExtractor.createForAutoTag(options) == null
+                ? null
+                : TagPeriodHandler.create(options);
+    }
+
+    private TagManager(
+            FileIO fileIO,
+            Path tablePath,
+            String branch,
+            @Nullable TagPeriodHandler tagPeriodHandler) {
         this.fileIO = fileIO;
         this.tablePath = tablePath;
         this.branch = BranchManager.normalizeBranch(branch);
+        this.tagPeriodHandler = tagPeriodHandler;
     }
 
+    // TODO: Current usage of this method only use the tag directory of new branch.
+    // If we will use the new branch TagManager to create tag, we need to pass TagPeriodHandler
+    // according to branch options.
     public TagManager copyWithBranch(String branchName) {
-        return new TagManager(fileIO, tablePath, branchName);
+        return new TagManager(fileIO, tablePath, branchName, (TagPeriodHandler) null);
     }
 
     /** Return the root Directory of tags. */
@@ -112,10 +140,17 @@ public class TagManager {
     }
 
     /** Replace a tag from given snapshot and save it in the storage. */
-    public void replaceTag(Snapshot snapshot, String tagName, Duration timeRetained) {
+    public void replaceTag(
+            Snapshot snapshot, String tagName, Duration timeRetained, List<TagCallback> callbacks) {
         checkArgument(!StringUtils.isNullOrWhitespaceOnly(tagName), "Tag name shouldn't be blank.");
         checkArgument(tagExists(tagName), "Tag '%s' doesn't exist.", tagName);
-        createOrReplaceTag(snapshot, tagName, timeRetained, null);
+        createOrReplaceTag(
+                snapshot,
+                tagName,
+                timeRetained,
+                callbacks.stream()
+                        .filter(callback -> callback instanceof IcebergCommitCallback)
+                        .collect(Collectors.toList()));
     }
 
     private void createOrReplaceTag(
@@ -123,6 +158,8 @@ public class TagManager {
             String tagName,
             @Nullable Duration timeRetained,
             @Nullable List<TagCallback> callbacks) {
+        validateNoAutoTag(tagName, snapshot);
+
         // When timeRetained is not defined, please do not write the tagCreatorTime field, as this
         // will cause older versions (<= 0.7) of readers to be unable to read this tag.
         // When timeRetained is defined, it is fine, because timeRetained is the new feature.
@@ -146,7 +183,7 @@ public class TagManager {
 
         if (callbacks != null) {
             try {
-                callbacks.forEach(callback -> callback.notifyCreation(tagName));
+                callbacks.forEach(callback -> callback.notifyCreation(tagName, snapshot.id()));
             } finally {
                 for (TagCallback tagCallback : callbacks) {
                     IOUtils.closeQuietly(tagCallback);
@@ -429,4 +466,18 @@ public class TagManager {
                         "Didn't find tag with snapshot id '%s'. This is unexpected.",
                         taggedSnapshot.id()));
     }
+
+    private void validateNoAutoTag(String tagName, Snapshot snapshot) {
+        if (tagPeriodHandler == null || !tagPeriodHandler.isAutoTag(tagName)) {
+            return;
+        }
+
+        List<String> autoTags = tags(tagPeriodHandler::isAutoTag).get(snapshot);
+        if (autoTags != null) {
+            throw new RuntimeException(
+                    String.format(
+                            "Snapshot %s is already auto-tagged with %s.",
+                            snapshot.id(), autoTags));
+        }
+    }
 }
