@@ -26,11 +26,11 @@ import org.apache.paimon.fs.FileIO;
 import org.apache.paimon.fs.local.LocalFileIO;
 import org.apache.paimon.index.DeletionVectorMeta;
 import org.apache.paimon.index.IndexFileMeta;
+import org.apache.paimon.index.IndexPathFactory;
 import org.apache.paimon.manifest.FileKind;
 import org.apache.paimon.manifest.IndexManifestEntry;
 import org.apache.paimon.table.sink.CommitMessageImpl;
 import org.apache.paimon.table.source.DeletionFile;
-import org.apache.paimon.utils.PathFactory;
 
 import org.junit.jupiter.api.io.TempDir;
 import org.junit.jupiter.params.ParameterizedTest;
@@ -66,14 +66,15 @@ class AppendDeletionFileMaintainerTest {
                         Collections.singletonMap("f3", Arrays.asList(1, 2, 3)));
         store.commit(commitMessage1, commitMessage2);
 
-        PathFactory indexPathFactory = store.pathFactory().indexFileFactory();
+        IndexPathFactory indexPathFactory =
+                store.pathFactory().indexFileFactory(BinaryRow.EMPTY_ROW, 0);
         Map<String, DeletionFile> dataFileToDeletionFiles = new HashMap<>();
         dataFileToDeletionFiles.putAll(
                 createDeletionFileMapFromIndexFileMetas(
-                        indexPathFactory, commitMessage1.indexIncrement().newIndexFiles()));
+                        indexPathFactory, commitMessage1.newFilesIncrement().newIndexFiles()));
         dataFileToDeletionFiles.putAll(
                 createDeletionFileMapFromIndexFileMetas(
-                        indexPathFactory, commitMessage2.indexIncrement().newIndexFiles()));
+                        indexPathFactory, commitMessage2.newFilesIncrement().newIndexFiles()));
 
         AppendDeleteFileMaintainer dvIFMaintainer =
                 store.createDVIFMaintainer(BinaryRow.EMPTY_ROW, dataFileToDeletionFiles);
@@ -99,7 +100,7 @@ class AppendDeletionFileMaintainerTest {
         assertThat(res.size()).isEqualTo(3);
         IndexManifestEntry entry =
                 res.stream().filter(file -> file.kind() == FileKind.ADD).findAny().get();
-        assertThat(entry.indexFile().deletionVectorMetas().containsKey("f2")).isTrue();
+        assertThat(entry.indexFile().dvRanges().containsKey("f2")).isTrue();
         entry =
                 res.stream()
                         .filter(file -> file.kind() == FileKind.DELETE)
@@ -107,7 +108,7 @@ class AppendDeletionFileMaintainerTest {
                         .findAny()
                         .get();
         assertThat(entry.indexFile())
-                .isEqualTo(commitMessage1.indexIncrement().newIndexFiles().get(0));
+                .isEqualTo(commitMessage1.newFilesIncrement().newIndexFiles().get(0));
         entry =
                 res.stream()
                         .filter(file -> file.kind() == FileKind.DELETE)
@@ -115,19 +116,19 @@ class AppendDeletionFileMaintainerTest {
                         .findAny()
                         .get();
         assertThat(entry.indexFile())
-                .isEqualTo(commitMessage2.indexIncrement().newIndexFiles().get(0));
+                .isEqualTo(commitMessage2.newFilesIncrement().newIndexFiles().get(0));
     }
 
     private Map<String, DeletionFile> createDeletionFileMapFromIndexFileMetas(
-            PathFactory indexPathFactory, List<IndexFileMeta> fileMetas) {
+            IndexPathFactory indexPathFactory, List<IndexFileMeta> fileMetas) {
         Map<String, DeletionFile> dataFileToDeletionFiles = new HashMap<>();
         for (IndexFileMeta indexFileMeta : fileMetas) {
             for (Map.Entry<String, DeletionVectorMeta> dvMeta :
-                    indexFileMeta.deletionVectorMetas().entrySet()) {
+                    indexFileMeta.dvRanges().entrySet()) {
                 dataFileToDeletionFiles.put(
                         dvMeta.getKey(),
                         new DeletionFile(
-                                indexPathFactory.toPath(indexFileMeta.fileName()).toString(),
+                                indexPathFactory.toPath(indexFileMeta).toString(),
                                 dvMeta.getValue().offset(),
                                 dvMeta.getValue().length(),
                                 dvMeta.getValue().cardinality()));
