@@ -22,7 +22,6 @@ import org.apache.paimon.catalog.PrimaryKeyTableTestBase;
 import org.apache.paimon.data.BinaryRow;
 import org.apache.paimon.io.CompactIncrement;
 import org.apache.paimon.io.DataIncrement;
-import org.apache.paimon.io.IndexIncrement;
 import org.apache.paimon.table.sink.CommitMessage;
 import org.apache.paimon.table.sink.CommitMessageImpl;
 import org.apache.paimon.table.sink.StreamTableCommit;
@@ -33,6 +32,7 @@ import org.junit.jupiter.api.Test;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.ValueSource;
 
+import java.io.IOException;
 import java.util.Arrays;
 import java.util.Collections;
 
@@ -215,21 +215,30 @@ public class HashBucketAssignerTest extends PrimaryKeyTableTestBase {
                 bucket,
                 totalBuckets,
                 new DataIncrement(
-                        Collections.emptyList(), Collections.emptyList(), Collections.emptyList()),
+                        Collections.emptyList(),
+                        Collections.emptyList(),
+                        Collections.emptyList(),
+                        Collections.singletonList(file),
+                        Collections.emptyList()),
                 new CompactIncrement(
-                        Collections.emptyList(), Collections.emptyList(), Collections.emptyList()),
-                new IndexIncrement(Collections.singletonList(file)));
+                        Collections.emptyList(), Collections.emptyList(), Collections.emptyList()));
     }
 
     @Test
-    public void testAssignRestore() {
-        IndexFileMeta bucket0 = fileHandler.writeHashIndex(new int[] {2, 5});
-        IndexFileMeta bucket2 = fileHandler.writeHashIndex(new int[] {4, 7});
+    public void testAssignRestore() throws IOException {
         commit.commit(
                 0,
                 Arrays.asList(
-                        createCommitMessage(row(1), 0, 3, bucket0),
-                        createCommitMessage(row(1), 2, 3, bucket2)));
+                        createCommitMessage(
+                                row(1),
+                                0,
+                                3,
+                                fileHandler.hashIndex(row(1), 0).write(new int[] {2, 5})),
+                        createCommitMessage(
+                                row(1),
+                                2,
+                                3,
+                                fileHandler.hashIndex(row(1), 2).write(new int[] {4, 7}))));
 
         HashBucketAssigner assigner0 = createAssigner(3, 3, 0);
         HashBucketAssigner assigner2 = createAssigner(3, 3, 2);
@@ -248,14 +257,20 @@ public class HashBucketAssignerTest extends PrimaryKeyTableTestBase {
     }
 
     @Test
-    public void testAssignRestoreWithUpperBound() {
-        IndexFileMeta bucket0 = fileHandler.writeHashIndex(new int[] {2, 5});
-        IndexFileMeta bucket2 = fileHandler.writeHashIndex(new int[] {4, 7});
+    public void testAssignRestoreWithUpperBound() throws IOException {
         commit.commit(
                 0,
                 Arrays.asList(
-                        createCommitMessage(row(1), 0, 3, bucket0),
-                        createCommitMessage(row(1), 2, 3, bucket2)));
+                        createCommitMessage(
+                                row(1),
+                                0,
+                                3,
+                                fileHandler.hashIndex(row(1), 0).write(new int[] {2, 5})),
+                        createCommitMessage(
+                                row(1),
+                                2,
+                                3,
+                                fileHandler.hashIndex(row(1), 2).write(new int[] {4, 7}))));
 
         HashBucketAssigner assigner0 = createAssigner(3, 3, 0, 1);
         HashBucketAssigner assigner2 = createAssigner(3, 3, 2, 1);
@@ -303,7 +318,7 @@ public class HashBucketAssignerTest extends PrimaryKeyTableTestBase {
     }
 
     @Test
-    public void testIndexEliminate() {
+    public void testIndexEliminate() throws IOException {
         HashBucketAssigner assigner = createAssigner(1, 1, 0);
 
         // checkpoint 0
@@ -314,9 +329,16 @@ public class HashBucketAssignerTest extends PrimaryKeyTableTestBase {
                 0,
                 Arrays.asList(
                         createCommitMessage(
-                                row(1), 0, 1, fileHandler.writeHashIndex(new int[] {0})),
+                                row(1),
+                                0,
+                                1,
+                                fileHandler.hashIndex(row(1), 0).write(new int[] {0})),
                         createCommitMessage(
-                                row(2), 0, 1, fileHandler.writeHashIndex(new int[] {0}))));
+                                row(2),
+                                0,
+                                1,
+                                fileHandler.hashIndex(row(2), 0).write(new int[] {0}))));
+
         assertThat(assigner.currentPartitions()).containsExactlyInAnyOrder(row(1), row(2));
 
         // checkpoint 1, but no commit
@@ -333,7 +355,11 @@ public class HashBucketAssignerTest extends PrimaryKeyTableTestBase {
                 1,
                 Collections.singletonList(
                         createCommitMessage(
-                                row(1), 0, 1, fileHandler.writeHashIndex(new int[] {1}))));
+                                row(1),
+                                0,
+                                1,
+                                fileHandler.hashIndex(row(1), 0).write(new int[] {1}))));
+
         assigner.prepareCommit(3);
         assertThat(assigner.currentPartitions()).isEmpty();
     }
