@@ -266,7 +266,7 @@ public class ExpireSnapshotsTest {
         // create DataFileMeta and ManifestEntry
         List<String> extraFiles = Arrays.asList("extra1", "extra2");
         DataFileMeta dataFile =
-                new DataFileMeta(
+                DataFileMeta.create(
                         "myDataFile",
                         1,
                         1,
@@ -284,9 +284,11 @@ public class ExpireSnapshotsTest {
                         null,
                         FileSource.APPEND,
                         null,
+                        null,
+                        null,
                         null);
-        ManifestEntry add = new ManifestEntry(FileKind.ADD, partition, 0, 1, dataFile);
-        ManifestEntry delete = new ManifestEntry(FileKind.DELETE, partition, 0, 1, dataFile);
+        ManifestEntry add = ManifestEntry.create(FileKind.ADD, partition, 0, 1, dataFile);
+        ManifestEntry delete = ManifestEntry.create(FileKind.DELETE, partition, 0, 1, dataFile);
 
         // expire
         expire.snapshotDeletion()
@@ -327,7 +329,7 @@ public class ExpireSnapshotsTest {
         // create DataFileMeta and ManifestEntry
         List<String> extraFiles = Arrays.asList("extra1", "extra2");
         DataFileMeta dataFile =
-                new DataFileMeta(
+                DataFileMeta.create(
                         fileName,
                         1,
                         1,
@@ -345,9 +347,11 @@ public class ExpireSnapshotsTest {
                         null,
                         FileSource.APPEND,
                         null,
-                        myDataFile.toString());
-        ManifestEntry add = new ManifestEntry(FileKind.ADD, partition, 0, 1, dataFile);
-        ManifestEntry delete = new ManifestEntry(FileKind.DELETE, partition, 0, 1, dataFile);
+                        myDataFile.toString(),
+                        null,
+                        null);
+        ManifestEntry add = ManifestEntry.create(FileKind.ADD, partition, 0, 1, dataFile);
+        ManifestEntry delete = ManifestEntry.create(FileKind.DELETE, partition, 0, 1, dataFile);
 
         // expire
         expire.snapshotDeletion()
