@@ -19,7 +19,6 @@
 package org.apache.paimon.rest;
 
 import org.apache.paimon.rest.auth.AuthProvider;
-import org.apache.paimon.rest.auth.AuthSession;
 import org.apache.paimon.rest.auth.BearTokenAuthProvider;
 import org.apache.paimon.rest.auth.RESTAuthFunction;
 import org.apache.paimon.rest.auth.RESTAuthParameter;
@@ -28,33 +27,21 @@ import org.apache.paimon.rest.responses.ErrorResponse;
 
 import org.apache.paimon.shade.guava30.com.google.common.collect.ImmutableMap;
 
-import okhttp3.Headers;
-import okhttp3.Interceptor;
-import okhttp3.OkHttpClient;
-import okhttp3.Request;
-import okhttp3.Response;
 import org.junit.After;
 import org.junit.Before;
 import org.junit.Test;
+import org.junit.jupiter.api.Assertions;
 
 import java.io.IOException;
-import java.time.Duration;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Map;
-import java.util.concurrent.atomic.AtomicInteger;
 import java.util.stream.Collectors;
 
-import static okhttp3.ConnectionSpec.CLEARTEXT;
-import static okhttp3.ConnectionSpec.COMPATIBLE_TLS;
-import static okhttp3.ConnectionSpec.MODERN_TLS;
 import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertThrows;
-import static org.mockito.ArgumentMatchers.any;
-import static org.mockito.Mockito.doAnswer;
-import static org.mockito.Mockito.spy;
 
 /** Test for {@link HttpClient}. */
 public class HttpClientTest {
@@ -85,9 +72,8 @@ public class HttpClientTest {
         httpClient = new HttpClient(server.getBaseUrl());
         httpClient.setErrorHandler(errorHandler);
         AuthProvider authProvider = new BearTokenAuthProvider(TOKEN);
-        AuthSession authSession = new AuthSession(authProvider);
         headers = new HashMap<>();
-        restAuthFunction = new RESTAuthFunction(headers, authSession);
+        restAuthFunction = new RESTAuthFunction(headers, authProvider);
     }
 
     @After
@@ -95,6 +81,13 @@ public class HttpClientTest {
         server.stop();
     }
 
+    @Test
+    public void testServerUriSchema() {
+        assertEquals("http://localhost", (new HttpClient("localhost")).uri());
+        assertEquals("http://localhost", (new HttpClient("http://localhost")).uri());
+        assertEquals("https://localhost", (new HttpClient("https://localhost")).uri());
+    }
+
     @Test
     public void testGetSuccess() {
         server.enqueueResponse(mockResponseDataStr, 200);
@@ -172,7 +165,8 @@ public class HttpClientTest {
     @Test
     public void testDeleteWithDataSuccess() {
         server.enqueueResponse(mockResponseDataStr, 200);
-        assertDoesNotThrow(() -> httpClient.delete(MOCK_PATH, mockResponseData, restAuthFunction));
+        Assertions.assertDoesNotThrow(
+                () -> httpClient.delete(MOCK_PATH, mockResponseData, restAuthFunction));
     }
 
     @Test
@@ -188,7 +182,8 @@ public class HttpClientTest {
         HttpClient httpClient = new HttpClient(server.getBaseUrl());
         server.enqueueResponse(mockResponseDataStr, 429);
         server.enqueueResponse(mockResponseDataStr, 200);
-        assertDoesNotThrow(() -> httpClient.get(MOCK_PATH, MockRESTData.class, restAuthFunction));
+        Assertions.assertDoesNotThrow(
+                () -> httpClient.get(MOCK_PATH, MockRESTData.class, restAuthFunction));
     }
 
     @Test
@@ -201,52 +196,16 @@ public class HttpClientTest {
                         "GET",
                         "");
         String url =
-                HttpClient.getRequestUrl(
-                        "http://a.b.c:8080",
-                        "/api/v1/tables/my_table$schemas",
-                        ImmutableMap.of("pageToken", "dt=20230101"));
+                (new HttpClient("http://a.b.c:8080"))
+                        .getRequestUrl(
+                                "/api/v1/tables/my_table$schemas",
+                                ImmutableMap.of("pageToken", "dt=20230101"));
         assertEquals(
                 "http://a.b.c:8080/api/v1/tables/my_table$schemas?pageToken=dt%3D20230101", url);
         Map<String, String> queryParameters = getParameters(url);
         assertEquals(restAuthParameter.parameters().get(queryKey), queryParameters.get(queryKey));
     }
 
-    @Test
-    public void testLoggingInterceptorWithRetry() throws IOException {
-        AtomicInteger retryCount = new AtomicInteger(0);
-        LoggingInterceptor loggingInterceptor = spy(new LoggingInterceptor());
-        doAnswer(
-                        invocation -> {
-                            retryCount.incrementAndGet();
-                            Object argument = invocation.getArguments()[0];
-                            Interceptor.Chain chain = (Interceptor.Chain) argument;
-                            return chain.proceed(chain.request());
-                        })
-                .when(loggingInterceptor)
-                .intercept(any());
-        OkHttpClient okHttpClient =
-                new OkHttpClient.Builder()
-                        .retryOnConnectionFailure(true)
-                        .connectionSpecs(Arrays.asList(MODERN_TLS, COMPATIBLE_TLS, CLEARTEXT))
-                        .addInterceptor(new ExponentialHttpRetryInterceptor(5))
-                        .addInterceptor(loggingInterceptor)
-                        .connectTimeout(Duration.ofMinutes(3))
-                        .readTimeout(Duration.ofMinutes(2))
-                        .build();
-        server.enqueueResponse(mockResponseDataStr, 429);
-        server.enqueueResponse(mockResponseDataStr, 429);
-        server.enqueueResponse(mockResponseDataStr, 200);
-        Request request =
-                new Request.Builder()
-                        .url(HttpClient.getRequestUrl(server.getBaseUrl(), MOCK_PATH, null))
-                        .get()
-                        .headers(Headers.of())
-                        .build();
-        Response response = okHttpClient.newCall(request).execute();
-        assertEquals(200, response.code());
-        assertEquals(3, retryCount.get());
-    }
-
     private Map<String, String> getParameters(String path) {
         String[] paths = path.split("\\?");
         if (paths.length == 1) {
