@@ -38,7 +38,6 @@ import org.apache.paimon.fs.local.LocalFileIO;
 import org.apache.paimon.io.DataFileMeta;
 import org.apache.paimon.mergetree.compact.ConcatRecordReader;
 import org.apache.paimon.operation.FileStoreTestUtils;
-import org.apache.paimon.options.MemorySize;
 import org.apache.paimon.options.Options;
 import org.apache.paimon.predicate.PredicateBuilder;
 import org.apache.paimon.reader.ReaderSupplier;
@@ -117,6 +116,7 @@ import static org.apache.paimon.format.FileFormat.fileFormat;
 import static org.apache.paimon.testutils.assertj.PaimonAssertions.anyCauseMatches;
 import static org.apache.paimon.utils.HintFileUtils.EARLIEST;
 import static org.apache.paimon.utils.HintFileUtils.LATEST;
+import static org.apache.paimon.utils.NextSnapshotFetcher.RANGE_CHECK_INTERVAL;
 import static org.apache.paimon.utils.Preconditions.checkNotNull;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.assertj.core.api.Assertions.assertThatExceptionOfType;
@@ -534,46 +534,6 @@ public abstract class SimpleTableTestBase {
         commit.close();
     }
 
-    @Test
-    public void testManifestCache() throws Exception {
-        FileStoreTable table =
-                createFileStoreTable(
-                        conf ->
-                                conf.set(
-                                        CoreOptions.WRITE_MANIFEST_CACHE,
-                                        MemorySize.ofMebiBytes(1)));
-        StreamTableWrite write = table.newWrite(commitUser);
-        StreamTableCommit commit = table.newCommit(commitUser);
-
-        // lots of commits, produce lots of manifest
-        List<String> expected = new ArrayList<>();
-        int cnt = 50;
-        for (int i = 0; i < cnt; i++) {
-            write.write(rowData(i, i, (long) i));
-            commit.commit(i, write.prepareCommit(false, i));
-            expected.add(
-                    String.format("%s|%s|%s|binary|varbinary|mapKey:mapVal|multiset", i, i, i));
-        }
-        write.close();
-
-        // create new write and reload manifests
-        write = table.newWrite(commitUser);
-        for (int i = 0; i < cnt; i++) {
-            write.write(rowData(i, i + 1, (long) i + 1));
-            expected.add(
-                    String.format(
-                            "%s|%s|%s|binary|varbinary|mapKey:mapVal|multiset", i, i + 1, i + 1));
-        }
-        commit.commit(cnt, write.prepareCommit(false, cnt));
-        commit.close();
-
-        // check result
-        List<String> result =
-                getResult(table.newRead(), table.newScan().plan().splits(), BATCH_ROW_TO_STRING);
-        assertThat(result.size()).isEqualTo(expected.size());
-        assertThat(result).containsExactlyInAnyOrderElementsOf(expected);
-    }
-
     @Test
     public void testWriteWithoutCompactionAndExpiration() throws Exception {
         FileStoreTable table =
@@ -763,7 +723,12 @@ public abstract class SimpleTableTestBase {
         commit.commit(9, write.prepareCommit(true, 9));
 
         StreamTableScan finalScan = scan;
-        assertThatThrownBy(finalScan::plan)
+        assertThatThrownBy(
+                        () -> {
+                            for (int i = 0; i < RANGE_CHECK_INTERVAL; i++) {
+                                finalScan.plan();
+                            }
+                        })
                 .satisfies(
                         anyCauseMatches(
                                 OutOfRangeException.class, "The snapshot with id 5 has expired."));
@@ -1380,7 +1345,7 @@ public abstract class SimpleTableTestBase {
         options.put(SNAPSHOT_EXPIRE_LIMIT.key(), "2");
 
         TableCommitImpl commit = table.copy(options).newCommit(commitUser);
-        ExecutorService executor = commit.getExpireMainExecutor();
+        ExecutorService executor = commit.getMaintainExecutor();
         CountDownLatch before = new CountDownLatch(1);
         CountDownLatch after = new CountDownLatch(1);
 
