@@ -21,5 +21,10 @@ package org.apache.paimon.flink;
 import org.apache.paimon.flink.action.RemoveOrphanFilesAction;
 import org.apache.paimon.flink.action.RemoveOrphanFilesActionITCaseBase;
 
-/** IT cases base for {@link RemoveOrphanFilesAction} in Flink 1.19. */
-public class RemoveOrphanFilesActionITCase extends RemoveOrphanFilesActionITCaseBase {}
+/** IT cases base for {@link RemoveOrphanFilesAction} in Flink 1.18. */
+public class RemoveOrphanFilesActionITCase extends RemoveOrphanFilesActionITCaseBase {
+
+    protected boolean supportNamedArgument() {
+        return false;
+    }
+}
