@@ -68,6 +68,9 @@ public abstract class SynchronizationActionBase extends ActionBase {
 
     protected Map<String, String> tableConfig = new HashMap<>();
     protected TypeMapping typeMapping = TypeMapping.defaultMapping();
+    // this is to specify if we should use primary keys from source
+    // in paimon schema if pkeys are not specified in action command
+    protected boolean syncPKeysFromSourceSchema = true;
     protected CdcMetadataConverter[] metadataConverters = new CdcMetadataConverter[] {};
 
     public SynchronizationActionBase(
@@ -102,6 +105,11 @@ public abstract class SynchronizationActionBase extends ActionBase {
         return this;
     }
 
+    public SynchronizationActionBase syncPKeysFromSourceSchema(boolean flag) {
+        this.syncPKeysFromSourceSchema = flag;
+        return this;
+    }
+
     @VisibleForTesting
     public Map<String, String> tableConfig() {
         return tableConfig;
