@@ -66,8 +66,13 @@ public class KafkaDebeziumJsonDeserializationSchema
         try {
             byte[] key = message.key();
             JsonNode keyNode = null;
-            if (key != null) {
-                keyNode = objectMapper.readValue(key, JsonNode.class);
+            if (key != null && key.length > 0) {
+                try {
+                    keyNode = objectMapper.readValue(key, JsonNode.class);
+                } catch (Exception ignore) {
+                    // If the key is not valid JSON, ignore it to avoid failing deserialization.
+                    // The CDC pipeline only relies on the JSON value payload.
+                }
             }
 
             JsonNode valueNode = objectMapper.readValue(message.value(), JsonNode.class);
