@@ -117,6 +117,33 @@ public class CdcDynamicBucketWriteOperator extends TableWriteOperator<Tuple2<Cdc
         }
     }
 
+    /** {@link StreamOperatorFactory} of {@link CdcDynamicBucketWriteOperator}. */
+    public static class CoordinatedFactory
+            extends TableWriteOperator.CoordinatedFactory<Tuple2<CdcRecord, Integer>> {
+
+        public CoordinatedFactory(
+                FileStoreTable table,
+                StoreSinkWrite.Provider storeSinkWriteProvider,
+                String initialCommitUser) {
+            super(table, storeSinkWriteProvider, initialCommitUser);
+        }
+
+        @Override
+        @SuppressWarnings("unchecked")
+        public <T extends TableWriteOperator<Tuple2<CdcRecord, Integer>>>
+                T createStreamOperatorImpl(StreamOperatorParameters<Committable> parameters) {
+            return (T)
+                    new CdcDynamicBucketWriteOperator(
+                            parameters, table, storeSinkWriteProvider, initialCommitUser);
+        }
+
+        @Override
+        @SuppressWarnings("rawtypes")
+        public Class<? extends StreamOperator> getStreamOperatorClass(ClassLoader classLoader) {
+            return CdcDynamicBucketWriteOperator.class;
+        }
+    }
+
     /** {@link StreamOperatorFactory} of {@link CdcDynamicBucketWriteOperator}. */
     public static class Factory extends TableWriteOperator.Factory<Tuple2<CdcRecord, Integer>> {
 
