@@ -404,26 +404,6 @@ public class FlinkConnectorOptions {
                     .withDescription(
                             "Whether trigger partition mark done when recover from state.");
 
-    public static final ConfigOption<String> CLUSTERING_COLUMNS =
-            key("sink.clustering.by-columns")
-                    .stringType()
-                    .noDefaultValue()
-                    .withDescription(
-                            "Specifies the column name(s) used for comparison during range partitioning, in the format 'columnName1,columnName2'. "
-                                    + "If not set or set to an empty string, it indicates that the range partitioning feature is not enabled. "
-                                    + "This option will be effective only for bucket unaware table without primary keys and batch execution mode.");
-
-    public static final ConfigOption<String> CLUSTERING_STRATEGY =
-            key("sink.clustering.strategy")
-                    .stringType()
-                    .defaultValue("auto")
-                    .withDescription(
-                            "Specifies the comparison algorithm used for range partitioning, including 'zorder', 'hilbert', and 'order', "
-                                    + "corresponding to the z-order curve algorithm, hilbert curve algorithm, and basic type comparison algorithm, "
-                                    + "respectively. When not configured, it will automatically determine the algorithm based on the number of columns "
-                                    + "in 'sink.clustering.by-columns'. 'order' is used for 1 column, 'zorder' for less than 5 columns, "
-                                    + "and 'hilbert' for 5 or more columns.");
-
     public static final ConfigOption<Boolean> CLUSTERING_SORT_IN_CLUSTER =
             key("sink.clustering.sort-in-cluster")
                     .booleanType()
@@ -523,6 +503,42 @@ public class FlinkConnectorOptions {
                             "Commit listener will be called after a successful commit. This option list custom commit "
                                     + "listener identifiers separated by comma.");
 
+    public static final ConfigOption<Boolean> SINK_WRITER_COORDINATOR_ENABLED =
+            key("sink.writer-coordinator.enabled")
+                    .booleanType()
+                    .defaultValue(false)
+                    .withDescription(
+                            "Enable sink writer coordinator to plan data files in Job Manager.");
+
+    public static final ConfigOption<MemorySize> SINK_WRITER_COORDINATOR_CACHE_MEMORY =
+            key("sink.writer-coordinator.cache-memory")
+                    .memoryType()
+                    .defaultValue(MemorySize.ofMebiBytes(2048))
+                    .withDescription(
+                            "Controls the cache memory of writer coordinator to cache manifest files in Job Manager.");
+
+    public static final ConfigOption<MemorySize> SINK_WRITER_COORDINATOR_PAGE_SIZE =
+            key("sink.writer-coordinator.page-size")
+                    .memoryType()
+                    .defaultValue(MemorySize.ofKibiBytes(32))
+                    .withDescription(
+                            "Controls the page size for one RPC request of writer coordinator.");
+
+    public static final ConfigOption<Boolean> FILESYSTEM_JOB_LEVEL_SETTINGS_ENABLED =
+            key("filesystem.job-level-settings.enabled")
+                    .booleanType()
+                    .defaultValue(true)
+                    .withDescription("Enable pass job level filesystem settings to table file IO.");
+
+    public static final ConfigOption<String> SINK_WRITER_REFRESH_DETECTORS =
+            key("sink.writer-refresh-detectors")
+                    .stringType()
+                    .noDefaultValue()
+                    .withDescription(
+                            "The option groups which are expected to be refreshed when streaming writing, "
+                                    + "multiple option group separated by commas. "
+                                    + "Now only 'external-paths' is supported.");
+
     public static List<ConfigOption<?>> getOptions() {
         final Field[] fields = FlinkConnectorOptions.class.getFields();
         final List<ConfigOption<?>> list = new ArrayList<>(fields.length);
