@@ -19,9 +19,13 @@
 package org.apache.paimon.flink.action;
 
 import org.apache.paimon.CoreOptions;
+import org.apache.paimon.append.cluster.IncrementalClusterManager;
+import org.apache.paimon.compact.CompactUnit;
 import org.apache.paimon.data.BinaryRow;
 import org.apache.paimon.data.InternalRow;
 import org.apache.paimon.flink.FlinkConnectorOptions;
+import org.apache.paimon.flink.cluster.IncrementalClusterSplitSource;
+import org.apache.paimon.flink.cluster.RewriteIncrementalClusterCommittableOperator;
 import org.apache.paimon.flink.compact.AppendTableCompactBuilder;
 import org.apache.paimon.flink.postpone.PostponeBucketCompactSplitSource;
 import org.apache.paimon.flink.postpone.RewritePostponeBucketCommittableOperator;
@@ -32,18 +36,24 @@ import org.apache.paimon.flink.sink.CompactorSinkBuilder;
 import org.apache.paimon.flink.sink.FixedBucketSink;
 import org.apache.paimon.flink.sink.FlinkSinkBuilder;
 import org.apache.paimon.flink.sink.FlinkStreamPartitioner;
+import org.apache.paimon.flink.sink.RowAppendTableSink;
 import org.apache.paimon.flink.sink.RowDataChannelComputer;
+import org.apache.paimon.flink.sorter.TableSortInfo;
+import org.apache.paimon.flink.sorter.TableSorter;
 import org.apache.paimon.flink.source.CompactorSourceBuilder;
 import org.apache.paimon.manifest.ManifestEntry;
 import org.apache.paimon.options.Options;
+import org.apache.paimon.partition.PartitionPredicate;
 import org.apache.paimon.predicate.PartitionPredicateVisitor;
 import org.apache.paimon.predicate.Predicate;
 import org.apache.paimon.predicate.PredicateBuilder;
+import org.apache.paimon.predicate.PredicateProjectionConverter;
 import org.apache.paimon.table.BucketMode;
 import org.apache.paimon.table.FileStoreTable;
+import org.apache.paimon.table.source.DataSplit;
+import org.apache.paimon.types.RowType;
 import org.apache.paimon.utils.InternalRowPartitionComputer;
 import org.apache.paimon.utils.Pair;
-import org.apache.paimon.utils.Preconditions;
 
 import org.apache.flink.api.common.RuntimeExecutionMode;
 import org.apache.flink.configuration.ExecutionOptions;
@@ -65,8 +75,11 @@ import java.util.Iterator;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.stream.Collectors;
 
+import static org.apache.paimon.partition.PartitionPredicate.createBinaryPartitions;
 import static org.apache.paimon.partition.PartitionPredicate.createPartitionPredicate;
+import static org.apache.paimon.utils.Preconditions.checkArgument;
 
 /** Table compact action for Flink. */
 public class CompactAction extends TableActionBase {
@@ -93,6 +106,9 @@ public class CompactAction extends TableActionBase {
                             "Only FileStoreTable supports compact action. The table type is '%s'.",
                             table.getClass().getName()));
         }
+        checkArgument(
+                !((FileStoreTable) table).coreOptions().dataEvolutionEnabled(),
+                "Compact action does not support data evolution table yet. ");
         HashMap<String, String> dynamicOptions = new HashMap<>(tableConf);
         dynamicOptions.put(CoreOptions.WRITE_ONLY.key(), "false");
         table = table.copy(dynamicOptions);
@@ -136,8 +152,12 @@ public class CompactAction extends TableActionBase {
         if (fileStoreTable.coreOptions().bucket() == BucketMode.POSTPONE_BUCKET) {
             return buildForPostponeBucketCompaction(env, fileStoreTable, isStreaming);
         } else if (fileStoreTable.bucketMode() == BucketMode.BUCKET_UNAWARE) {
-            buildForAppendTableCompact(env, fileStoreTable, isStreaming);
-            return true;
+            if (fileStoreTable.coreOptions().clusteringIncrementalEnabled()) {
+                return buildForIncrementalClustering(env, fileStoreTable, isStreaming);
+            } else {
+                buildForAppendTableCompact(env, fileStoreTable, isStreaming);
+                return true;
+            }
         } else {
             buildForBucketedTableCompact(env, fileStoreTable, isStreaming);
             return true;
@@ -150,7 +170,7 @@ public class CompactAction extends TableActionBase {
         if (fullCompaction == null) {
             fullCompaction = !isStreaming;
         } else {
-            Preconditions.checkArgument(
+            checkArgument(
                     !(fullCompaction && isStreaming),
                     "The full compact strategy is only supported in batch mode. Please add -Dexecution.runtime-mode=BATCH.");
         }
@@ -170,7 +190,7 @@ public class CompactAction extends TableActionBase {
                 new CompactorSourceBuilder(identifier.getFullName(), table);
         CompactorSinkBuilder sinkBuilder = new CompactorSinkBuilder(table, fullCompaction);
 
-        sourceBuilder.withPartitionPredicate(getPredicate());
+        sourceBuilder.withPartitionPredicate(getPartitionPredicate());
         DataStreamSource<RowData> source =
                 sourceBuilder
                         .withEnv(env)
@@ -185,30 +205,164 @@ public class CompactAction extends TableActionBase {
             throws Exception {
         AppendTableCompactBuilder builder =
                 new AppendTableCompactBuilder(env, identifier.getFullName(), table);
-        builder.withPartitionPredicate(getPredicate());
+        builder.withPartitionPredicate(getPartitionPredicate());
         builder.withContinuousMode(isStreaming);
         builder.withPartitionIdleTime(partitionIdleTime);
         builder.build();
     }
 
-    protected Predicate getPredicate() throws Exception {
-        Preconditions.checkArgument(
+    private boolean buildForIncrementalClustering(
+            StreamExecutionEnvironment env, FileStoreTable table, boolean isStreaming)
+            throws Exception {
+        checkArgument(!isStreaming, "Incremental clustering currently only supports batch mode");
+
+        IncrementalClusterManager incrementalClusterManager =
+                new IncrementalClusterManager(table, getPartitionPredicate());
+
+        // non-full strategy as default for incremental clustering
+        if (fullCompaction == null) {
+            fullCompaction = false;
+        }
+        Options options = new Options(table.options());
+        int localSampleMagnification = table.coreOptions().getLocalSampleMagnification();
+        if (localSampleMagnification < 20) {
+            throw new IllegalArgumentException(
+                    String.format(
+                            "the config '%s=%d' should not be set too small, greater than or equal to 20 is needed.",
+                            CoreOptions.SORT_COMPACTION_SAMPLE_MAGNIFICATION.key(),
+                            localSampleMagnification));
+        }
+        String commitUser = CoreOptions.createCommitUser(options);
+        InternalRowPartitionComputer partitionComputer =
+                new InternalRowPartitionComputer(
+                        table.coreOptions().partitionDefaultName(),
+                        table.store().partitionType(),
+                        table.partitionKeys().toArray(new String[0]),
+                        table.coreOptions().legacyPartitionName());
+
+        // 1. pick cluster files for each partition
+        Map<BinaryRow, CompactUnit> compactUnits =
+                incrementalClusterManager.prepareForCluster(fullCompaction);
+        if (compactUnits.isEmpty()) {
+            LOGGER.warn(
+                    "No partition needs to be incrementally clustered. "
+                            + "Please set '--compact_strategy full' if you need forcibly trigger the cluster."
+                            + "Please set '--force_start_flink_job true' if you need forcibly start a flink job.");
+            return false;
+        }
+        Map<BinaryRow, DataSplit[]> partitionSplits =
+                compactUnits.entrySet().stream()
+                        .collect(
+                                Collectors.toMap(
+                                        Map.Entry::getKey,
+                                        entry ->
+                                                incrementalClusterManager
+                                                        .toSplits(
+                                                                entry.getKey(),
+                                                                entry.getValue().files())
+                                                        .toArray(new DataSplit[0])));
+
+        // 2. read，sort and write in partition
+        List<DataStream<Committable>> dataStreams = new ArrayList<>();
+
+        for (Map.Entry<BinaryRow, DataSplit[]> entry : partitionSplits.entrySet()) {
+            DataSplit[] splits = entry.getValue();
+            LinkedHashMap<String, String> partitionSpec =
+                    partitionComputer.generatePartValues(entry.getKey());
+            // 2.1 generate source for current partition
+            Pair<DataStream<RowData>, DataStream<Committable>> sourcePair =
+                    IncrementalClusterSplitSource.buildSource(
+                            env,
+                            table,
+                            partitionSpec,
+                            splits,
+                            options.get(FlinkConnectorOptions.SCAN_PARALLELISM));
+
+            // 2.2 cluster in partition
+            Integer sinkParallelism = options.get(FlinkConnectorOptions.SINK_PARALLELISM);
+            if (sinkParallelism == null) {
+                sinkParallelism = sourcePair.getLeft().getParallelism();
+            }
+            TableSortInfo sortInfo =
+                    new TableSortInfo.Builder()
+                            .setSortColumns(incrementalClusterManager.clusterKeys())
+                            .setSortStrategy(incrementalClusterManager.clusterCurve())
+                            .setSinkParallelism(sinkParallelism)
+                            .setLocalSampleSize(sinkParallelism * localSampleMagnification)
+                            .setGlobalSampleSize(sinkParallelism * 1000)
+                            .setRangeNumber(sinkParallelism * 10)
+                            .build();
+            DataStream<RowData> sorted =
+                    TableSorter.getSorter(env, sourcePair.getLeft(), table, sortInfo).sort();
+
+            // 2.3 write and then reorganize the committable
+            // set parallelism to null, and it'll forward parallelism when doWrite()
+            RowAppendTableSink sink = new RowAppendTableSink(table, null, null, null);
+            DataStream<Committable> written =
+                    sink.doWrite(
+                            FlinkSinkBuilder.mapToInternalRow(sorted, table.rowType()),
+                            commitUser,
+                            null);
+            DataStream<Committable> clusterCommittable =
+                    written.forward()
+                            .transform(
+                                    "Rewrite cluster committable",
+                                    new CommittableTypeInfo(),
+                                    new RewriteIncrementalClusterCommittableOperator(
+                                            table,
+                                            compactUnits.entrySet().stream()
+                                                    .collect(
+                                                            Collectors.toMap(
+                                                                    Map.Entry::getKey,
+                                                                    unit ->
+                                                                            unit.getValue()
+                                                                                    .outputLevel()))))
+                            .setParallelism(written.getParallelism());
+            dataStreams.add(clusterCommittable);
+            dataStreams.add(sourcePair.getRight());
+        }
+
+        // 3. commit
+        RowAppendTableSink sink = new RowAppendTableSink(table, null, null, null);
+        DataStream<Committable> dataStream = dataStreams.get(0);
+        for (int i = 1; i < dataStreams.size(); i++) {
+            dataStream = dataStream.union(dataStreams.get(i));
+        }
+        sink.doCommit(dataStream, commitUser);
+        return true;
+    }
+
+    protected PartitionPredicate getPartitionPredicate() throws Exception {
+        checkArgument(
                 partitions == null || whereSql == null,
                 "partitions and where cannot be used together.");
         Predicate predicate = null;
+        RowType partitionType = table.rowType().project(table.partitionKeys());
+        String partitionDefaultName = ((FileStoreTable) table).coreOptions().partitionDefaultName();
         if (partitions != null) {
-            predicate =
-                    PredicateBuilder.or(
-                            partitions.stream()
-                                    .map(
-                                            p ->
-                                                    createPartitionPredicate(
-                                                            p,
-                                                            table.rowType(),
-                                                            ((FileStoreTable) table)
-                                                                    .coreOptions()
-                                                                    .partitionDefaultName()))
-                                    .toArray(Predicate[]::new));
+            boolean fullMode =
+                    partitions.stream()
+                            .allMatch(part -> part.size() == partitionType.getFieldCount());
+            if (fullMode) {
+                List<BinaryRow> binaryPartitions =
+                        createBinaryPartitions(partitions, partitionType, partitionDefaultName);
+                return PartitionPredicate.fromMultiple(partitionType, binaryPartitions);
+            } else {
+                // partitions may be partial partition fields, so here must to use predicate way.
+                predicate =
+                        partitions.stream()
+                                .map(
+                                        partition ->
+                                                createPartitionPredicate(
+                                                        partition,
+                                                        table.rowType(),
+                                                        partitionDefaultName))
+                                .reduce(PredicateBuilder::or)
+                                .orElseThrow(
+                                        () ->
+                                                new RuntimeException(
+                                                        "Failed to get partition filter."));
+            }
         } else if (whereSql != null) {
             SimpleSqlPredicateConvertor simpleSqlPredicateConvertor =
                     new SimpleSqlPredicateConvertor(table.rowType());
@@ -220,22 +374,31 @@ public class CompactAction extends TableActionBase {
             LOGGER.info("the partition predicate of compaction is {}", predicate);
             PartitionPredicateVisitor partitionPredicateVisitor =
                     new PartitionPredicateVisitor(table.partitionKeys());
-            Preconditions.checkArgument(
+            checkArgument(
                     predicate.visit(partitionPredicateVisitor),
                     "Only partition key can be specialized in compaction action.");
+            predicate =
+                    predicate
+                            .visit(
+                                    new PredicateProjectionConverter(
+                                            table.rowType().projectIndexes(table.partitionKeys())))
+                            .orElseThrow(
+                                    () ->
+                                            new RuntimeException(
+                                                    "Failed to convert partition predicate."));
         }
 
-        return predicate;
+        return PartitionPredicate.fromPredicate(partitionType, predicate);
     }
 
     private boolean buildForPostponeBucketCompaction(
             StreamExecutionEnvironment env, FileStoreTable table, boolean isStreaming) {
-        Preconditions.checkArgument(
+        checkArgument(
                 !isStreaming, "Postpone bucket compaction currently only supports batch mode");
-        Preconditions.checkArgument(
+        checkArgument(
                 partitions == null,
                 "Postpone bucket compaction currently does not support specifying partitions");
-        Preconditions.checkArgument(
+        checkArgument(
                 whereSql == null,
                 "Postpone bucket compaction currently does not support predicates");
 
