@@ -18,7 +18,7 @@
 
 package org.apache.paimon.flink.compact;
 
-import org.apache.paimon.append.MultiTableUnawareAppendCompactionTask;
+import org.apache.paimon.append.MultiTableAppendCompactTask;
 import org.apache.paimon.catalog.Catalog;
 import org.apache.paimon.catalog.CatalogLoader;
 import org.apache.paimon.catalog.Identifier;
@@ -45,8 +45,8 @@ import static org.apache.paimon.flink.utils.MultiTablesCompactorUtil.shouldCompa
  *     <ol>
  *       <li>Tuple2<{@link Split},String> for the table with multi buckets, such as dynamic or fixed
  *           bucket table.
- *       <li>{@link MultiTableUnawareAppendCompactionTask} for the table witch fixed single bucket
- *           ,such as unaware bucket table.
+ *       <li>{@link MultiTableAppendCompactTask} for the table witch fixed single bucket ,such as
+ *           unaware bucket table.
  *     </ol>
  */
 public abstract class MultiTableScanBase<T> implements AutoCloseable {
