@@ -18,11 +18,8 @@
 
 package org.apache.paimon.flink.sink;
 
-import org.apache.paimon.Snapshot;
-import org.apache.paimon.operation.TagDeletion;
 import org.apache.paimon.table.FileStoreTable;
-import org.apache.paimon.utils.SnapshotManager;
-import org.apache.paimon.utils.TagManager;
+import org.apache.paimon.tag.TagBatchCreation;
 
 import org.apache.flink.metrics.groups.OperatorMetricGroup;
 import org.apache.flink.runtime.checkpoint.CheckpointOptions;
@@ -37,12 +34,6 @@ import org.apache.flink.streaming.runtime.streamrecord.LatencyMarker;
 import org.apache.flink.streaming.runtime.streamrecord.StreamRecord;
 import org.apache.flink.streaming.runtime.watermarkstatus.WatermarkStatus;
 
-import java.time.Instant;
-import java.time.LocalDateTime;
-import java.time.ZoneId;
-import java.time.format.DateTimeFormatter;
-import java.util.List;
-
 /**
  * Commit {@link Committable} for snapshot using the {@link CommitterOperator}. When the task is
  * completed, the corresponding tag is generated.
@@ -58,10 +49,13 @@ public class BatchWriteGeneratorTagOperator<CommitT, GlobalCommitT>
 
     protected final FileStoreTable table;
 
+    protected final TagBatchCreation tagBatchCreation;
+
     public BatchWriteGeneratorTagOperator(
             CommitterOperator<CommitT, GlobalCommitT> commitOperator, FileStoreTable table) {
         this.table = table;
         this.commitOperator = commitOperator;
+        this.tagBatchCreation = new TagBatchCreation(table);
     }
 
     @Override
@@ -91,77 +85,6 @@ public class BatchWriteGeneratorTagOperator<CommitT, GlobalCommitT>
         commitOperator.notifyCheckpointAborted(checkpointId);
     }
 
-    private void createTag() {
-        SnapshotManager snapshotManager = table.snapshotManager();
-        Snapshot snapshot = snapshotManager.latestSnapshot();
-        if (snapshot == null) {
-            return;
-        }
-        TagManager tagManager = table.tagManager();
-        TagDeletion tagDeletion = table.store().newTagDeletion();
-        Instant instant = Instant.ofEpochMilli(snapshot.timeMillis());
-        LocalDateTime localDateTime = LocalDateTime.ofInstant(instant, ZoneId.systemDefault());
-        String tagName =
-                BATCH_WRITE_TAG_PREFIX
-                        + localDateTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
-        try {
-            // If the tag already exists, delete the tag
-            tagManager.deleteTag(
-                    tagName, tagDeletion, snapshotManager, table.store().createTagCallbacks());
-            // Create a new tag
-            tagManager.createTag(
-                    snapshot,
-                    tagName,
-                    table.coreOptions().tagDefaultTimeRetained(),
-                    table.store().createTagCallbacks(),
-                    false);
-            // Expire the tag
-            expireTag();
-        } catch (Exception e) {
-            if (tagManager.tagExists(tagName)) {
-                tagManager.deleteTag(
-                        tagName, tagDeletion, snapshotManager, table.store().createTagCallbacks());
-            }
-        }
-    }
-
-    private void expireTag() {
-        Integer tagNumRetainedMax = table.coreOptions().tagNumRetainedMax();
-        if (tagNumRetainedMax != null) {
-            SnapshotManager snapshotManager = table.snapshotManager();
-            if (snapshotManager.latestSnapshot() == null) {
-                return;
-            }
-            TagManager tagManager = table.tagManager();
-            TagDeletion tagDeletion = table.store().newTagDeletion();
-            long tagCount = tagManager.tagCount();
-
-            while (tagCount > tagNumRetainedMax) {
-                for (List<String> tagNames : tagManager.tags().values()) {
-                    if (tagCount - tagNames.size() >= tagNumRetainedMax) {
-                        tagManager.deleteAllTagsOfOneSnapshot(
-                                tagNames, tagDeletion, snapshotManager);
-                        tagCount = tagCount - tagNames.size();
-                    } else {
-                        List<String> sortedTagNames = tagManager.sortTagsOfOneSnapshot(tagNames);
-                        for (String toBeDeleted : sortedTagNames) {
-                            tagManager.deleteTag(
-                                    toBeDeleted,
-                                    tagDeletion,
-                                    snapshotManager,
-                                    table.store().createTagCallbacks());
-                            tagCount--;
-                            if (tagCount == tagNumRetainedMax) {
-                                break;
-                            }
-                        }
-                        break;
-                    }
-                }
-            }
-        }
-    }
-
     @Override
     public void open() throws Exception {
         commitOperator.open();
@@ -189,7 +112,7 @@ public class BatchWriteGeneratorTagOperator<CommitT, GlobalCommitT>
 
     @Override
     public void finish() throws Exception {
-        createTag();
+        tagBatchCreation.createTag();
         commitOperator.finish();
     }
 
