@@ -53,13 +53,12 @@ public interface Committer<CommitT, GlobalCommitT> extends AutoCloseable {
      * Filter out all {@link GlobalCommitT} which have committed, and commit the remaining {@link
      * GlobalCommitT}.
      */
-    int filterAndCommit(List<GlobalCommitT> globalCommittables, boolean checkAppendFiles)
+    int filterAndCommit(
+            List<GlobalCommitT> globalCommittables,
+            boolean checkAppendFiles,
+            boolean partitionMarkDoneRecoverFromState)
             throws IOException;
 
-    default int filterAndCommit(List<GlobalCommitT> globalCommittables) throws IOException {
-        return filterAndCommit(globalCommittables, true);
-    }
-
     Map<Long, List<CommitT>> groupByCheckpoint(Collection<CommitT> committables);
 
     /** Factory to create {@link Committer}. */
