@@ -42,8 +42,8 @@ import org.apache.paimon.table.sink.CommitMessageImpl;
 import org.apache.paimon.table.source.DataSplit;
 import org.apache.paimon.types.DataField;
 import org.apache.paimon.types.RowType;
+import org.apache.paimon.utils.DataFilePathFactories;
 import org.apache.paimon.utils.FileStorePathFactory;
-import org.apache.paimon.utils.Pair;
 
 import org.apache.flink.streaming.api.operators.OneInputStreamOperatorFactory;
 import org.apache.flink.streaming.api.operators.StreamOperator;
@@ -127,16 +127,16 @@ public class RewriteFileIndexSink extends FlinkWriteSink<ManifestEntry> {
             ManifestEntry entry = element.getValue();
             BinaryRow partition = entry.partition();
             int bucket = entry.bucket();
-            DataFileMeta file = entry.file();
-            DataFileMeta indexedFile = fileIndexProcessor.process(partition, bucket, file);
+            DataFileMeta indexedFile = fileIndexProcessor.process(partition, bucket, entry);
 
             CommitMessageImpl commitMessage =
                     new CommitMessageImpl(
                             partition,
                             bucket,
+                            entry.totalBuckets(),
                             DataIncrement.emptyIncrement(),
                             new CompactIncrement(
-                                    Collections.singletonList(file),
+                                    Collections.singletonList(entry.file()),
                                     Collections.singletonList(indexedFile),
                                     Collections.emptyList()));
 
@@ -161,7 +161,7 @@ public class RewriteFileIndexSink extends FlinkWriteSink<ManifestEntry> {
         private final FileIndexOptions fileIndexOptions;
         private final FileIO fileIO;
         private final FileStorePathFactory pathFactory;
-        private final Map<Pair<BinaryRow, Integer>, DataFilePathFactory> dataFilePathFactoryMap;
+        private final DataFilePathFactories pathFactories;
         private final SchemaCache schemaInfoCache;
         private final long sizeInMeta;
 
@@ -170,19 +170,16 @@ public class RewriteFileIndexSink extends FlinkWriteSink<ManifestEntry> {
             this.fileIndexOptions = table.coreOptions().indexColumnsOptions();
             this.fileIO = table.fileIO();
             this.pathFactory = table.store().pathFactory();
-            this.dataFilePathFactoryMap = new HashMap<>();
+            this.pathFactories = new DataFilePathFactories(pathFactory);
             this.schemaInfoCache =
                     new SchemaCache(fileIndexOptions, new SchemaManager(fileIO, table.location()));
             this.sizeInMeta = table.coreOptions().fileIndexInManifestThreshold();
         }
 
-        public DataFileMeta process(BinaryRow partition, int bucket, DataFileMeta dataFileMeta)
+        public DataFileMeta process(BinaryRow partition, int bucket, ManifestEntry manifestEntry)
                 throws IOException {
-            DataFilePathFactory dataFilePathFactory =
-                    dataFilePathFactoryMap.computeIfAbsent(
-                            Pair.of(partition, bucket),
-                            p -> pathFactory.createDataFilePathFactory(partition, bucket));
-
+            DataFileMeta dataFileMeta = manifestEntry.file();
+            DataFilePathFactory dataFilePathFactory = pathFactories.get(partition, bucket);
             SchemaInfo schemaInfo = schemaInfoCache.schemaInfo(dataFileMeta.schemaId());
             List<String> extras = new ArrayList<>(dataFileMeta.extraFiles());
             List<String> indexFiles =
@@ -249,6 +246,7 @@ public class RewriteFileIndexSink extends FlinkWriteSink<ManifestEntry> {
                                                         pathFactory
                                                                 .bucketPath(partition, bucket)
                                                                 .toString())
+                                                .withTotalBuckets(manifestEntry.totalBuckets())
                                                 .withDataFiles(
                                                         Collections.singletonList(dataFileMeta))
                                                 .rawConvertible(true)
