@@ -178,14 +178,18 @@ public class StoreMultiCommitter
 
     @Override
     public int filterAndCommit(
-            List<WrappedManifestCommittable> globalCommittables, boolean checkAppendFiles)
-            throws IOException {
+            List<WrappedManifestCommittable> globalCommittables,
+            boolean checkAppendFiles,
+            boolean partitionMarkDoneRecoverFromState) {
         int result = 0;
         for (Map.Entry<Identifier, List<ManifestCommittable>> entry :
                 groupByTable(globalCommittables).entrySet()) {
             result +=
                     getStoreCommitter(entry.getKey())
-                            .filterAndCommit(entry.getValue(), checkAppendFiles);
+                            .filterAndCommit(
+                                    entry.getValue(),
+                                    checkAppendFiles,
+                                    partitionMarkDoneRecoverFromState);
         }
         return result;
     }
