@@ -22,6 +22,8 @@ import org.apache.paimon.annotation.VisibleForTesting;
 import org.apache.paimon.flink.source.assigners.FIFOSplitAssigner;
 import org.apache.paimon.flink.source.assigners.PreAssignSplitAssigner;
 import org.apache.paimon.flink.source.assigners.SplitAssigner;
+import org.apache.paimon.postpone.PostponeBucketFileStoreWrite;
+import org.apache.paimon.table.BucketMode;
 import org.apache.paimon.table.sink.ChannelComputer;
 import org.apache.paimon.table.source.DataSplit;
 import org.apache.paimon.table.source.EndOfScanException;
@@ -69,6 +71,8 @@ public class ContinuousFileSplitEnumerator
 
     protected final StreamTableScan scan;
 
+    protected final int splitMaxPerTask;
+
     protected final SplitAssigner splitAssigner;
 
     protected final ConsumerProgressCalculator consumerProgressCalculator;
@@ -93,7 +97,7 @@ public class ContinuousFileSplitEnumerator
             @Nullable Long nextSnapshotId,
             long discoveryInterval,
             StreamTableScan scan,
-            boolean unawareBucket,
+            boolean unordered,
             int splitMaxPerTask,
             boolean shuffleBucketWithPartition,
             int maxSnapshotCount) {
@@ -104,8 +108,9 @@ public class ContinuousFileSplitEnumerator
         this.readersAwaitingSplit = new LinkedHashSet<>();
         this.splitGenerator = new FileStoreSourceSplitGenerator();
         this.scan = scan;
-        this.splitAssigner = createSplitAssigner(unawareBucket);
+        this.splitMaxPerTask = splitMaxPerTask;
         this.splitMaxNum = context.currentParallelism() * splitMaxPerTask;
+        this.splitAssigner = createSplitAssigner(unordered);
         this.shuffleBucketWithPartition = shuffleBucketWithPartition;
         addSplits(remainSplits);
 
@@ -301,17 +306,29 @@ public class ContinuousFileSplitEnumerator
 
     protected int assignSuggestedTask(FileStoreSourceSplit split) {
         DataSplit dataSplit = ((DataSplit) split.split());
+        int parallelism = context.currentParallelism();
+
+        int bucketId;
+        if (dataSplit.bucket() == BucketMode.POSTPONE_BUCKET) {
+            bucketId =
+                    PostponeBucketFileStoreWrite.getWriteId(dataSplit.dataFiles().get(0).fileName())
+                            % parallelism;
+        } else {
+            bucketId = dataSplit.bucket();
+        }
+
         if (shuffleBucketWithPartition) {
-            return ChannelComputer.select(
-                    dataSplit.partition(), dataSplit.bucket(), context.currentParallelism());
+            return ChannelComputer.select(dataSplit.partition(), bucketId, parallelism);
+        } else {
+            return ChannelComputer.select(bucketId, parallelism);
         }
-        return ChannelComputer.select(dataSplit.bucket(), context.currentParallelism());
     }
 
-    protected SplitAssigner createSplitAssigner(boolean unawareBucket) {
-        return unawareBucket
+    protected SplitAssigner createSplitAssigner(boolean unordered) {
+        return unordered
                 ? new FIFOSplitAssigner(Collections.emptyList())
-                : new PreAssignSplitAssigner(1, context, Collections.emptyList());
+                : new PreAssignSplitAssigner(
+                        this.splitMaxPerTask, context, Collections.emptyList());
     }
 
     protected boolean noMoreSplits() {
