@@ -35,12 +35,47 @@ import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.atomic.AtomicBoolean;
 
 import static org.assertj.core.api.Assertions.assertThat;
+import static org.assertj.core.api.Assertions.assertThatThrownBy;
 
 /** IT cases for postpone bucket tables. */
 public class PostponeBucketTableITCase extends AbstractTestBase {
 
     private static final int TIMEOUT = 120;
 
+    @Test
+    public void testRetractOnPartialUpdate() {
+        String warehouse = getTempDirPath();
+        TableEnvironment tEnv =
+                tableEnvironmentBuilder()
+                        .batchMode()
+                        .setConf(TableConfigOptions.TABLE_DML_SYNC, true)
+                        .build();
+
+        tEnv.executeSql(
+                "CREATE CATALOG mycat WITH (\n"
+                        + "  'type' = 'paimon',\n"
+                        + "  'warehouse' = '"
+                        + warehouse
+                        + "'\n"
+                        + ")");
+        tEnv.executeSql("USE CATALOG mycat");
+        tEnv.executeSql(
+                "CREATE TABLE T (\n"
+                        + "  k INT,\n"
+                        + "  v1 INT,\n"
+                        + "  v2 INT,\n"
+                        + "  row_kind_col STRING,\n"
+                        + "  PRIMARY KEY (k) NOT ENFORCED\n"
+                        + ") WITH (\n"
+                        + "  'bucket' = '-2',\n"
+                        + "  'merge-engine' = 'partial-update',\n"
+                        + "  'rowkind.field' = 'row_kind_col'\n"
+                        + ")");
+        assertThatThrownBy(() -> tEnv.executeSql("INSERT INTO T VALUES (1, 1, 1, '-D')").await())
+                .rootCause()
+                .hasMessageContaining("By default, Partial update can not accept delete records");
+    }
+
     @Test
     public void testWriteThenCompact() throws Exception {
         String warehouse = getTempDirPath();
@@ -679,7 +714,7 @@ public class PostponeBucketTableITCase extends AbstractTestBase {
         TableEnvironment sEnv =
                 tableEnvironmentBuilder()
                         .streamingMode()
-                        .parallelism(1)
+                        .parallelism(2)
                         .checkpointIntervalMs(500)
                         .build();
         String createCatalog =
@@ -715,15 +750,27 @@ public class PostponeBucketTableITCase extends AbstractTestBase {
         bEnv.executeSql("INSERT INTO T VALUES (1, 101), (3, 31)").await();
         bEnv.executeSql("CALL sys.compact(`table` => 'default.T')").await();
 
+        TableResult streamingSelect = sEnv.executeSql("SELECT * FROM T");
+        Thread.sleep(1000);
+
+        bEnv.executeSql("INSERT INTO T VALUES (1, 102), (4, 42)").await();
+        bEnv.executeSql("INSERT INTO T VALUES (1, 103), (5, 53)").await();
+        bEnv.executeSql("CALL sys.compact(`table` => 'default.T')").await();
+
         assertThat(collect(bEnv.executeSql("SELECT * FROM T")))
-                .containsExactlyInAnyOrder("+I[1, 101]", "+I[2, 20]", "+I[3, 31]");
-        TableResult streamingSelect =
-                sEnv.executeSql("SELECT * FROM T /*+ OPTIONS('scan.snapshot-id' = '1') */");
+                .containsExactlyInAnyOrder(
+                        "+I[1, 103]", "+I[2, 20]", "+I[3, 31]", "+I[4, 42]", "+I[5, 53]");
         JobClient client = streamingSelect.getJobClient().get();
         CloseableIterator<Row> it = streamingSelect.collect();
-        assertThat(collect(client, it, 5))
+        assertThat(collect(client, it, 7))
                 .containsExactlyInAnyOrder(
-                        "+I[1, 10]", "+I[2, 20]", "+I[1, 100]", "+I[1, 101]", "+I[3, 31]");
+                        "+I[1, 101]",
+                        "+I[2, 20]",
+                        "+I[3, 31]",
+                        "+I[1, 102]",
+                        "+I[4, 42]",
+                        "+I[1, 103]",
+                        "+I[5, 53]");
     }
 
     private List<String> collect(TableResult result) throws Exception {
