@@ -60,13 +60,17 @@ import java.util.concurrent.ThreadLocalRandom;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
 
+import static org.apache.paimon.flink.FlinkConnectorOptions.SINK_WRITER_COORDINATOR_ENABLED;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.assertj.core.api.Assertions.assertThatCode;
 
 /** Tests for changelog table with primary keys. */
 public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
 
-    private static final int TIMEOUT = 480;
+    // This test is to discover unknown consistency bugs.
+    // When this test fails, look carefully into its core reason.
+    // Do not simply increase the timeout and pretend that everything is OK.
+    private static final int TIMEOUT = 180;
     private static final Logger LOG = LoggerFactory.getLogger(PrimaryKeyFileStoreTableITCase.class);
 
     // ------------------------------------------------------------------------
@@ -88,6 +92,9 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
         if (random.nextBoolean()) {
             tableDefaultProperties.put(CoreOptions.LOCAL_MERGE_BUFFER_SIZE.key(), "5m");
         }
+        if (random.nextBoolean()) {
+            tableDefaultProperties.put(SINK_WRITER_COORDINATOR_ENABLED.key(), "true");
+        }
     }
 
     private String createCatalogSql(String catalogName, String warehouse) {
@@ -422,6 +429,91 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
         assertThat(fileIO.exists(new Path(externalPath2))).isFalse();
     }
 
+    @Test
+    public void testDropTableWithBranchExternalPaths() throws Exception {
+        TableEnvironment sEnv =
+                tableEnvironmentBuilder()
+                        .streamingMode()
+                        .checkpointIntervalMs(ThreadLocalRandom.current().nextInt(900) + 100)
+                        .parallelism(1)
+                        .build();
+
+        sEnv.executeSql(createCatalogSql("testCatalog", path + "/warehouse"));
+        sEnv.executeSql("USE CATALOG testCatalog");
+        String externalPaths = TraceableFileIO.SCHEME + "://" + externalPath1;
+        String externalPath2s = LocalFileIOLoader.SCHEME + "://" + externalPath2;
+        sEnv.executeSql(
+                "CREATE TABLE T2 ( k INT, v STRING, PRIMARY KEY (k) NOT ENFORCED ) "
+                        + "WITH ( "
+                        + "'bucket' = '1',"
+                        + "'data-file.external-paths' = '"
+                        + externalPaths
+                        + "',"
+                        + "'data-file.external-paths.strategy' = 'round-robin'"
+                        + ")");
+
+        // create branch
+        sEnv.executeSql(
+                String.format("CALL sys.create_branch('%s.%s', 'branch1')", "default", "T2"));
+
+        // insert data to branch
+        sEnv.executeSql("INSERT INTO T2/*+ OPTIONS('branch' = 'branch1') */ VALUES (1, 'A')")
+                .await();
+
+        CloseableIterator<Row> it =
+                collect(sEnv.executeSql("SELECT * FROM T2 /*+ OPTIONS('branch' = 'branch1') */"));
+
+        // read initial data
+        List<String> actual = new ArrayList<>();
+        for (int i = 0; i < 1; i++) {
+            actual.add(it.next().toString());
+        }
+        assertThat(actual).containsExactlyInAnyOrder("+I[1, A]");
+
+        String sql =
+                String.format(
+                        "ALTER TABLE `T2$branch_branch1` SET ( 'data-file.external-paths' = '%s')",
+                        externalPath2s);
+        sEnv.executeSql(sql);
+        // insert data
+        sEnv.executeSql("INSERT INTO T2/*+ OPTIONS('branch' = 'branch1') */ VALUES (2, 'B')")
+                .await();
+
+        for (int i = 0; i < 1; i++) {
+            actual.add(it.next().toString());
+        }
+
+        // alter table external path
+        sEnv.executeSql(
+                "ALTER TABLE T2 SET ( "
+                        + "'data-file.external-paths' = '"
+                        + externalPath2s
+                        + "'"
+                        + ")");
+
+        // insert data
+        sEnv.executeSql("INSERT INTO T2/*+ OPTIONS('branch' = 'branch1') */ VALUES (3, 'C')")
+                .await();
+
+        for (int i = 0; i < 1; i++) {
+            actual.add(it.next().toString());
+        }
+
+        assertThat(actual).containsExactlyInAnyOrder("+I[1, A]", "+I[2, B]", "+I[3, C]");
+
+        LocalFileIO fileIO = LocalFileIO.create();
+
+        assertThat(fileIO.exists(new Path(externalPath1))).isTrue();
+        assertThat(fileIO.exists(new Path(externalPath2))).isTrue();
+
+        // drop table
+        sEnv.executeSql("DROP TABLE T2");
+
+        assertThat(fileIO.exists(new Path(path + "/warehouse" + "/default.db" + "/T2"))).isFalse();
+        assertThat(fileIO.exists(new Path(externalPath1))).isFalse();
+        assertThat(fileIO.exists(new Path(externalPath2))).isFalse();
+    }
+
     @Test
     public void testTableReadWriteWithExternalPathSpecificFS() throws Exception {
         TableEnvironment sEnv =
@@ -471,6 +563,76 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
         assertThat(actual).containsExactlyInAnyOrder("+I[1, A]", "+I[2, B]", "+I[3, C]");
     }
 
+    @Test
+    public void testTableReadWriteWithChangedExternalPath() throws Exception {
+        TableEnvironment sEnv =
+                tableEnvironmentBuilder()
+                        .streamingMode()
+                        .checkpointIntervalMs(ThreadLocalRandom.current().nextInt(900) + 100)
+                        .parallelism(1)
+                        .build();
+
+        sEnv.executeSql(createCatalogSql("testCatalog", path + "/warehouse"));
+        sEnv.executeSql("USE CATALOG testCatalog");
+        String externalPaths1 = TraceableFileIO.SCHEME + "://" + externalPath1.toString();
+        String externalPaths2 = LocalFileIOLoader.SCHEME + "://" + externalPath2.toString();
+
+        int bucket = ThreadLocalRandom.current().nextBoolean() ? 1 : -1;
+        sEnv.executeSql(
+                "CREATE TABLE T2 ( k INT, v INT, PRIMARY KEY (k) NOT ENFORCED ) "
+                        + "WITH ( "
+                        + "'bucket' = '"
+                        + bucket
+                        + "',"
+                        + "'sink.writer-refresh-detectors' = 'external-paths',"
+                        + "'data-file.external-paths' = '"
+                        + externalPaths1
+                        + "',"
+                        + "'data-file.external-paths.strategy' = 'round-robin'"
+                        + ")");
+
+        // Create a datagen source table
+        sEnv.executeSql(
+                "CREATE TEMPORARY TABLE datagen_source (k INT, v INT) WITH ("
+                        + "'connector' = 'datagen', "
+                        + "'rows-per-second' = '1', "
+                        + "'fields.k.kind' = 'sequence', "
+                        + "'fields.k.start' = '1', "
+                        + "'fields.k.end' = '5', "
+                        + "'fields.v.kind' = 'sequence', "
+                        + "'fields.v.start' = '1', "
+                        + "'fields.v.end' = '5'"
+                        + ")");
+
+        CloseableIterator<Row> it = collect(sEnv.executeSql("SELECT * FROM T2"));
+
+        // Insert data from datagen source into T2
+        sEnv.executeSql("INSERT INTO T2 SELECT * FROM datagen_source");
+
+        sEnv.executeSql(
+                "ALTER TABLE T2 SET ( "
+                        + "'data-file.external-paths' = '"
+                        + externalPaths2
+                        + "'"
+                        + ")");
+
+        // Read and verify the data
+        List<String> resultRows = new ArrayList<>();
+        for (int i = 0; i < 5; i++) {
+            resultRows.add(it.next().toString());
+        }
+
+        assertThat(resultRows)
+                .containsExactlyInAnyOrder(
+                        "+I[1, 1]", "+I[2, 2]", "+I[3, 3]", "+I[4, 4]", "+I[5, 5]");
+
+        LocalFileIO fileIO = LocalFileIO.create();
+        assertThat(fileIO.exists(new Path(externalPath1))).isTrue();
+        assertThat(fileIO.exists(new Path(externalPath2))).isTrue();
+        assertThat(fileIO.listStatus(new Path(externalPath1)).length).isGreaterThanOrEqualTo(1);
+        assertThat(fileIO.listStatus(new Path(externalPath2)).length).isGreaterThanOrEqualTo(1);
+    }
+
     @Test
     public void testTableReadWriteBranch() throws Exception {
         TableEnvironment sEnv =
@@ -1256,6 +1418,10 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
                         + tableProperties
                         + ")");
 
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Creating test table with properties {}", tableProperties);
+        }
+
         // input data must be strictly ordered
         tEnv.getConfig()
                 .getConfiguration()
@@ -1291,7 +1457,7 @@ public class PrimaryKeyFileStoreTableITCase extends AbstractTestBase {
                                 + "  'fields.i.kind' = 'sequence',"
                                 + "  'fields.i.start' = '0',"
                                 + "  'fields.i.end' = '"
-                                + (usefulNumRows - 1) * factor
+                                + (usefulNumRows * factor - 1)
                                 + "',"
                                 + "  'number-of-rows' = '"
                                 + usefulNumRows * factor
