@@ -26,7 +26,7 @@ import org.apache.paimon.flink.FlinkRowData;
 import org.apache.paimon.flink.lookup.PrimaryKeyPartialLookupTable.LocalQueryExecutor;
 import org.apache.paimon.flink.lookup.PrimaryKeyPartialLookupTable.QueryExecutor;
 import org.apache.paimon.flink.lookup.PrimaryKeyPartialLookupTable.RemoteQueryExecutor;
-import org.apache.paimon.lookup.RocksDBOptions;
+import org.apache.paimon.lookup.rocksdb.RocksDBOptions;
 import org.apache.paimon.options.Options;
 import org.apache.paimon.schema.Schema;
 import org.apache.paimon.schema.SchemaManager;
@@ -108,7 +108,11 @@ public class FileStoreLookupFunctionTest {
     private FileStoreLookupFunction createLookupFunction(
             FileStoreTable table, boolean joinEqualPk) {
         return new FileStoreLookupFunction(
-                table, new int[] {0, 1}, joinEqualPk ? new int[] {0, 1} : new int[] {1}, null);
+                table,
+                new int[] {0, 1},
+                joinEqualPk ? new int[] {0, 1} : new int[] {1},
+                null,
+                null);
     }
 
     private FileStoreTable createFileStoreTable(
