@@ -19,21 +19,33 @@
 package org.apache.spark.sql.execution.shim
 
 import org.apache.paimon.CoreOptions
+import org.apache.paimon.spark.SparkCatalog
 import org.apache.paimon.spark.catalog.FormatTableCatalog
 
 import org.apache.spark.sql.{SparkSession, Strategy}
-import org.apache.spark.sql.catalyst.plans.logical.{CreateTableAsSelect, LogicalPlan}
-import org.apache.spark.sql.connector.catalog.{CatalogV2Util, StagingTableCatalog}
-import org.apache.spark.sql.execution.SparkPlan
+import org.apache.spark.sql.catalyst.analysis.ResolvedDBObjectName
+import org.apache.spark.sql.catalyst.plans.logical.{CreateTableAsSelect, LogicalPlan, TableSpec}
+import org.apache.spark.sql.connector.catalog.StagingTableCatalog
+import org.apache.spark.sql.execution.{PaimonStrategyHelper, SparkPlan}
 import org.apache.spark.sql.execution.datasources.v2.CreateTableAsSelectExec
 import org.apache.spark.sql.util.CaseInsensitiveStringMap
 
 import scala.collection.JavaConverters._
 
-case class PaimonCreateTableAsSelectStrategy(spark: SparkSession) extends Strategy {
+case class PaimonCreateTableAsSelectStrategy(spark: SparkSession)
+  extends Strategy
+  with PaimonStrategyHelper {
+
+  import org.apache.spark.sql.connector.catalog.CatalogV2Implicits._
 
   override def apply(plan: LogicalPlan): Seq[SparkPlan] = plan match {
-    case CreateTableAsSelect(catalog, ident, parts, query, props, options, ifNotExists) =>
+    case CreateTableAsSelect(
+          ResolvedDBObjectName(catalog: SparkCatalog, ident),
+          parts,
+          query,
+          tableSpec: TableSpec,
+          options,
+          ifNotExists) =>
       catalog match {
         case _: StagingTableCatalog =>
           throw new RuntimeException("Paimon can't extend StagingTableCatalog for now.")
@@ -42,12 +54,12 @@ case class PaimonCreateTableAsSelectStrategy(spark: SparkSession) extends Strate
           val (coreOptions, writeOptions) = options.partition {
             case (key, _) => coreOptionKeys.contains(key)
           }
-          val newProps = CatalogV2Util.withDefaultOwnership(props) ++ coreOptions
+          val newTableSpec = tableSpec.copy(properties = tableSpec.properties ++ coreOptions)
 
           val isPartitionedFormatTable = {
             catalog match {
               case catalog: FormatTableCatalog =>
-                catalog.isFormatTable(newProps.get("provider").orNull) && parts.nonEmpty
+                catalog.isFormatTable(newTableSpec.provider.orNull) && parts.nonEmpty
               case _ => false
             }
           }
@@ -58,12 +70,12 @@ case class PaimonCreateTableAsSelectStrategy(spark: SparkSession) extends Strate
           }
 
           CreateTableAsSelectExec(
-            catalog,
-            ident,
+            catalog.asTableCatalog,
+            ident.asIdentifier,
             parts,
             query,
             planLater(query),
-            newProps,
+            qualifyLocInTableSpec(newTableSpec),
             new CaseInsensitiveStringMap(writeOptions.asJava),
             ifNotExists
           ) :: Nil
