@@ -23,17 +23,23 @@ import org.apache.paimon.spark.{PaimonAppendBucketedTableTest, PaimonAppendNonBu
 class MergeIntoPrimaryKeyBucketedTableTest
   extends MergeIntoTableTestBase
   with MergeIntoPrimaryKeyTableTest
+  with MergeIntoNotMatchedBySourceTest
   with PaimonPrimaryKeyBucketedTableTest {}
 
 class MergeIntoPrimaryKeyNonBucketTableTest
   extends MergeIntoTableTestBase
   with MergeIntoPrimaryKeyTableTest
+  with MergeIntoNotMatchedBySourceTest
   with PaimonPrimaryKeyNonBucketTableTest {}
 
 class MergeIntoAppendBucketedTableTest
   extends MergeIntoTableTestBase
+  with MergeIntoAppendTableTest
+  with MergeIntoNotMatchedBySourceTest
   with PaimonAppendBucketedTableTest {}
 
 class MergeIntoAppendNonBucketedTableTest
   extends MergeIntoTableTestBase
+  with MergeIntoAppendTableTest
+  with MergeIntoNotMatchedBySourceTest
   with PaimonAppendNonBucketTableTest {}
