@@ -18,7 +18,6 @@
 
 package org.apache.paimon.spark.catalog;
 
-import org.apache.paimon.catalog.Catalog;
 import org.apache.paimon.spark.SparkProcedures;
 import org.apache.paimon.spark.SparkSource;
 import org.apache.paimon.spark.analysis.NoSuchProcedureException;
@@ -28,9 +27,16 @@ import org.apache.paimon.spark.procedure.ProcedureBuilder;
 import org.apache.spark.sql.connector.catalog.Identifier;
 import org.apache.spark.sql.connector.catalog.SupportsNamespaces;
 import org.apache.spark.sql.connector.catalog.TableCatalog;
+import org.apache.spark.sql.connector.catalog.TableCatalogCapability;
 
 import javax.annotation.Nullable;
 
+import java.util.Collections;
+import java.util.Set;
+
+import static org.apache.paimon.catalog.Catalog.SYSTEM_DATABASE_NAME;
+import static org.apache.spark.sql.connector.catalog.TableCatalogCapability.SUPPORT_COLUMN_DEFAULT_VALUE;
+
 /** Spark base catalog. */
 public abstract class SparkBaseCatalog
         implements TableCatalog, SupportsNamespaces, ProcedureCatalog, WithPaimonCatalog {
@@ -42,9 +48,13 @@ public abstract class SparkBaseCatalog
         return catalogName;
     }
 
+    public Set<TableCatalogCapability> capabilities() {
+        return Collections.singleton(SUPPORT_COLUMN_DEFAULT_VALUE);
+    }
+
     @Override
     public Procedure loadProcedure(Identifier identifier) throws NoSuchProcedureException {
-        if (Catalog.SYSTEM_DATABASE_NAME.equals(identifier.namespace()[0])) {
+        if (isSystemNamespace(identifier.namespace())) {
             ProcedureBuilder builder = SparkProcedures.newBuilder(identifier.name());
             if (builder != null) {
                 return builder.withTableCatalog(this).build();
@@ -53,7 +63,11 @@ public abstract class SparkBaseCatalog
         throw new NoSuchProcedureException(identifier);
     }
 
-    public boolean usePaimon(@Nullable String provider) {
+    public static boolean usePaimon(@Nullable String provider) {
         return provider == null || SparkSource.NAME().equalsIgnoreCase(provider);
     }
+
+    public static boolean isSystemNamespace(String[] namespace) {
+        return namespace.length == 1 && namespace[0].equalsIgnoreCase(SYSTEM_DATABASE_NAME);
+    }
 }
