@@ -23,29 +23,37 @@ import org.apache.paimon.CoreOptions.OrderType;
 import org.apache.paimon.annotation.VisibleForTesting;
 import org.apache.paimon.append.AppendCompactCoordinator;
 import org.apache.paimon.append.AppendCompactTask;
+import org.apache.paimon.append.cluster.IncrementalClusterManager;
+import org.apache.paimon.compact.CompactUnit;
 import org.apache.paimon.data.BinaryRow;
 import org.apache.paimon.disk.IOManager;
+import org.apache.paimon.io.CompactIncrement;
+import org.apache.paimon.io.DataFileMeta;
+import org.apache.paimon.io.DataIncrement;
 import org.apache.paimon.manifest.PartitionEntry;
 import org.apache.paimon.operation.BaseAppendFileStoreWrite;
+import org.apache.paimon.partition.PartitionPredicate;
 import org.apache.paimon.predicate.Predicate;
-import org.apache.paimon.spark.PaimonSplitScan;
 import org.apache.paimon.spark.SparkUtils;
-import org.apache.paimon.spark.catalyst.Compatibility;
 import org.apache.paimon.spark.catalyst.analysis.expressions.ExpressionUtils;
 import org.apache.paimon.spark.commands.PaimonSparkWriter;
 import org.apache.paimon.spark.sort.TableSorter;
+import org.apache.paimon.spark.util.ScanPlanHelper$;
 import org.apache.paimon.table.BucketMode;
 import org.apache.paimon.table.FileStoreTable;
+import org.apache.paimon.table.SpecialFields;
 import org.apache.paimon.table.sink.AppendCompactTaskSerializer;
 import org.apache.paimon.table.sink.BatchTableCommit;
 import org.apache.paimon.table.sink.BatchTableWrite;
 import org.apache.paimon.table.sink.BatchWriteBuilder;
 import org.apache.paimon.table.sink.CommitMessage;
+import org.apache.paimon.table.sink.CommitMessageImpl;
 import org.apache.paimon.table.sink.CommitMessageSerializer;
 import org.apache.paimon.table.sink.TableCommitImpl;
 import org.apache.paimon.table.source.DataSplit;
 import org.apache.paimon.table.source.EndOfScanException;
 import org.apache.paimon.table.source.snapshot.SnapshotReader;
+import org.apache.paimon.types.RowType;
 import org.apache.paimon.utils.Pair;
 import org.apache.paimon.utils.ParameterUtils;
 import org.apache.paimon.utils.ProcedureUtils;
@@ -90,6 +98,9 @@ import java.util.Optional;
 import java.util.Set;
 import java.util.stream.Collectors;
 
+import scala.collection.JavaConverters;
+import scala.collection.Seq;
+
 import static org.apache.paimon.CoreOptions.createCommitUser;
 import static org.apache.paimon.utils.Preconditions.checkArgument;
 import static org.apache.spark.sql.types.DataTypes.StringType;
@@ -144,8 +155,7 @@ public class CompactProcedure extends BaseProcedure {
     public InternalRow[] call(InternalRow args) {
         Identifier tableIdent = toIdentifier(args.getString(0), PARAMETERS[0].name());
         String partitions = blank(args, 1) ? null : args.getString(1);
-        // make full compact strategy as default.
-        String compactStrategy = blank(args, 2) ? FULL : args.getString(2);
+        String compactStrategy = blank(args, 2) ? null : args.getString(2);
         String sortType = blank(args, 3) ? OrderType.NONE.name() : args.getString(3);
         List<String> sortColumns =
                 blank(args, 4)
@@ -164,7 +174,9 @@ public class CompactProcedure extends BaseProcedure {
                     "sort compact do not support 'partition_idle_time'.");
         }
 
-        if (!(compactStrategy.equalsIgnoreCase(FULL) || compactStrategy.equalsIgnoreCase(MINOR))) {
+        if (!(compactStrategy == null
+                || compactStrategy.equalsIgnoreCase(FULL)
+                || compactStrategy.equalsIgnoreCase(MINOR))) {
             throw new IllegalArgumentException(
                     String.format(
                             "The compact strategy only supports 'full' or 'minor', but '%s' is configured.",
@@ -203,6 +215,12 @@ public class CompactProcedure extends BaseProcedure {
                             dynamicOptions, CoreOptions.WRITE_ONLY.key(), "false");
                     ProcedureUtils.putAllOptions(dynamicOptions, options);
                     table = table.copy(dynamicOptions);
+                    if (((FileStoreTable) table).coreOptions().clusteringIncrementalEnabled()
+                            && (!OrderType.NONE.name().equals(sortType))) {
+                        throw new IllegalArgumentException(
+                                "The table has enabled incremental clustering, do not support sort compact.");
+                    }
+
                     InternalRow internalRow =
                             newInternalRow(
                                     execute(
@@ -236,26 +254,46 @@ public class CompactProcedure extends BaseProcedure {
             @Nullable Duration partitionIdleTime) {
         BucketMode bucketMode = table.bucketMode();
         OrderType orderType = OrderType.of(sortType);
+
+        boolean clusterIncrementalEnabled = table.coreOptions().clusteringIncrementalEnabled();
+        if (compactStrategy == null) {
+            // make full compact strategy as default for compact.
+            // make non-full compact strategy as default for incremental clustering.
+            compactStrategy = clusterIncrementalEnabled ? MINOR : FULL;
+        }
         boolean fullCompact = compactStrategy.equalsIgnoreCase(FULL);
-        Predicate filter =
+        RowType partitionType = table.schema().logicalPartitionType();
+        Predicate partitionFilter =
                 condition == null
                         ? null
                         : ExpressionUtils.convertConditionToPaimonPredicate(
                                         condition,
                                         ((LogicalPlan) relation).output(),
-                                        table.rowType(),
+                                        partitionType,
                                         false)
                                 .getOrElse(null);
+        PartitionPredicate partitionPredicate =
+                PartitionPredicate.fromPredicate(partitionType, partitionFilter);
+
         if (orderType.equals(OrderType.NONE)) {
             JavaSparkContext javaSparkContext = new JavaSparkContext(spark().sparkContext());
             switch (bucketMode) {
                 case HASH_FIXED:
                 case HASH_DYNAMIC:
                     compactAwareBucketTable(
-                            table, fullCompact, filter, partitionIdleTime, javaSparkContext);
+                            table,
+                            fullCompact,
+                            partitionPredicate,
+                            partitionIdleTime,
+                            javaSparkContext);
                     break;
                 case BUCKET_UNAWARE:
-                    compactUnAwareBucketTable(table, filter, partitionIdleTime, javaSparkContext);
+                    if (clusterIncrementalEnabled) {
+                        clusterIncrementalUnAwareBucketTable(table, fullCompact, relation);
+                    } else {
+                        compactUnAwareBucketTable(
+                                table, partitionPredicate, partitionIdleTime, javaSparkContext);
+                    }
                     break;
                 default:
                     throw new UnsupportedOperationException(
@@ -264,7 +302,8 @@ public class CompactProcedure extends BaseProcedure {
         } else {
             switch (bucketMode) {
                 case BUCKET_UNAWARE:
-                    sortCompactUnAwareBucketTable(table, orderType, sortColumns, relation, filter);
+                    sortCompactUnAwareBucketTable(
+                            table, orderType, sortColumns, relation, partitionFilter);
                     break;
                 default:
                     throw new UnsupportedOperationException(
@@ -279,12 +318,12 @@ public class CompactProcedure extends BaseProcedure {
     private void compactAwareBucketTable(
             FileStoreTable table,
             boolean fullCompact,
-            @Nullable Predicate filter,
+            @Nullable PartitionPredicate partitionPredicate,
             @Nullable Duration partitionIdleTime,
             JavaSparkContext javaSparkContext) {
         SnapshotReader snapshotReader = table.newSnapshotReader();
-        if (filter != null) {
-            snapshotReader.withFilter(filter);
+        if (partitionPredicate != null) {
+            snapshotReader.withPartitionFilter(partitionPredicate);
         }
         Set<BinaryRow> partitionToBeCompacted =
                 getHistoryPartition(snapshotReader, partitionIdleTime);
@@ -358,12 +397,12 @@ public class CompactProcedure extends BaseProcedure {
 
     private void compactUnAwareBucketTable(
             FileStoreTable table,
-            @Nullable Predicate filter,
+            @Nullable PartitionPredicate partitionPredicate,
             @Nullable Duration partitionIdleTime,
             JavaSparkContext javaSparkContext) {
         List<AppendCompactTask> compactionTasks;
         try {
-            compactionTasks = new AppendCompactCoordinator(table, false, filter).run();
+            compactionTasks = new AppendCompactCoordinator(table, false, partitionPredicate).run();
         } catch (EndOfScanException e) {
             compactionTasks = new ArrayList<>();
         }
@@ -411,6 +450,12 @@ public class CompactProcedure extends BaseProcedure {
                                             BaseAppendFileStoreWrite write =
                                                     (BaseAppendFileStoreWrite)
                                                             table.store().newWrite(commitUser);
+                                            CoreOptions coreOptions = table.coreOptions();
+                                            if (coreOptions.rowTrackingEnabled()) {
+                                                write.withWriteType(
+                                                        SpecialFields.rowTypeWithRowTracking(
+                                                                table.rowType()));
+                                            }
                                             AppendCompactTaskSerializer ser =
                                                     new AppendCompactTaskSerializer();
                                             List<byte[]> messages = new ArrayList<>();
@@ -477,10 +522,10 @@ public class CompactProcedure extends BaseProcedure {
             OrderType orderType,
             List<String> sortColumns,
             DataSourceV2Relation relation,
-            @Nullable Predicate filter) {
+            @Nullable Predicate partitionFilter) {
         SnapshotReader snapshotReader = table.newSnapshotReader();
-        if (filter != null) {
-            snapshotReader.withFilter(filter);
+        if (partitionFilter != null) {
+            snapshotReader.withPartitionFilter(partitionFilter);
         }
         Map<BinaryRow, DataSplit[]> packedSplits = packForSort(snapshotReader.read().dataSplits());
         TableSorter sorter = TableSorter.getSorter(table, orderType, sortColumns);
@@ -491,22 +536,113 @@ public class CompactProcedure extends BaseProcedure {
                                     Dataset<Row> dataset =
                                             PaimonUtils.createDataset(
                                                     spark(),
-                                                    Compatibility.createDataSourceV2ScanRelation(
-                                                            relation,
-                                                            PaimonSplitScan.apply(table, split),
-                                                            relation.output()));
+                                                    ScanPlanHelper$.MODULE$.createNewScanPlan(
+                                                            split, relation));
                                     return sorter.sort(dataset);
                                 })
                         .reduce(Dataset::union)
                         .orElse(null);
         if (datasetForWrite != null) {
-            PaimonSparkWriter writer = new PaimonSparkWriter(table);
+            PaimonSparkWriter writer = PaimonSparkWriter.apply(table);
             // Use dynamic partition overwrite
             writer.writeBuilder().withOverwrite();
             writer.commit(writer.write(datasetForWrite));
         }
     }
 
+    private void clusterIncrementalUnAwareBucketTable(
+            FileStoreTable table, boolean fullCompaction, DataSourceV2Relation relation) {
+        IncrementalClusterManager incrementalClusterManager = new IncrementalClusterManager(table);
+        Map<BinaryRow, CompactUnit> compactUnits =
+                incrementalClusterManager.prepareForCluster(fullCompaction);
+
+        // generate splits for each partition
+        Map<BinaryRow, DataSplit[]> partitionSplits =
+                compactUnits.entrySet().stream()
+                        .collect(
+                                Collectors.toMap(
+                                        Map.Entry::getKey,
+                                        entry ->
+                                                incrementalClusterManager
+                                                        .toSplits(
+                                                                entry.getKey(),
+                                                                entry.getValue().files())
+                                                        .toArray(new DataSplit[0])));
+
+        // sort in partition
+        TableSorter sorter =
+                TableSorter.getSorter(
+                        table,
+                        incrementalClusterManager.clusterCurve(),
+                        incrementalClusterManager.clusterKeys());
+        LOG.info(
+                "Start to sort in partition, cluster curve is {}, cluster keys is {}",
+                incrementalClusterManager.clusterCurve(),
+                incrementalClusterManager.clusterKeys());
+
+        Dataset<Row> datasetForWrite =
+                partitionSplits.values().stream()
+                        .map(
+                                split -> {
+                                    Dataset<Row> dataset =
+                                            PaimonUtils.createDataset(
+                                                    spark(),
+                                                    ScanPlanHelper$.MODULE$.createNewScanPlan(
+                                                            split, relation));
+                                    return sorter.sort(dataset);
+                                })
+                        .reduce(Dataset::union)
+                        .orElse(null);
+        if (datasetForWrite != null) {
+            // set to write only to prevent invoking compaction
+            // do not use overwrite, we don't need to overwrite the whole partition
+            PaimonSparkWriter writer = PaimonSparkWriter.apply(table).writeOnly();
+            Seq<CommitMessage> commitMessages = writer.write(datasetForWrite);
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Commit messages after writing:{}", commitMessages);
+            }
+
+            // re-organize the commit messages to generate the compact messages
+            Map<BinaryRow, List<DataFileMeta>> partitionClustered = new HashMap<>();
+            for (CommitMessage commitMessage : JavaConverters.seqAsJavaList(commitMessages)) {
+                checkArgument(commitMessage.bucket() == 0);
+                partitionClustered
+                        .computeIfAbsent(commitMessage.partition(), k -> new ArrayList<>())
+                        .addAll(((CommitMessageImpl) commitMessage).newFilesIncrement().newFiles());
+            }
+
+            List<CommitMessage> clusterMessages = new ArrayList<>();
+            for (Map.Entry<BinaryRow, List<DataFileMeta>> entry : partitionClustered.entrySet()) {
+                BinaryRow partition = entry.getKey();
+                List<DataFileMeta> clusterBefore = compactUnits.get(partition).files();
+                // upgrade the clustered file to outputLevel
+                List<DataFileMeta> clusterAfter =
+                        IncrementalClusterManager.upgrade(
+                                entry.getValue(), compactUnits.get(partition).outputLevel());
+                LOG.info(
+                        "Partition {}: upgrade file level to {}",
+                        partition,
+                        compactUnits.get(partition).outputLevel());
+                CompactIncrement compactIncrement =
+                        new CompactIncrement(clusterBefore, clusterAfter, Collections.emptyList());
+                clusterMessages.add(
+                        new CommitMessageImpl(
+                                partition,
+                                // bucket 0 is bucket for unaware-bucket table
+                                // for compatibility with the old design
+                                0,
+                                table.coreOptions().bucket(),
+                                DataIncrement.emptyIncrement(),
+                                compactIncrement));
+            }
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Commit messages after reorganizing:{}", clusterMessages);
+            }
+
+            writer.commit(JavaConverters.asScalaBuffer(clusterMessages).toSeq());
+        }
+    }
+
     private Map<BinaryRow, DataSplit[]> packForSort(List<DataSplit> dataSplits) {
         // Make a single partition as a compact group
         return dataSplits.stream()
