@@ -18,25 +18,30 @@
 
 package org.apache.paimon.spark
 
-import org.apache.paimon.predicate.{PartitionPredicateVisitor, Predicate, PredicateBuilder}
-import org.apache.paimon.spark.aggregate.{AggregatePushDownUtils, LocalAggregator}
-import org.apache.paimon.table.{FileStoreTable, Table}
-import org.apache.paimon.table.source.DataSplit
+import org.apache.paimon.predicate._
+import org.apache.paimon.predicate.SortValue.{NullOrdering, SortDirection}
+import org.apache.paimon.spark.aggregate.AggregatePushDownUtils.tryPushdownAggregation
+import org.apache.paimon.table.{FileStoreTable, InnerTable}
 
 import org.apache.spark.sql.PaimonUtils
+import org.apache.spark.sql.connector.expressions
+import org.apache.spark.sql.connector.expressions.{NamedReference, SortOrder}
 import org.apache.spark.sql.connector.expressions.aggregate.Aggregation
 import org.apache.spark.sql.connector.expressions.filter.{Predicate => SparkPredicate}
-import org.apache.spark.sql.connector.read.{Scan, SupportsPushDownAggregates, SupportsPushDownLimit, SupportsPushDownV2Filters}
+import org.apache.spark.sql.connector.read._
 import org.apache.spark.sql.sources.Filter
 
+import java.util.Collections
+
 import scala.collection.JavaConverters._
 import scala.collection.mutable
 
-class PaimonScanBuilder(table: Table)
+class PaimonScanBuilder(table: InnerTable)
   extends PaimonBaseScanBuilder(table)
   with SupportsPushDownV2Filters
   with SupportsPushDownLimit
-  with SupportsPushDownAggregates {
+  with SupportsPushDownAggregates
+  with SupportsPushDownTopN {
 
   private var localScan: Option[Scan] = None
 
@@ -90,6 +95,55 @@ class PaimonScanBuilder(table: Table)
     false
   }
 
+  override def pushTopN(orders: Array[SortOrder], limit: Int): Boolean = {
+    if (hasPostScanPredicates) {
+      return false
+    }
+
+    if (!table.isInstanceOf[FileStoreTable]) {
+      return false
+    }
+
+    val sorts: List[SortValue] = orders
+      .map(
+        order => {
+          val fieldName = order.expression() match {
+            case nr: NamedReference => nr.fieldNames.mkString(".")
+            case _ => return false
+          }
+
+          val rowType = table.rowType()
+          if (rowType.notContainsField(fieldName)) {
+            return false
+          }
+
+          val field = rowType.getField(fieldName)
+          val ref = new FieldRef(field.id(), field.name(), field.`type`())
+
+          val nullOrdering = order.nullOrdering() match {
+            case expressions.NullOrdering.NULLS_LAST => NullOrdering.NULLS_LAST
+            case expressions.NullOrdering.NULLS_FIRST => NullOrdering.NULLS_FIRST
+            case _ => return false
+          }
+
+          val direction = order.direction() match {
+            case expressions.SortDirection.DESCENDING => SortDirection.DESCENDING
+            case expressions.SortDirection.ASCENDING => SortDirection.ASCENDING
+            case _ => return false
+          }
+
+          new SortValue(ref, direction, nullOrdering)
+        })
+      .toList
+
+    pushDownTopN = Some(new TopN(sorts.asJava, limit))
+
+    // just make the best effort to push down TopN
+    false
+  }
+
+  override def isPartiallyPushed: Boolean = super.isPartiallyPushed
+
   override def supportCompletePushDown(aggregation: Aggregation): Boolean = {
     // for now, we only support complete push down, so there is no difference with `pushAggregation`
     pushAggregation(aggregation)
@@ -115,25 +169,14 @@ class PaimonScanBuilder(table: Table)
       val pushedPartitionPredicate = PredicateBuilder.and(pushedPaimonPredicates.toList.asJava)
       readBuilder.withFilter(pushedPartitionPredicate)
     }
-    val dataSplits = if (AggregatePushDownUtils.hasMinMaxAggregation(aggregation)) {
-      readBuilder.newScan().plan().splits().asScala.map(_.asInstanceOf[DataSplit])
-    } else {
-      readBuilder.dropStats().newScan().plan().splits().asScala.map(_.asInstanceOf[DataSplit])
-    }
-    if (AggregatePushDownUtils.canPushdownAggregation(table, aggregation, dataSplits.toSeq)) {
-      val aggregator = new LocalAggregator(table.asInstanceOf[FileStoreTable])
-      aggregator.initialize(aggregation)
-      dataSplits.foreach(aggregator.update)
-      localScan = Some(
-        PaimonLocalScan(
-          aggregator.result(),
-          aggregator.resultSchema(),
-          table,
-          pushedPaimonPredicates)
-      )
-      true
-    } else {
-      false
+
+    tryPushdownAggregation(table.asInstanceOf[FileStoreTable], aggregation, readBuilder) match {
+      case Some(agg) =>
+        localScan = Some(
+          PaimonLocalScan(agg.result(), agg.resultSchema(), table, pushedPaimonPredicates)
+        )
+        true
+      case None => false
     }
   }
 
