@@ -18,30 +18,33 @@
 
 package org.apache.paimon.spark.commands
 
-import org.apache.paimon.CoreOptions.WRITE_ONLY
+import org.apache.paimon.{CoreOptions, Snapshot}
+import org.apache.paimon.CoreOptions.{PartitionSinkStrategy, WRITE_ONLY}
 import org.apache.paimon.codegen.CodeGenUtils
 import org.apache.paimon.crosspartition.{IndexBootstrap, KeyPartOrRow}
 import org.apache.paimon.data.serializer.InternalSerializers
 import org.apache.paimon.deletionvectors.DeletionVector
 import org.apache.paimon.deletionvectors.append.BaseAppendDeleteFileMaintainer
+import org.apache.paimon.fs.Path
 import org.apache.paimon.index.{BucketAssigner, SimpleHashBucketAssigner}
-import org.apache.paimon.io.{CompactIncrement, DataIncrement, IndexIncrement}
+import org.apache.paimon.io.{CompactIncrement, DataIncrement}
 import org.apache.paimon.manifest.FileKind
 import org.apache.paimon.spark.{SparkRow, SparkTableWrite, SparkTypeUtils}
 import org.apache.paimon.spark.catalog.functions.BucketFunction
 import org.apache.paimon.spark.schema.SparkSystemColumns.{BUCKET_COL, ROW_KIND_COL}
+import org.apache.paimon.spark.sort.TableSorter
 import org.apache.paimon.spark.util.OptionUtils.paimonExtensionEnabled
 import org.apache.paimon.spark.util.SparkRowUtils
 import org.apache.paimon.spark.write.WriteHelper
+import org.apache.paimon.table.{FileStoreTable, SpecialFields}
 import org.apache.paimon.table.BucketMode._
-import org.apache.paimon.table.FileStoreTable
 import org.apache.paimon.table.sink._
 import org.apache.paimon.types.{RowKind, RowType}
 import org.apache.paimon.utils.SerializationUtils
 
 import org.apache.spark.{Partitioner, TaskContext}
 import org.apache.spark.rdd.RDD
-import org.apache.spark.sql.{DataFrame, Dataset, Row, SparkSession}
+import org.apache.spark.sql._
 import org.apache.spark.sql.functions._
 
 import java.io.IOException
@@ -49,29 +52,44 @@ import java.util.Collections.singletonMap
 
 import scala.collection.JavaConverters._
 
-case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
+case class PaimonSparkWriter(table: FileStoreTable, writeRowTracking: Boolean = false)
+  extends WriteHelper {
 
   private lazy val tableSchema = table.schema
 
-  private lazy val rowType = table.rowType()
-
   private lazy val bucketMode = table.bucketMode
 
   @transient private lazy val serializer = new CommitMessageSerializer
 
+  private val writeType = {
+    if (writeRowTracking) {
+      SpecialFields.rowTypeWithRowTracking(table.rowType(), true)
+    } else {
+      table.rowType()
+    }
+  }
+
   val writeBuilder: BatchWriteBuilder = table.newBatchWriteBuilder()
 
   def writeOnly(): PaimonSparkWriter = {
     PaimonSparkWriter(table.copy(singletonMap(WRITE_ONLY.key(), "true")))
   }
 
+  def withRowTracking(): PaimonSparkWriter = {
+    if (coreOptions.rowTrackingEnabled()) {
+      PaimonSparkWriter(table, writeRowTracking = true)
+    } else {
+      this
+    }
+  }
+
   def write(data: DataFrame): Seq[CommitMessage] = {
     val sparkSession = data.sparkSession
     import sparkSession.implicits._
 
     val withInitBucketCol = bucketMode match {
       case BUCKET_UNAWARE => data
-      case CROSS_PARTITION if !data.schema.fieldNames.contains(ROW_KIND_COL) =>
+      case KEY_DYNAMIC if !data.schema.fieldNames.contains(ROW_KIND_COL) =>
         data
           .withColumn(ROW_KIND_COL, lit(RowKind.INSERT.toByteValue))
           .withColumn(BUCKET_COL, lit(-1))
@@ -81,7 +99,7 @@ case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
     val bucketColIdx = SparkRowUtils.getFieldIndex(withInitBucketCol.schema, BUCKET_COL)
     val encoderGroupWithBucketCol = EncoderSerDeGroup(withInitBucketCol.schema)
 
-    def newWrite(): SparkTableWrite = new SparkTableWrite(writeBuilder, rowType, rowKindColIdx)
+    def newWrite() = SparkTableWrite(writeBuilder, writeType, rowKindColIdx, writeRowTracking)
 
     def sparkParallelism = {
       val defaultParallelism = sparkSession.sparkContext.defaultParallelism
@@ -148,7 +166,7 @@ case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
     }
 
     val written: Dataset[Array[Byte]] = bucketMode match {
-      case CROSS_PARTITION =>
+      case KEY_DYNAMIC =>
         // Topology: input -> bootstrap -> shuffle by key hash -> bucket-assigner -> shuffle by partition & bucket
         val rowType = SparkTypeUtils.toPaimonType(withInitBucketCol.schema).asInstanceOf[RowType]
         val assignerParallelism = Option(coreOptions.dynamicBucketAssignerParallelism)
@@ -211,7 +229,7 @@ case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
                   coreOptions.dynamicBucketMaxBuckets
                 )
               row => {
-                val sparkRow = new SparkRow(rowType, row)
+                val sparkRow = new SparkRow(writeType, row)
                 assigner.assign(
                   extractor.partition(sparkRow),
                   extractor.trimmedPrimaryKey(sparkRow).hashCode)
@@ -232,7 +250,21 @@ case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
         }
 
       case BUCKET_UNAWARE | POSTPONE_MODE =>
-        writeWithoutBucket(data)
+        var input = data
+        if (tableSchema.partitionKeys().size() > 0) {
+          coreOptions.partitionSinkStrategy match {
+            case PartitionSinkStrategy.HASH =>
+              input = data.repartition(partitionCols(data): _*)
+            case _ =>
+          }
+        }
+        val clusteringColumns = coreOptions.clusteringColumns()
+        if ((!coreOptions.clusteringIncrementalEnabled()) && (!clusteringColumns.isEmpty)) {
+          val strategy = coreOptions.clusteringStrategy(tableSchema.fields().size())
+          val sorter = TableSorter.getSorter(table, strategy, clusteringColumns)
+          input = sorter.sort(data)
+        }
+        writeWithoutBucket(input)
 
       case HASH_FIXED =>
         if (paimonExtensionEnabled && BucketFunction.supportsTable(table)) {
@@ -244,7 +276,9 @@ case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
             .map(tableSchema.fieldNames().indexOf(_))
             .map(x => col(data.schema.fieldNames(x)))
             .toSeq
-          val args = Seq(lit(bucketNumber)) ++ bucketKeyCol
+          val args = Seq(
+            lit(new CoreOptions(tableSchema.options()).bucketFunctionType().toString),
+            lit(bucketNumber)) ++ bucketKeyCol
           val repartitioned =
             repartitionByPartitionsAndBucket(
               data.withColumn(BUCKET_COL, call_udf(BucketExpression.FIXED_BUCKET, args: _*)))
@@ -268,15 +302,16 @@ case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
 
   /**
    * Write all the deletion vectors to the index files. If it's in unaware mode, one index file maps
-   * deletion vectors; else, one index file will contains all deletion vector with the same
-   * partition and bucket.
+   * deletion vectors; else, one index file will contain all deletion vector with the same partition
+   * and bucket.
    */
-  def persistDeletionVectors(deletionVectors: Dataset[SparkDeletionVector]): Seq[CommitMessage] = {
+  def persistDeletionVectors(
+      deletionVectors: Dataset[SparkDeletionVector],
+      snapshot: Snapshot): Seq[CommitMessage] = {
     val sparkSession = deletionVectors.sparkSession
     import sparkSession.implicits._
-    val snapshot = table.snapshotManager().latestSnapshotFromFileSystem()
     val serializedCommits = deletionVectors
-      .groupByKey(_.partitionAndBucket)
+      .groupByKey(_.bucketPath)
       .mapGroups {
         (_, iter: Iterator[SparkDeletionVector]) =>
           val indexHandler = table.store().newIndexFileHandler()
@@ -300,7 +335,7 @@ case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
             }
 
             dvIndexFileMaintainer.notifyNewDeletionVector(
-              sdv.dataFileName,
+              new Path(sdv.dataFilePath).getName,
               DeletionVector.deserializeFromBytes(sdv.deletionVector))
           }
           val indexEntries = dvIndexFileMaintainer.persist()
@@ -311,9 +346,13 @@ case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
             dvIndexFileMaintainer.getPartition,
             dvIndexFileMaintainer.getBucket,
             null,
-            DataIncrement.emptyIncrement(),
-            CompactIncrement.emptyIncrement(),
-            new IndexIncrement(added.map(_.indexFile).asJava, deleted.map(_.indexFile).asJava)
+            new DataIncrement(
+              java.util.Collections.emptyList(),
+              java.util.Collections.emptyList(),
+              java.util.Collections.emptyList(),
+              added.map(_.indexFile).asJava,
+              deleted.map(_.indexFile).asJava),
+            CompactIncrement.emptyIncrement()
           )
           val serializer = new CommitMessageSerializer
           serializer.serialize(commitMessage)
@@ -357,6 +396,7 @@ case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
               .bootstrap(numSparkPartitions, sparkPartitionId)
               .toCloseableIterator
             TaskContext.get().addTaskCompletionListener[Unit](_ => bootstrapIterator.close())
+            val toPaimonRow = SparkRowUtils.toPaimonRow(rowType, rowKindColIdx)
 
             bootstrapIterator.asScala
               .map(
@@ -366,8 +406,7 @@ case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
                   (keyPartProject(row).hashCode(), (KeyPartOrRow.KEY_PART, bytes))
                 }) ++ iter.map(
               r => {
-                val sparkRow =
-                  new SparkRow(rowType, r, SparkRowUtils.getRowKind(r, rowKindColIdx))
+                val sparkRow = toPaimonRow(r)
                 val bytes: Array[Byte] =
                   SerializationUtils.serializeBinaryRow(rowSer.toBinaryRow(sparkRow))
                 (rowProject(sparkRow).hashCode(), (KeyPartOrRow.ROW, bytes))
@@ -391,7 +430,7 @@ case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
             val rowPartitionKeyExtractor = new RowPartitionKeyExtractor(tableSchema)
             iterator.map(
               row => {
-                val sparkRow = new SparkRow(rowType, row)
+                val sparkRow = new SparkRow(writeType, row)
                 val partitionHash = rowPartitionKeyExtractor.partition(sparkRow).hashCode
                 val keyHash = rowPartitionKeyExtractor.trimmedPrimaryKey(sparkRow).hashCode
                 (
@@ -407,14 +446,17 @@ case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
   }
 
   private def repartitionByPartitionsAndBucket(df: DataFrame): DataFrame = {
+    df.repartition(partitionCols(df) ++ Seq(col(BUCKET_COL)): _*)
+  }
+
+  def partitionCols(df: DataFrame): Seq[Column] = {
     val inputSchema = df.schema
-    val partitionCols = tableSchema
+    tableSchema
       .partitionKeys()
       .asScala
       .map(tableSchema.fieldNames().indexOf(_))
       .map(x => col(inputSchema.fieldNames(x)))
       .toSeq
-    df.repartition(partitionCols ++ Seq(col(BUCKET_COL)): _*)
   }
 
   private def deserializeCommitMessage(
@@ -433,3 +475,10 @@ case class PaimonSparkWriter(table: FileStoreTable) extends WriteHelper {
     override def getPartition(key: Any): Int = Math.abs(key.asInstanceOf[Int] % numPartitions)
   }
 }
+
+object PaimonSparkWriter {
+
+  def apply(table: FileStoreTable): PaimonSparkWriter = {
+    new PaimonSparkWriter(table)
+  }
+}
