@@ -18,28 +18,18 @@
 
 package org.apache.paimon.spark.util
 
-import org.apache.paimon.types.RowKind
+import org.apache.paimon.spark.SparkRow
+import org.apache.paimon.types.{RowKind, RowType}
 
 import org.apache.spark.sql.Row
-import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.types.StructType
 
 object SparkRowUtils {
 
-  def getRowKind(row: Row, rowkindColIdx: Int): RowKind = {
+  def toPaimonRow(writeType: RowType, rowkindColIdx: Int): Row => SparkRow = {
     if (rowkindColIdx != -1) {
-      RowKind.fromByteValue(row.getByte(rowkindColIdx))
-    } else {
-      RowKind.INSERT
-    }
-  }
-
-  def getRowKind(row: InternalRow, rowkindColIdx: Int): RowKind = {
-    if (rowkindColIdx != -1) {
-      RowKind.fromByteValue(row.getByte(rowkindColIdx))
-    } else {
-      RowKind.INSERT
-    }
+      row => new SparkRow(writeType, row, RowKind.fromByteValue(row.getByte(rowkindColIdx)))
+    } else { row => new SparkRow(writeType, row) }
   }
 
   def getFieldIndex(schema: StructType, colName: String): Int = {
@@ -50,5 +40,4 @@ object SparkRowUtils {
         -1
     }
   }
-
 }
