@@ -28,6 +28,7 @@ import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.io.TempDir;
 
 import java.io.FileNotFoundException;
+import java.io.IOException;
 
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.assertj.core.api.Assertions.assertThatThrownBy;
@@ -48,7 +49,7 @@ public class SparkGenericCatalogWithHiveTest {
     }
 
     @Test
-    public void testBuildWithHive(@TempDir java.nio.file.Path tempDir) {
+    public void testBuildWithHive(@TempDir java.nio.file.Path tempDir) throws IOException {
         // firstly, we use hive metastore to create table, and check the result.
         Path warehousePath = new Path("file:" + tempDir.toString());
         SparkSession spark =
@@ -59,12 +60,15 @@ public class SparkGenericCatalogWithHiveTest {
                         .config(
                                 "spark.sql.catalog.spark_catalog",
                                 SparkGenericCatalog.class.getName())
+                        .config("spark.sql.catalog.paimon.warehouse", warehousePath.toString())
+                        .config("spark.sql.catalog.paimon", SparkCatalog.class.getName())
                         .config(
                                 "spark.sql.extensions",
                                 "org.apache.paimon.spark.extensions.PaimonSparkSessionExtensions")
                         .master("local[2]")
                         .getOrCreate();
 
+        spark.sql("USE spark_catalog");
         spark.sql("CREATE DATABASE my_db");
         spark.sql("USE my_db");
         spark.sql(
@@ -79,33 +83,26 @@ public class SparkGenericCatalogWithHiveTest {
                                 .map(s -> s.get(1))
                                 .map(Object::toString))
                 .containsExactlyInAnyOrder("t1");
-        spark.close();
 
-        // secondly, we close catalog with hive metastore, and start a filesystem metastore to check
-        // the result.
-        SparkSession spark2 =
-                SparkSession.builder()
-                        .config("spark.sql.catalog.paimon.warehouse", warehousePath.toString())
-                        .config("spark.sql.catalogImplementation", "in-memory")
-                        .config("spark.sql.catalog.paimon", SparkCatalog.class.getName())
-                        .config(
-                                "spark.sql.extensions",
-                                "org.apache.paimon.spark.extensions.PaimonSparkSessionExtensions")
-                        .master("local[2]")
-                        .getOrCreate();
-        spark2.sql("USE paimon");
-        spark2.sql("USE my_db");
-        assertThat(spark2.sql("SHOW NAMESPACES").collectAsList().stream().map(Object::toString))
+        // secondly, use filesystem metastore to check the result.
+        spark.sql("USE paimon");
+        spark.sql("USE my_db");
+        assertThat(spark.sql("SHOW NAMESPACES").collectAsList().stream().map(Object::toString))
                 .containsExactlyInAnyOrder("[default]", "[my_db]");
         assertThat(
-                        spark2.sql("SHOW TABLES").collectAsList().stream()
+                        spark.sql("SHOW TABLES").collectAsList().stream()
                                 .map(s -> s.get(1))
                                 .map(Object::toString))
                 .containsExactlyInAnyOrder("t1");
+
+        spark.sql("USE spark_catalog");
+        spark.sql("DROP TABLE my_db.t1");
+        spark.sql("DROP DATABASE my_db");
+        spark.close();
     }
 
     @Test
-    public void testHiveCatalogOptions(@TempDir java.nio.file.Path tempDir) {
+    public void testHiveCatalogOptions(@TempDir java.nio.file.Path tempDir) throws IOException {
         Path warehousePath = new Path("file:" + tempDir.toString());
         SparkSession spark =
                 SparkSession.builder()
