@@ -25,6 +25,7 @@ import org.apache.paimon.fs.local.LocalFileIO;
 import org.apache.paimon.spark.extensions.PaimonSparkSessionExtensions;
 import org.apache.paimon.table.FileStoreTable;
 import org.apache.paimon.table.FileStoreTableFactory;
+import org.apache.paimon.utils.SnapshotManager;
 
 import org.apache.spark.sql.Row;
 import org.apache.spark.sql.SparkSession;
@@ -35,11 +36,14 @@ import org.junit.jupiter.api.BeforeAll;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.TestInstance;
 import org.junit.jupiter.api.io.TempDir;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.ValueSource;
 
 import java.io.IOException;
 import java.util.Arrays;
 import java.util.Comparator;
 import java.util.List;
+import java.util.Objects;
 import java.util.stream.Collectors;
 
 import static org.assertj.core.api.Assertions.assertThat;
@@ -124,6 +128,228 @@ public class SparkWriteITCase {
                         "[[1,2,my_value], [2,2,my_value], [3,2,my_value], [4,2,my_value], [5,3,my_value]]");
     }
 
+    @Test
+    public void testWriteWithArrayDefaultValue() {
+        // Test Array type default value - using Spark SQL function syntax
+        spark.sql(
+                "CREATE TABLE T (id INT, tags ARRAY<STRING> DEFAULT ARRAY('tag1', 'tag2'), numbers ARRAY<INT> DEFAULT ARRAY(1, 2, 3)) TBLPROPERTIES"
+                        + " ('file.format'='avro')");
+
+        // test show create table for array
+        List<Row> show = spark.sql("SHOW CREATE TABLE T").collectAsList();
+        assertThat(show.toString())
+                .contains("tags ARRAY<STRING> DEFAULT ARRAY('tag1', 'tag2')")
+                .contains("numbers ARRAY<INT> DEFAULT ARRAY(1, 2, 3)");
+
+        // test partial write with array defaults
+        spark.sql("INSERT INTO T (id) VALUES (1), (2)").collectAsList();
+        List<Row> rows = spark.sql("SELECT * FROM T").collectAsList();
+        // Support both Spark 3.x (WrappedArray) and Spark 4.x (ArraySeq) formats
+        assertThat(rows.toString())
+                .satisfiesAnyOf(
+                        s -> assertThat(s).contains("WrappedArray('tag1', 'tag2')"),
+                        s -> assertThat(s).contains("ArraySeq('tag1', 'tag2')"));
+        assertThat(rows.toString())
+                .satisfiesAnyOf(
+                        s -> assertThat(s).contains("WrappedArray(1, 2, 3)"),
+                        s -> assertThat(s).contains("ArraySeq(1, 2, 3)"));
+        assertThat(rows.size()).isEqualTo(2);
+
+        // test write with DEFAULT keyword for arrays
+        spark.sql("INSERT INTO T VALUES (3, DEFAULT, DEFAULT)").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString())
+                .satisfiesAnyOf(
+                        s -> assertThat(s).contains("WrappedArray('tag1', 'tag2')"),
+                        s -> assertThat(s).contains("ArraySeq('tag1', 'tag2')"));
+        assertThat(rows.size()).isEqualTo(3);
+
+        // test empty array default value
+        spark.sql("DROP TABLE IF EXISTS T");
+        spark.sql(
+                "CREATE TABLE T (id INT, empty_array ARRAY<STRING> DEFAULT ARRAY()) TBLPROPERTIES"
+                        + " ('file.format'='avro')");
+
+        spark.sql("INSERT INTO T (id) VALUES (1)").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString())
+                .satisfiesAnyOf(
+                        s -> assertThat(s).contains("WrappedArray()"),
+                        s -> assertThat(s).contains("ArraySeq()"));
+
+        // test write with DEFAULT keyword for empty array
+        spark.sql("INSERT INTO T VALUES (2, DEFAULT)").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString())
+                .satisfiesAnyOf(
+                        s -> assertThat(s).contains("WrappedArray()"),
+                        s -> assertThat(s).contains("ArraySeq()"));
+        assertThat(rows.size()).isEqualTo(2);
+    }
+
+    @Test
+    public void testWriteWithMapDefaultValue() {
+        // Test Map type default value - using Spark SQL function syntax
+        spark.sql(
+                "CREATE TABLE T (id INT, properties MAP<STRING, STRING> DEFAULT MAP('key1', 'value1', 'key2', 'value2')) TBLPROPERTIES"
+                        + " ('file.format'='avro')");
+
+        // test show create table for map
+        List<Row> show = spark.sql("SHOW CREATE TABLE T").collectAsList();
+        assertThat(show.toString())
+                .contains(
+                        "properties MAP<STRING, STRING> DEFAULT MAP('key1', 'value1', 'key2', 'value2')");
+
+        // test partial write with map defaults
+        spark.sql("INSERT INTO T (id) VALUES (1), (2)").collectAsList();
+        List<Row> rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString()).contains("'key1' -> 'value1'");
+        assertThat(rows.toString()).contains("'key2' -> 'value2'");
+        assertThat(rows.size()).isEqualTo(2);
+
+        // test write with DEFAULT keyword for map
+        spark.sql("INSERT INTO T VALUES (3, DEFAULT)").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString()).contains("'key1' -> 'value1'");
+        assertThat(rows.toString()).contains("'key2' -> 'value2'");
+        assertThat(rows.size()).isEqualTo(3);
+
+        // test empty map default value
+        spark.sql("DROP TABLE IF EXISTS T");
+        spark.sql(
+                "CREATE TABLE T (id INT, empty_map MAP<STRING, INT> DEFAULT MAP()) TBLPROPERTIES"
+                        + " ('file.format'='avro')");
+
+        spark.sql("INSERT INTO T (id) VALUES (1)").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString()).contains("Map()");
+
+        // test write with DEFAULT keyword for empty map
+        spark.sql("INSERT INTO T VALUES (2, DEFAULT)").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString()).contains("Map()");
+        assertThat(rows.size()).isEqualTo(2);
+    }
+
+    @Test
+    public void testWriteWithStructDefaultValue() {
+        // Test Struct/Row type default value - using Spark SQL function syntax
+        spark.sql(
+                "CREATE TABLE T (id INT, nested STRUCT<x: INT, y: STRING> DEFAULT STRUCT(42, 'default_value')) TBLPROPERTIES"
+                        + " ('file.format'='avro')");
+
+        // test show create table for struct
+        List<Row> show = spark.sql("SHOW CREATE TABLE T").collectAsList();
+        assertThat(show.toString())
+                .contains("nested STRUCT<x: INT, y: STRING> DEFAULT STRUCT(42, 'default_value')");
+
+        // test partial write with struct defaults
+        spark.sql("INSERT INTO T (id) VALUES (1), (2)").collectAsList();
+        List<Row> rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString()).contains("[42,'default_value']");
+        assertThat(rows.size()).isEqualTo(2);
+
+        // test write with DEFAULT keyword for struct
+        spark.sql("INSERT INTO T VALUES (3, DEFAULT)").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString()).contains("[42,'default_value']");
+        assertThat(rows.size()).isEqualTo(3);
+
+        // test complex struct with multiple types
+        spark.sql("DROP TABLE IF EXISTS T");
+        spark.sql(
+                "CREATE TABLE T (id INT, config STRUCT<enabled: BOOLEAN, timeout: INT, name: STRING> DEFAULT STRUCT(true, 30, 'config_name')) TBLPROPERTIES"
+                        + " ('file.format'='avro')");
+
+        spark.sql("INSERT INTO T (id) VALUES (1)").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString()).contains("[true,30,'config_name']");
+        assertThat(rows.size()).isEqualTo(1);
+    }
+
+    @Test
+    public void testWriteWithNestedComplexDefaultValue() {
+        // Test nested complex types with default values - using Spark SQL function syntax
+        spark.sql(
+                "CREATE TABLE T (id INT, "
+                        + "nested_array ARRAY<STRUCT<name: STRING, value: INT>> DEFAULT ARRAY(STRUCT('item1', 10), STRUCT('item2', 20)), "
+                        + "map_of_arrays MAP<STRING, ARRAY<INT>> DEFAULT MAP('list1', ARRAY(1, 2), 'list2', ARRAY(3, 4))) TBLPROPERTIES"
+                        + " ('file.format'='avro')");
+
+        // test show create table for nested complex types
+        List<Row> show = spark.sql("SHOW CREATE TABLE T").collectAsList();
+        assertThat(show.toString())
+                .contains(
+                        "nested_array ARRAY<STRUCT<name: STRING, value: INT>> DEFAULT ARRAY(STRUCT('item1', 10), STRUCT('item2', 20))")
+                .contains(
+                        "map_of_arrays MAP<STRING, ARRAY<INT>> DEFAULT MAP('list1', ARRAY(1, 2), 'list2', ARRAY(3, 4))");
+
+        // test partial write with nested complex type defaults
+        spark.sql("INSERT INTO T (id) VALUES (1)").collectAsList();
+        List<Row> rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.size()).isEqualTo(1);
+        assertThat(rows.get(0).getInt(0)).isEqualTo(1);
+
+        // test write with DEFAULT keyword for nested complex types
+        spark.sql("INSERT INTO T VALUES (2, DEFAULT, DEFAULT)").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.size()).isEqualTo(2);
+
+        // test mixed simple and complex types with defaults
+        spark.sql("DROP TABLE IF EXISTS T");
+        spark.sql(
+                "CREATE TABLE T (id INT, "
+                        + "name STRING DEFAULT 'default_name', "
+                        + "tags ARRAY<STRING> DEFAULT ARRAY('default_tag'), "
+                        + "metadata MAP<STRING, STRING> DEFAULT MAP('created_by', 'system'), "
+                        + "config STRUCT<enabled: BOOLEAN, timeout: INT> DEFAULT STRUCT(true, 30)) TBLPROPERTIES"
+                        + " ('file.format'='avro')");
+
+        // test partial write with mixed defaults
+        spark.sql("INSERT INTO T (id) VALUES (1)").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString()).contains("default_name");
+        assertThat(rows.toString())
+                .satisfiesAnyOf(
+                        s -> assertThat(s).contains("WrappedArray('default_tag')"),
+                        s -> assertThat(s).contains("ArraySeq('default_tag')"));
+        assertThat(rows.toString()).contains("Map('created_by' -> 'system')");
+        assertThat(rows.toString()).contains("[true,30]");
+
+        // test selective column insertion with mixed defaults
+        spark.sql("INSERT INTO T (id, name) VALUES (2, 'custom_name')").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString()).contains("custom_name");
+        assertThat(rows.toString())
+                .satisfiesAnyOf(
+                        s -> assertThat(s).contains("WrappedArray('default_tag')"),
+                        s -> assertThat(s).contains("ArraySeq('default_tag')"));
+        assertThat(rows.size()).isEqualTo(2);
+
+        // test write with some DEFAULT keywords for mixed types
+        spark.sql("INSERT INTO T VALUES (3, DEFAULT, ARRAY('custom_tag'), DEFAULT, DEFAULT)")
+                .collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString()).contains("default_name");
+        assertThat(rows.toString())
+                .satisfiesAnyOf(
+                        s -> assertThat(s).contains("WrappedArray(custom_tag)"),
+                        s -> assertThat(s).contains("ArraySeq(custom_tag)"));
+        assertThat(rows.size()).isEqualTo(3);
+    }
+
+    @ParameterizedTest
+    @ValueSource(strings = {"order", "zorder", "hilbert"})
+    public void testWriteWithClustering(String clusterStrategy) {
+        spark.sql(
+                "CREATE TABLE T (a INT, b INT) TBLPROPERTIES ("
+                        + "'clustering.columns'='a,b',"
+                        + String.format("'clustering.strategy'='%s')", clusterStrategy));
+        spark.sql("INSERT INTO T VALUES (2, 2), (1, 1), (3, 3)").collectAsList();
+        List<Row> rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString()).isEqualTo("[[1,1], [2,2], [3,3]]");
+    }
+
     @Test
     public void testWrite() {
         spark.sql(
@@ -295,7 +521,7 @@ public class SparkWriteITCase {
                 rows.stream().map(x -> x.getString(0)).collect(Collectors.toList());
         Assertions.assertEquals(3, fileNames.size());
         for (String fileName : fileNames) {
-            Assertions.assertTrue(fileName.startsWith("data-"));
+            Assertions.assertTrue(fileName.contains("data-"));
         }
     }
 
@@ -316,7 +542,7 @@ public class SparkWriteITCase {
                 rows.stream().map(x -> x.getString(0)).collect(Collectors.toList());
         Assertions.assertEquals(3, fileNames.size());
         for (String fileName : fileNames) {
-            Assertions.assertTrue(fileName.startsWith("test-"));
+            Assertions.assertTrue(fileName.contains("test-"));
         }
     }
 
@@ -337,7 +563,7 @@ public class SparkWriteITCase {
                 rows.stream().map(x -> x.getString(0)).collect(Collectors.toList());
         Assertions.assertEquals(3, fileNames.size());
         for (String fileName : fileNames) {
-            Assertions.assertTrue(fileName.startsWith("test-"));
+            Assertions.assertTrue(fileName.contains("test-"));
         }
 
         // reset config, it will affect other tests
@@ -412,8 +638,8 @@ public class SparkWriteITCase {
         Assertions.assertEquals(4, files.size());
 
         String defaultExtension = "." + "parquet";
-        String newExtension = "." + "zstd" + "." + "parquet";
-        // two data files end with ".parquet", two data file end with ".zstd.parquet"
+        String newExtension = "." + "zst" + "." + "parquet";
+        // two data files end with ".parquet", two data file end with ".zst.parquet"
         Assertions.assertEquals(
                 2,
                 files.stream()
@@ -453,11 +679,11 @@ public class SparkWriteITCase {
         List<String> files =
                 Arrays.stream(fileIO.listStatus(new Path(tabLocation, "bucket-0")))
                         .map(name -> name.getPath().getName())
-                        .filter(name -> name.startsWith("changelog-"))
+                        .filter(name -> name.contains("changelog-"))
                         .collect(Collectors.toList());
         String defaultExtension = "." + "parquet";
-        String newExtension = "." + "zstd" + "." + "parquet";
-        // one changelog file end with ".parquet", one changelog file end with ".zstd.parquet"
+        String newExtension = "." + "zst" + "." + "parquet";
+        // one changelog file end with ".parquet", one changelog file end with ".zst.parquet"
         Assertions.assertEquals(
                 1,
                 files.stream()
@@ -473,6 +699,42 @@ public class SparkWriteITCase {
         spark.conf().unset("spark.paimon.file.suffix.include.compression");
     }
 
+    @Test
+    public void testIgnoreEmptyCommitConfigurable() {
+        spark.sql(
+                "CREATE TABLE T (id INT, name STRING) "
+                        + "TBLPROPERTIES ("
+                        + "'bucket-key'='id', "
+                        + "'bucket' = '1', "
+                        + "'file.format' = 'avro')");
+
+        FileStoreTable table = getTable("T");
+        SnapshotManager snapshotManager = table.snapshotManager();
+
+        spark.sql("insert into T values (1, 'aa')");
+        Assertions.assertEquals(1, snapshotManager.latestSnapshotId());
+
+        spark.sql("delete from T where id = 1");
+        Assertions.assertEquals(2, snapshotManager.latestSnapshotId());
+        Assertions.assertEquals(
+                -1, Objects.requireNonNull(snapshotManager.latestSnapshot()).deltaRecordCount());
+
+        // in batch write, ignore.empty.commit default is true
+        spark.sql("delete from T where id = 1");
+        Assertions.assertEquals(2, snapshotManager.latestSnapshotId());
+        Assertions.assertEquals(
+                -1, Objects.requireNonNull(snapshotManager.latestSnapshot()).deltaRecordCount());
+
+        // set false to allow commit empty snapshot
+        spark.conf().set("spark.paimon.snapshot.ignore-empty-commit", "false");
+        spark.sql("delete from T where id = 1");
+        Assertions.assertEquals(3, snapshotManager.latestSnapshotId());
+        Assertions.assertEquals(
+                0, Objects.requireNonNull(snapshotManager.latestSnapshot()).deltaRecordCount());
+
+        spark.conf().unset("spark.paimon.snapshot.ignore-empty-commit");
+    }
+
     protected static FileStoreTable getTable(String tableName) {
         return FileStoreTableFactory.create(
                 LocalFileIO.create(),
@@ -480,8 +742,6 @@ public class SparkWriteITCase {
     }
 
     private long dataFileCount(FileStatus[] files, String filePrefix) {
-        return Arrays.stream(files)
-                .filter(f -> f.getPath().getName().startsWith(filePrefix))
-                .count();
+        return Arrays.stream(files).filter(f -> f.getPath().getName().contains(filePrefix)).count();
     }
 }
