@@ -19,11 +19,12 @@
 package org.apache.paimon.spark
 
 import org.apache.paimon.{CoreOptions, Snapshot}
-import org.apache.paimon.manifest.{IndexManifestEntry, ManifestCommittable, ManifestEntry}
+import org.apache.paimon.manifest.{IndexManifestEntry, ManifestCommittable, ManifestEntry, SimpleFileEntry}
 import org.apache.paimon.table.sink.CommitCallback
 
 import org.junit.jupiter.api.Assertions
 
+import java.util
 import java.util.List
 
 class PaimonCommitTest extends PaimonSparkTestBase {
@@ -64,7 +65,8 @@ object PaimonCommitTest {
 case class CustomCommitCallback(testId: String) extends CommitCallback {
 
   override def call(
-      committedEntries: List[ManifestEntry],
+      baseFiles: List[SimpleFileEntry],
+      deltaFiles: List[ManifestEntry],
       indexFiles: List[IndexManifestEntry],
       snapshot: Snapshot): Unit = {
     PaimonCommitTest.id = testId
