@@ -18,7 +18,6 @@
 
 package org.apache.paimon.spark.procedure
 
-import org.apache.paimon.data.Timestamp
 import org.apache.paimon.fs.Path
 import org.apache.paimon.spark.PaimonSparkTestBase
 import org.apache.paimon.utils.DateTimeUtils
@@ -62,10 +61,6 @@ class RemoveOrphanFilesProcedureTest extends PaimonSparkTestBase {
           TimeUnit.SECONDS.toMillis(1)),
       3)
 
-    System.out.println("orphanFile2ModTime is : " + orphanFile2ModTime);
-    System.out.println("older_than1 is : " + older_than1)
-    System.out.println("in ut Timestamp.now() is : " + Timestamp.now)
-
     checkAnswer(
       spark.sql(s"CALL sys.remove_orphan_files(table => 'T', older_than => '$older_than1')"),
       Row(1, 1) :: Nil)
@@ -209,7 +204,7 @@ class RemoveOrphanFilesProcedureTest extends PaimonSparkTestBase {
 
     checkAnswer(
       spark.sql(
-        s"CALL sys.remove_orphan_files(table => 'T', older_than => '$older_than1', mode => 'diSTributed')"),
+        s"CALL sys.remove_orphan_files(table => 'T', older_than => '$older_than1', mode => 'distributed')"),
       Row(1, 1) :: Nil)
 
     val older_than2 = DateTimeUtils.formatLocalDateTime(
@@ -245,4 +240,75 @@ class RemoveOrphanFilesProcedureTest extends PaimonSparkTestBase {
       sql(s"CALL sys.remove_orphan_files(table => 'T', older_than => '$older_than')"),
       Row(1, 1) :: Nil)
   }
+
+  test("Paimon procedure: clean empty directory after removing orphan files") {
+    spark.sql("""
+                |CREATE TABLE T (k STRING, pt STRING)
+                |using paimon TBLPROPERTIES ('primary-key'='k,pt', 'bucket'='1',
+                |'snapshot.clean-empty-directories'='false') PARTITIONED BY (pt);
+                |""".stripMargin)
+
+    spark.sql("""
+                |insert into T values
+                |("a", "2024-06-02"),("b", "2024-06-02"),("d", "2024-06-03"),
+                |("c", "2024-06-01"),("Never-expire", "9999-09-09");
+                |
+                |""".stripMargin)
+
+    // by default, no file deleted
+    checkAnswer(spark.sql(s"CALL sys.remove_orphan_files(table => 'T')"), Row(0, 0) :: Nil)
+
+    spark.sql(
+      "CALL sys.expire_partitions(table => 'T' , " +
+        "expiration_time => '1 d', timestamp_formatter => 'yyyy-MM-dd', max_expires => 3);")
+
+    // insert a new snapshot to clean expired partitioned files
+    spark.sql("insert into T values ('Never-expire-2', '9999-09-09')")
+
+    val table = loadTable("T")
+    val fileIO = table.fileIO()
+    val tablePath = table.location()
+
+    val partitionValue = "pt=2024-06-01"
+    val partitionPath = tablePath + "/" + partitionValue
+    val orphanFile1 = new Path(partitionPath, ORPHAN_FILE_1)
+    val orphanFile2 = new Path(partitionPath, ORPHAN_FILE_2)
+    fileIO.writeFile(orphanFile1, "a", true)
+    Thread.sleep(2000)
+    fileIO.writeFile(orphanFile2, "b", true)
+
+    checkAnswer(
+      spark.sql("CALL paimon.sys.expire_snapshots(table => 'T', retain_max => 1, retain_min => 1)"),
+      Row(2) :: Nil)
+
+    val older_than1 = new java.sql.Timestamp(
+      fileIO.getFileStatus(orphanFile2).getModificationTime - TimeUnit.SECONDS.toMillis(1))
+
+    // partition 'pt=2024-06-01' has one orphan file left
+    assertResult(true)(
+      fileIO
+        .listDirectories(tablePath)
+        .map(status => status.getPath.getName)
+        .contains(partitionValue))
+
+    checkAnswer(
+      spark.sql(
+        s"CALL sys.remove_orphan_files(table => 'T', older_than => '$older_than1', mode => 'distributed')"),
+      Row(1, 1) :: Nil)
+
+    val older_than2 = new java.sql.Timestamp(System.currentTimeMillis())
+
+    checkAnswer(
+      spark.sql(
+        s"CALL sys.remove_orphan_files(table => 'T', older_than => '$older_than2', mode => 'local')"),
+      Row(1, 1) :: Nil)
+
+    // partition 'pt=2024-06-01' has no orphan files, clean empty directory
+    assertResult(false)(
+      fileIO
+        .listDirectories(tablePath)
+        .map(status => status.getPath.getName)
+        .contains(partitionValue))
+  }
+
 }
