@@ -731,6 +731,89 @@ abstract class DDLWithHiveCatalogTestBase extends PaimonHiveTestBase {
     }
   }
 
+  test("Paimon DDL with hive catalog: default value") {
+    // Spark support default value since 3.4
+    if (gteqSpark3_4) {
+      Seq(sparkCatalogName, paimonHiveCatalogName).foreach {
+        catalogName =>
+          spark.sql(s"USE $catalogName")
+          val databaseName = "paimon_db"
+          withDatabase(databaseName) {
+            withTable("t1", "t2") {
+              // test paimon table
+              spark.sql("CREATE TABLE t1 (a INT, b int default 3) using paimon")
+              spark.sql("INSERT INTO t1 (a) VALUES (1)")
+              checkAnswer(spark.sql("SELECT * FROM t1"), Seq(Row(1, 3)))
+
+              // test non paimon table
+              // todo: support default value for paimon format table
+              if (catalogName != paimonHiveCatalogName) {
+                spark.sql("CREATE TABLE t2 (a INT, b int default 3) using csv")
+                spark.sql("INSERT INTO t2 (a) VALUES (1)")
+                checkAnswer(spark.sql("SELECT * FROM t2"), Seq(Row(1, 3)))
+              }
+            }
+          }
+      }
+    }
+  }
+
+  test("Paimon Dataframe: read write with catalog name") {
+    withTempDir {
+      tmpDir =>
+        spark.sql(s"USE $paimonHiveCatalogName")
+        val dbName = "paimon_db"
+        withDatabase(dbName) {
+          spark.sql(s"CREATE DATABASE $dbName")
+          spark.sql(s"USE $dbName")
+          withTable("t") {
+            val tblLocation = tmpDir.getCanonicalPath
+            spark.sql(s"CREATE TABLE t (a INT, b STRING) USING paimon LOCATION '$tblLocation'")
+
+            // write by path
+            Seq((1, "x1"), (2, "x2"))
+              .toDF("a", "b")
+              .write
+              .format("paimon")
+              .option("catalog", paimonHiveCatalogName)
+              .option("database", dbName)
+              .option("table", "t")
+              .mode("append")
+              .save(tblLocation)
+
+            // write by name
+            Seq((3, "x3"), (2, "x4"))
+              .toDF("a", "b")
+              .write
+              .format("paimon")
+              .mode("append")
+              .saveAsTable(s"$paimonHiveCatalogName.$dbName.t")
+
+            // read by path with time travel
+            checkAnswer(
+              spark.read
+                .format("paimon")
+                .option("catalog", paimonHiveCatalogName)
+                .option("database", dbName)
+                .option("table", "t")
+                .option("scan.snapshot-id", 1)
+                .load(tblLocation),
+              Seq(Row(1, "x1"), Row(2, "x2"))
+            )
+
+            // read by name with time travel
+            checkAnswer(
+              spark.read
+                .format("paimon")
+                .option("scan.snapshot-id", 1)
+                .table(s"$paimonHiveCatalogName.$dbName.t"),
+              Seq(Row(1, "x1"), Row(2, "x2"))
+            )
+          }
+        }
+    }
+  }
+
   def getDatabaseProp(dbName: String, propertyName: String): String = {
     spark
       .sql(s"DESC DATABASE EXTENDED $dbName")
