@@ -23,13 +23,11 @@ import org.apache.paimon.spark.PaimonSparkTestBase
 import org.apache.paimon.spark.catalyst.analysis.expressions.ExpressionHelper
 import org.apache.paimon.spark.catalyst.optimizer.MergePaimonScalarSubqueries
 
-import org.apache.spark.sql.Row
+import org.apache.spark.sql.{PaimonUtils, Row}
 import org.apache.spark.sql.catalyst.expressions.{Attribute, CreateNamedStruct, Literal, NamedExpression}
 import org.apache.spark.sql.catalyst.plans.logical.{CTERelationDef, LogicalPlan, OneRowRelation, WithCTE}
 import org.apache.spark.sql.catalyst.rules.RuleExecutor
 import org.apache.spark.sql.functions._
-import org.apache.spark.sql.paimon.Utils
-import org.apache.spark.sql.paimon.shims.SparkShimLoader
 import org.junit.jupiter.api.Assertions
 
 import scala.collection.immutable
@@ -61,8 +59,9 @@ abstract class PaimonOptimizationTestBase extends PaimonSparkTestBase with Expre
                                |  (SELECT AVG(b) AS avg_b FROM T)
                                |""".stripMargin)
       val optimizedPlan = Optimize.execute(query.queryExecution.analyzed)
+      val id = optimizedPlan.asInstanceOf[WithCTE].cteDefs.head.id.toInt
 
-      val df = Utils.createDataFrame(spark, createRelationV2("T"))
+      val df = PaimonUtils.createDataset(spark, createRelationV2("T"))
       val mergedSubquery = df
         .select(
           toColumn(count(Literal(1))).as("cnt"),
@@ -84,11 +83,11 @@ abstract class PaimonOptimizationTestBase extends PaimonSparkTestBase with Expre
       val correctAnswer = WithCTE(
         OneRowRelation()
           .select(
-            extractorExpression(0, analyzedMergedSubquery.output, 0),
-            extractorExpression(0, analyzedMergedSubquery.output, 1),
-            extractorExpression(0, analyzedMergedSubquery.output, 2)
+            extractorExpression(id, analyzedMergedSubquery.output, 0),
+            extractorExpression(id, analyzedMergedSubquery.output, 1),
+            extractorExpression(id, analyzedMergedSubquery.output, 2)
           ),
-        Seq(definitionNode(analyzedMergedSubquery, 0))
+        Seq(definitionNode(analyzedMergedSubquery, id))
       )
       // Check the plan applied MergePaimonScalarSubqueries.
       comparePlans(optimizedPlan.analyze, correctAnswer.analyze)
